#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 Class: ColorArrow

 Desc: A colorful note that floats up from the bottom of the screen.

 Copyright (c) 2001-2002 by the names listed below.  All rights reserved.
	Brian Bugh
	Ben Nordstrom
	Chris Danford
-----------------------------------------------------------------------------
*/


#include "ColorArrow.h"
#include "Steps.h"
#include "ThemeManager.h"
#include "GameConstants.h"


const int NUM_FRAMES_IN_GRAY_PART = 4;



ColorArrow::ColorArrow()
{
	// load textures
	m_sprGrayPart.Load( THEME->GetPathTo(GRAPHIC_COLOR_ARROW_GRAY_PART) );	
	m_sprColorPart.Load( THEME->GetPathTo(GRAPHIC_COLOR_ARROW_COLOR_PART) );
	m_sprColorPart.TurnShadowOff();

	m_sprGrayPart.StopAnimating();
	m_sprGrayPart.StopAnimating();
	m_sprGrayPart.TurnShadowOff();
}





const D3DXCOLOR g_TweenColors[] = { 
           D3DXCOLOR(0.50f,1.00f,0.00f,1), 
           D3DXCOLOR(0.25f,1.00f,0.25f,1), 
           D3DXCOLOR(0.00f,1.00f,0.50f,1), 
           D3DXCOLOR(0.00f,1.00f,0.75f,1), 
           D3DXCOLOR(0.00f,1.00f,1.00f,1),  // cyan
           D3DXCOLOR(0.00f,0.75f,1.00f,1),
           D3DXCOLOR(0.00f,0.50f,1.00f,1), 
           D3DXCOLOR(0.25f,0.25f,1.00f,1), 
           D3DXCOLOR(0.50f,0.00f,1.00f,1), 
           D3DXCOLOR(0.75f,0.00f,1.00f,1), 
           D3DXCOLOR(1.00f,0.00f,1.00f,1),   // purple
           D3DXCOLOR(1.00f,0.00f,0.75f,1), 
           D3DXCOLOR(1.00f,0.00f,0.50f,1), 
           D3DXCOLOR(1.00f,0.25f,0.25f,1), 
           D3DXCOLOR(1.00f,0.50f,0.00f,1), 
           D3DXCOLOR(0.75f,0.75f,0.00f,1), 

};
const int NUM_TWEEN_COLORS = sizeof(g_TweenColors)/sizeof(D3DXCOLOR);



void ColorArrow::SetGrayPartFromIndexAndBeat( const int iStepIndex, const float fSongBeat )
{
	// calculate gray part animation frame
	int iFrameNo = roundf(fSongBeat * NUM_FRAMES_IN_GRAY_PART) % NUM_FRAMES_IN_GRAY_PART;
	if( iFrameNo < 0 )
		iFrameNo += NUM_FRAMES_IN_GRAY_PART;
	m_sprGrayPart.SetState( iFrameNo );
	m_sprGrayPart.SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
}

void ColorArrow::SetColorPartFromIndexAndBeat( const int iStepIndex, const float fSongBeat, const PlayerOptions::ColorType ct )
{
	// calculate color
	float fPercentThroughMeasure = fmodf(fSongBeat, (float)BEATS_PER_MEASURE) / BEATS_PER_MEASURE;
	int iBaseColorIndex = roundf(fPercentThroughMeasure * NUM_TWEEN_COLORS);


	////////////////////////////////////////////////////////////
	// ADDED 2-14-2002 - BrianB 
	//
	// This section has been added for the different arrow
	// display types:
	//
	// Note: Each note has a different color, but does not tween
	// Flat: Each note has the same color, and does tween
	// Plain: Each note has the same color, but does not tween
	//
	////////////////////////////////////////////////////////////

	switch( ct )
	{
	case PlayerOptions::COLOR_ARCADE:
		{
		float fPercentThroughBeat = (iStepIndex % ELEMENTS_PER_BEAT)/(float)ELEMENTS_PER_BEAT;
		int iOffsetColorIndex = roundf( fPercentThroughBeat * NUM_TWEEN_COLORS );
		int iColorIndex = iBaseColorIndex + iOffsetColorIndex;
		if( iColorIndex >= NUM_TWEEN_COLORS )
			iColorIndex = iColorIndex % NUM_TWEEN_COLORS;

		// use different colors for the left and right edges so that the arrows look more colorful
		m_sprColorPart.SetDiffuseColorLeftEdge( g_TweenColors[iColorIndex] );
		
		int iRightEdgeColorIndex = iColorIndex + NUM_TWEEN_COLORS/4;
		if( iRightEdgeColorIndex >= NUM_TWEEN_COLORS )
			iRightEdgeColorIndex = iRightEdgeColorIndex % NUM_TWEEN_COLORS;
		m_sprColorPart.SetDiffuseColorRightEdge( g_TweenColors[iRightEdgeColorIndex] );
		}
		break;
	case PlayerOptions::COLOR_NOTE:
		{
		D3DXCOLOR color = GetNoteColorFromIndex( iStepIndex );
		// add a little bit of white so the note doesn't look so plain
		color.r += fPercentThroughMeasure * 0.3f;
		color.g += fPercentThroughMeasure * 0.3f;
		color.b += fPercentThroughMeasure * 0.3f;
		m_sprColorPart.SetDiffuseColor( color );	// sets both left and right edge
		}
		break;
	case PlayerOptions::COLOR_FLAT:
		{
		// use different colors for the left and right edges so that the arrows look more colorful
		m_sprColorPart.SetDiffuseColorLeftEdge( g_TweenColors[iBaseColorIndex] );
		
		int iRightEdgeColorIndex = iBaseColorIndex + NUM_TWEEN_COLORS/4;
		if( iRightEdgeColorIndex >= NUM_TWEEN_COLORS )
			iRightEdgeColorIndex = iRightEdgeColorIndex % NUM_TWEEN_COLORS;
		m_sprColorPart.SetDiffuseColorRightEdge( g_TweenColors[iRightEdgeColorIndex] );
		}
		break;
	case PlayerOptions::COLOR_PLAIN:
		{
		D3DXCOLOR color = NOTE_TYPE_TO_COLOR[NOTE_4TH];
		// add a little bit of white so the note doesn't look so plain
		color += D3DXCOLOR(0.1f,0.1f,0.1f,0) * fPercentThroughMeasure;
		m_sprColorPart.SetDiffuseColor( color );	// sets both left and right edge
		}
		break;
	default:
		ASSERT( false );	// unexpected ColorType
	}

}



void ColorArrow::SetDiffuseColor( D3DXCOLOR color )
{
	m_sprColorPart.SetDiffuseColor( color );
}

void ColorArrow::SetAlpha( float fAlpha )
{
	float fDistFromCenter = fabsf( 0.5f - fAlpha );
	float fAddAlpha = 1-(fDistFromCenter*2);
	m_sprColorPart.SetDiffuseColor( m_sprColorPart.GetDiffuseColor()*fAlpha );
	m_sprColorPart.SetAddColor( D3DXCOLOR(1,1,1,1)*fAddAlpha );
	m_sprGrayPart.SetDiffuseColor( m_sprGrayPart.GetDiffuseColor()*fAlpha );
	m_sprGrayPart.SetAddColor( D3DXCOLOR(1,1,1,1)*fAddAlpha );
}

void ColorArrow::SetGrayPartClear()
{
	m_sprGrayPart.SetState( 0 );
}

void ColorArrow::SetGrayPartFull()
{
	m_sprGrayPart.SetState( 1 );
}
