/*
-----------------------------------------------------------------------------
 Class: ColorArrow

 Desc: A colorful note that floats up from the bottom of the screen.

 Copyright (c) 2001-2002 by the names listed below.  All rights reserved.
	Brian Bugh
	Ben Nordstrom
	Chris Danford
-----------------------------------------------------------------------------
*/


#ifndef _COLORARROW_H_
#define _COLORARROW_H_


#include "Sprite.h"
#include "GameTypes.h"


const int	ARROW_SIZE	=	 64;
const float ARROW_GAP						= ARROW_SIZE + 6;
const int	NUM_FRAMES_IN_COLOR_ARROW_SPRITE	= 12;


class ColorArrow  
{
public:
	ColorArrow();

	void  SetRotation( float rot ) { m_sprGrayPart.SetRotation( rot ); m_sprColorPart.SetRotation( rot ); };
	void  Draw() { m_sprGrayPart.Draw(); m_sprColorPart.Draw(); };
	void  DrawGrayPart() { m_sprGrayPart.Draw(); };
	void  DrawColorPart() { m_sprColorPart.Draw(); };
	float GetX() { return m_sprGrayPart.GetX( ); };
	float GetY() { return m_sprGrayPart.GetY( ); };
	void  SetX( float x ) { m_sprGrayPart.SetX( x ); m_sprColorPart.SetX( x ); };
	void  SetY( float y ) { m_sprGrayPart.SetY( y ); m_sprColorPart.SetY( y ); };
	void  SetXY( float x, float y ) { SetX( x ); SetY( y ); };
	void  Update( float fDeltaTime ) { m_sprGrayPart.Update( fDeltaTime ); m_sprColorPart.Update( fDeltaTime ); };
	void  SetZoom( float fZoom ) { m_sprGrayPart.SetZoom( fZoom ); m_sprColorPart.SetZoom( fZoom ); };

	void BeginTweening( float time ) { m_sprGrayPart.BeginTweening( time ); m_sprColorPart.BeginTweening( time ); };
	void SetTweenX( float x ) { m_sprGrayPart.SetTweenX( x ); m_sprColorPart.SetTweenX( x ); };
	void SetTweenY( float y ) { m_sprGrayPart.SetTweenY( y ); m_sprColorPart.SetTweenY( y ); };
	virtual void SetTweenXY( float x, float y ) { SetTweenX( x ); SetTweenY( y ); };

	void  SetGrayPartFromIndexAndBeat( const int iIndex, const float fSongBeat );
	void  SetColorPartFromIndexAndBeat( const int iIndex, const float fSongBeat, const PlayerOptions::ColorType ct );
	/////////////////////////
	// ADDED 2-14-02 - BrianB
	void  SetNoteColorPartFromIndexAndBeat( const int iIndex, const float fSongBeat );
	void  SetFlatColorPartFromIndexAndBeat( const int iIndex, const float fSongBeat );
	void  SetPlainColorPartFromIndexAndBeat( const int iIndex, const float fSongBeat );
	// END ADDED 2-14-02 - BrianB
	/////////////////////////
	void  SetDiffuseColor( D3DXCOLOR color );
	void  SetAlpha( float fAlpha );
	void  SetGrayPartClear();
	void  SetGrayPartFull();

protected:
	Sprite		m_sprGrayPart;		// this is usually solid black or gray
	Sprite		m_sprColorPart;		// this part changes colors

};

#endif 
