#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: ColorArrowField.cpp

 Desc: A pattern of ColorArrows that past Y==0.

 Copyright (c) 2001-2002 by the names listed below.  All rights reserved.
	Chris Danford
-----------------------------------------------------------------------------
*/

#include "ColorArrowField.h"
#include "RageUtil.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "ArrowEffects.h"


ColorArrowField::ColorArrowField()
{
	for( int i=0; i<MAX_TAP_STEP_ELEMENTS; i++ )
	{
		m_TapSteps[i] = STEP_NONE;
	}
	
	m_iNumHoldSteps = 0;

	m_textMeasureNumber.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textMeasureNumber.SetZoom( 0.5f );
}


void ColorArrowField::Load( Style style, Steps steps, PlayerOptions po, float fNumBeatsToDrawBehind, float fNumBeatsToDrawAhead )
{
	m_Style = style;
	m_PlayerOptions = po;
	m_fNumBeatsToDrawBehind = fNumBeatsToDrawBehind;
	m_fNumBeatsToDrawAhead = fNumBeatsToDrawAhead;

	// init arrow rotations and X positions
	for( int c=0; c<m_Style.m_iNumColumns; c++ ) 
	{
		m_ColorArrow[c].SetRotation( m_Style.m_ColumnToRotation[c] );
		float fColOffsetFromCenter = c - (m_Style.m_iNumColumns-1)/2.0f;
		m_ColorArrow[c].SetX( fColOffsetFromCenter*ARROW_SIZE );
	}

	// load TapSteps
	for( int i=0; i<MAX_TAP_STEP_ELEMENTS; i++ )
	{
		m_TapSteps[i] = steps.m_TapSteps[i];
	}

	// load HoldSteps
	for( i=0; i<MAX_HOLD_STEP_ELEMENTS; i++ )
	{
		m_HoldSteps[i] = steps.m_HoldSteps[i];
		m_HoldStepLife[i] = 1;
	}

	m_iNumHoldSteps = steps.m_iNumHoldSteps;
}

void ColorArrowField::Update( float fDeltaTime, float fSongBeat )
{
	m_fSongBeat = fSongBeat;
}




void ColorArrowField::DrawTapStep( int iCol, float fIndex )
{
	float fYOffset	= ArrowGetYOffset(	m_PlayerOptions, fIndex, m_fSongBeat );
	float fYPos		= ArrowGetYPos(		m_PlayerOptions, fYOffset );
	float fRotation = ArrowGetRotation(	m_PlayerOptions, m_Style, iCol, fYOffset );
	float fXPos		= ArrowGetXPos(		m_PlayerOptions, m_Style, iCol, fYOffset, m_fSongBeat );
	float fAlpha	= ArrowGetAlpha(	m_PlayerOptions, fYPos );

	m_ColorArrow[iCol].SetXY( fXPos, fYPos );
	m_ColorArrow[iCol].SetRotation( fRotation );
	m_ColorArrow[iCol].SetColorPartFromIndexAndBeat( (int)fIndex, m_fSongBeat, m_PlayerOptions.m_ColorType );
	m_ColorArrow[iCol].SetGrayPartFromIndexAndBeat( (int)fIndex, m_fSongBeat );
	m_ColorArrow[iCol].SetAlpha( fAlpha );
	m_ColorArrow[iCol].Draw();
}

void ColorArrowField::DrawTapStep( int iCol, float fIndex, D3DXCOLOR color )
{
	float fYOffset	= ArrowGetYOffset(	m_PlayerOptions, fIndex, m_fSongBeat );
	float fYPos		= ArrowGetYPos(		m_PlayerOptions, fYOffset );
	float fRotation = ArrowGetRotation(	m_PlayerOptions, m_Style, iCol, fYOffset );
	float fXPos		= ArrowGetXPos(		m_PlayerOptions, m_Style, iCol, fYOffset, m_fSongBeat );
	float fAlpha	= ArrowGetAlpha(	m_PlayerOptions, fYPos );

	m_ColorArrow[iCol].SetXY( fXPos, fYPos );
	m_ColorArrow[iCol].SetRotation( fRotation );
	m_ColorArrow[iCol].SetDiffuseColor( color );
	m_ColorArrow[iCol].SetGrayPartFromIndexAndBeat( (int)fIndex, m_fSongBeat );
	m_ColorArrow[iCol].SetAlpha( fAlpha );
	m_ColorArrow[iCol].Draw();
}

void ColorArrowField::DrawHoldStepColorPart( int iCol, float fIndex, HoldStep &hs, float fHoldStepLife )
{
	float fYOffset	= ArrowGetYOffset(	m_PlayerOptions, fIndex, m_fSongBeat );
	if( fHoldStepLife > 0  &&  fYOffset < 0 )
		return;		// don't draw
	float fYPos		= ArrowGetYPos(		m_PlayerOptions, fYOffset );
	float fRotation = ArrowGetRotation(	m_PlayerOptions, m_Style, iCol, fYOffset );
	float fXPos		= ArrowGetXPos(		m_PlayerOptions, m_Style, iCol, fYOffset, m_fSongBeat );
	float fAlpha	= ArrowGetAlpha(	m_PlayerOptions, fYPos );

	D3DXCOLOR color( (fIndex-hs.m_iStartIndex)/(hs.m_iEndIndex-hs.m_iStartIndex), 1, 0, 1 ); // color shifts from green to yellow
	color *= 1 - (1-fHoldStepLife) / 1.2f;
	color.a = 1;

	m_ColorArrow[iCol].SetXY( fXPos, fYPos );
	m_ColorArrow[iCol].SetRotation( fRotation );
	m_ColorArrow[iCol].SetDiffuseColor( color );	
	m_ColorArrow[iCol].SetAlpha( fAlpha );
	m_ColorArrow[iCol].DrawColorPart();
}

void ColorArrowField::DrawHoldStepGrayPart( int iCol, float fIndex, HoldStep &hs, float fHoldStepLife )
{
	float fYOffset	= ArrowGetYOffset(	m_PlayerOptions, fIndex, m_fSongBeat );
	if( fHoldStepLife > 0  &&  fYOffset < 0 )
		return;		// don't draw
	float fYPos		= ArrowGetYPos(		m_PlayerOptions, fYOffset );
	float fRotation = ArrowGetRotation(	m_PlayerOptions, m_Style, iCol, fYOffset );
	float fXPos		= ArrowGetXPos(		m_PlayerOptions, m_Style, iCol, fYOffset, m_fSongBeat );
	float fAlpha	= ArrowGetAlpha(	m_PlayerOptions, fYPos );

	m_ColorArrow[iCol].SetXY( fXPos, fYPos );
	m_ColorArrow[iCol].SetRotation( fRotation );
	m_ColorArrow[iCol].SetAlpha( fAlpha );
	m_ColorArrow[iCol].DrawGrayPart();
}

void ColorArrowField::DrawMeasureBar( int iIndex, int iMeasureNo )
{
	float fYOffset	= ArrowGetYOffset(	m_PlayerOptions, (float)iIndex, m_fSongBeat );
	float fYPos		= ArrowGetYPos(		m_PlayerOptions, fYOffset );

	m_rectMeasureBar.SetXY( 0, fYPos );
	m_rectMeasureBar.SetWidth( (float)(m_Style.m_iNumColumns+1) * ARROW_SIZE );
	m_rectMeasureBar.SetHeight( 20 );
	m_rectMeasureBar.SetDiffuseColor( D3DXCOLOR(0,0,0,0.5f) );
	m_rectMeasureBar.Draw();

	m_textMeasureNumber.SetText( ssprintf("%d", iMeasureNo) );
	m_textMeasureNumber.SetXY( -m_rectMeasureBar.GetZoomedWidth()/2 + 10, fYPos );
	m_textMeasureNumber.Draw();
}


void ColorArrowField::RenderPrimitives()
{
	//RageLog( "ColorArrowField::RenderPrimitives()" );

	if( m_fSongBeat < 0 )
		m_fSongBeat = 0;

	int iBaseFrameNo = (int)(m_fSongBeat*2.5) % NUM_FRAMES_IN_COLOR_ARROW_SPRITE;	// 2.5 is a "fudge number" :-)  This should be based on BPM
	
	int iIndexFirstArrowToDraw = BeatToStepIndex( m_fSongBeat - m_fNumBeatsToDrawBehind );	// 2 beats earlier
	if( iIndexFirstArrowToDraw < 0 ) iIndexFirstArrowToDraw = 0;
	int iIndexLastArrowToDraw  = BeatToStepIndex( m_fSongBeat + m_fNumBeatsToDrawAhead );	// 7 beats later

	//RageLog( "Drawing elements %d through %d", iIndexFirstArrowToDraw, iIndexLastArrowToDraw );


	//
	// Draw measure bars
	//
	if( m_PlayerOptions.m_bShowMeasureBars )
	{
		for( int i=iIndexFirstArrowToDraw; i<=iIndexLastArrowToDraw; i++ )
		{
			if( i % ELEMENTS_PER_MEASURE == 0 )
				DrawMeasureBar( i, i / ELEMENTS_PER_MEASURE );
		}
	}

	//
	// Draw all TapSteps
	//
	for( int i=iIndexFirstArrowToDraw; i<=iIndexLastArrowToDraw; i++ )	//	 for each row
	{				
		if( m_TapSteps[i] != 0 )
		{
			//RageLog( "iYPos: %d, iFrameNo: %d, m_OriginalStep[i]: %d", iYPos, iFrameNo, m_OriginalStep[i] );

			// See if there is a hold step that begins on this beat.  Terribly inefficient!
			bool bHoldStepOnThisBeat = false;
			for( int j=0; j<m_iNumHoldSteps; j++ )
			{
				if( m_HoldSteps[j].m_iStartIndex == i )
				{
					bHoldStepOnThisBeat = true;
					break;
				}
			}

			for( int c=0; c<m_Style.m_iNumColumns; c++ )	// for each arrow column
			{
				if( m_TapSteps[i] & m_Style.m_ColumnToTapStep[c] ) 	// this column is still unstepped on?
				{
					if( bHoldStepOnThisBeat )
						DrawTapStep( c, (float)i, D3DXCOLOR(0,1,0,1) );
					else
						DrawTapStep( c, (float)i );
				}
			}


		}	// end if there is a step
	}	// end foreach arrow to draw

	//
	// Draw all HoldSteps
	//
	for( i=0; i<m_iNumHoldSteps; i++ )
	{
		HoldStep &hs = m_HoldSteps[i];

		// If no part of this HoldStep is on the screen, skip it
		if( !( iIndexFirstArrowToDraw <= hs.m_iEndIndex && hs.m_iEndIndex <= iIndexLastArrowToDraw  ||
			   iIndexFirstArrowToDraw <= hs.m_iStartIndex  && hs.m_iStartIndex <= iIndexLastArrowToDraw  ||
			   hs.m_iStartIndex < iIndexFirstArrowToDraw && hs.m_iEndIndex > iIndexLastArrowToDraw ) )
		{
			continue;
		}



		int iCol = m_Style.TapStepToColumnNumber( hs.m_TapStep );
		float fHoldStepLife = m_HoldStepLife[i];
		
		bool bActive = StepIndexToBeat(hs.m_iStartIndex) > m_fSongBeat  &&  m_fSongBeat < StepIndexToBeat(hs.m_iEndIndex);

		if( bActive )
			m_ColorArrow[iCol].SetGrayPartFull();
		else
			m_ColorArrow[iCol].SetGrayPartClear();


		// draw the gray parts
		for( float j=(float)hs.m_iStartIndex;
			 j<=hs.m_iEndIndex; 
			 j+=1.9f/m_PlayerOptions.m_fArrowScrollSpeed )	// for each arrow in the run
		{
 			// check if this arrow is off the the screen
			if( j < iIndexFirstArrowToDraw || iIndexLastArrowToDraw < j)
				continue;	// skip this arrow

			DrawHoldStepGrayPart( iCol, j, hs, fHoldStepLife );
		}

		// draw the color parts
		for( j=(float)hs.m_iStartIndex; 
			 j<=hs.m_iEndIndex; 
			 j+=1.9f/m_PlayerOptions.m_fArrowScrollSpeed )	// for each arrow in the run
		{
			// check if this arrow is off the the screen
			if( j < iIndexFirstArrowToDraw || iIndexLastArrowToDraw < j )
				continue;	// skip this arrow

			DrawHoldStepColorPart( iCol, j, hs, fHoldStepLife );
		}

		
		// draw the first arrow on top of the others
		j = (float)hs.m_iStartIndex;

		DrawHoldStepGrayPart( iCol, j, hs, fHoldStepLife );
		DrawHoldStepColorPart( iCol, j, hs, fHoldStepLife );
	}
}




void ColorArrowField::RemoveTapStepRow( int iIndex )
{
	m_TapSteps[iIndex] = STEP_NONE;
}

void ColorArrowField::SetHoldStepLife( int iIndex, float fLife )
{
	m_HoldStepLife[iIndex] = fLife;
}