/*
-----------------------------------------------------------------------------
 File: ColorArrowField.h

 Desc: A pattern of ColorArrows that scroll past Y==0.

 Copyright (c) 2001-2002 by the names listed below.  All rights reserved.
	Chris Danford
-----------------------------------------------------------------------------
*/


#ifndef _ColorArrowField_H_
#define _ColorArrowField_H_


#include "Sprite.h"
#include "ActorFrame.h"
#include "Song.h"
#include "BitmapText.h"
#include "ThemeManager.h"
#include "Style.h"
#include "ColorArrow.h"
#include "BitmapText.h"
#include "Rectangle.h"




class ColorArrowField : public ActorFrame
{
public:
	ColorArrowField();
	virtual void Update( float fDeltaTime, float fSongBeat );
	virtual void RenderPrimitives();
	
	void Load( Style style, Steps steps, PlayerOptions po, float fNumArrowsToDrawBehind, float fNumArrowsToDrawAhead );
	void RemoveTapStepRow( int iIndex );
	void SetHoldStepLife( int iIndex, float fLife );

	TapStep			m_TapSteps[MAX_TAP_STEP_ELEMENTS];
	HoldStep		m_HoldSteps[MAX_HOLD_STEP_ELEMENTS];
	float			m_HoldStepLife[MAX_HOLD_STEP_ELEMENTS];		// 1.0 = full life, 0 = dead
	int				m_iNumHoldSteps;

protected:
	void DrawTapStep( int iCol, float fIndex );
	void DrawTapStep( int iCol, float fIndex, D3DXCOLOR color );
	void DrawHoldStepColorPart( int iCol, float fIndex, HoldStep &hs, float fHoldStepLife );
	void DrawHoldStepGrayPart( int iCol, float fIndex, HoldStep &hs, float fHoldStepLife );
	void DrawMeasureBar( int iIndex, int iMeasureNo );

	Style			m_Style;
	PlayerOptions	m_PlayerOptions;

	float			m_fNumBeatsToDrawBehind;
	float			m_fNumBeatsToDrawAhead;

	float			m_fSongBeat;


	// color arrows
	ColorArrow		m_ColorArrow[MAX_NUM_COLUMNS];
	
	RectangleActor	m_rectMeasureBar;
	BitmapText		m_textMeasureNumber;
};

#endif