/*
-----------------------------------------------------------------------------
 File: GameTypes.h

 Desc: These types don't belong in any particular file

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#ifndef _GameTypes_H_
#define _GameTypes_H_


enum GameMode 
{ 
	MODE_SINGLE  = 0, 
	MODE_VERSUS,
	MODE_DOUBLE,
	MODE_COUPLE,	// and battle
	MODE_SOLO,
	NUM_GAMEMODES	// leave this at the end!
};

enum PlayerNumber {
	PLAYER_1 = 0,
	PLAYER_2,
	NUM_PLAYERS,	// leave this at the end
	PLAYER_NONE
};

enum SongSortOrder { 
	SORT_GROUP, 
	SORT_TITLE, 
	SORT_BPM, 
	SORT_MOST_PLAYED, 
	NUM_SORT_ORDERS 
};

enum NoteType 
{ 
	NOTE_4TH,	// quarter notes
	NOTE_8TH,	// eighth notes
	NOTE_12TH,	// triplets
	NOTE_16TH,	// sixteenth notes
	NUM_NOTE_TYPES
};


struct PlayerOptions
{
	PlayerOptions() {
		m_fArrowScrollSpeed = 1.0f;
		m_EffectType = EFFECT_NONE;
		m_AppearanceType = APPEARANCE_VISIBLE;
		m_TurnType = TURN_NONE;
		m_bLittle = false;
		//m_bFlat = false;
		m_bReverseScroll = false;
		m_ColorType = COLOR_ARCADE;
		m_bAllowFreezeArrows = true;
		m_bShowMeasureBars = false;
	};

	float m_fArrowScrollSpeed;
	enum EffectType	{ EFFECT_NONE=0, EFFECT_BOOST, EFFECT_WAVE, EFFECT_DRUNK, EFFECT_DIZZY, EFFECT_SPACE };
	EffectType m_EffectType;
	enum AppearanceType	{ APPEARANCE_VISIBLE=0, APPEARANCE_HIDDEN, APPEARANCE_SUDDEN, APPEARANCE_STEALTH };
	AppearanceType m_AppearanceType;
	enum TurnType { TURN_NONE=0, TURN_MIRROR, TURN_LEFT, TURN_RIGHT, TURN_SHUFFLE };
	TurnType m_TurnType;
	bool m_bLittle;
	bool m_bReverseScroll;
	enum ColorType { COLOR_ARCADE=0, COLOR_NOTE, COLOR_FLAT, COLOR_PLAIN };
	ColorType m_ColorType;
	bool m_bAllowFreezeArrows;
	bool m_bShowMeasureBars;
};

struct GameOptions
{
	GameOptions() 
	{
		m_bWindowed = false;
		m_iResolution = 640;
		m_iDisplayColor = 16;
		m_iTextureColor = 16;
		m_bFilterTextures = true;
		m_bShadows = false;
		m_bIgnoreJoyAxes = false;
		m_bShowFPS = true;
		m_bUseRandomVis = false;
		m_bSkipCaution = false;
		m_bAnnouncer = true;
	};
	bool m_bWindowed;
	int m_iResolution;
	int m_iDisplayColor;
	int m_iTextureColor;
	bool m_bFilterTextures;
	bool m_bShadows;
	bool m_bIgnoreJoyAxes;
	bool m_bShowFPS;
	bool m_bUseRandomVis;
	bool m_bSkipCaution;
	bool m_bAnnouncer;
};

struct SongOptions
{
	SongOptions() {
		m_FailType = FAIL_ARCADE;
		m_AssistType = ASSIST_NONE;
		m_fMusicRate = 1.0f;
	};

	enum FailType { FAIL_ARCADE=0, FAIL_END_OF_SONG, FAIL_OFF };
	FailType m_FailType;
	enum AssistType { ASSIST_NONE=0, ASSIST_TICK };
	AssistType m_AssistType;
	float m_fMusicRate;
};

struct ScoreSummary {
	ScoreSummary() { perfect=great=good=boo=miss=ok=ng=max_combo=0; score=0; };
	int perfect, great, good, boo, miss, ok, ng, max_combo;
	float score;
};

#endif