#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: GhostArrows.h

 Desc: A graphic displayed in the GhostArrows during Dancing.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "GhostArrows.h"
#include "RageUtil.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "ColorArrow.h"
#include "ArrowEffects.h"



GhostArrows::GhostArrows()
{

}

void GhostArrows::Load( PlayerOptions po, Style style )
{
	m_PlayerOptions = po;
	m_Style = style;

	// init arrows
	for( int c=0; c<style.m_iNumColumns; c++ ) 
	{
		m_GhostArrows[c].SetRotation( style.m_ColumnToRotation[c] );
		m_GhostArrowsBright[c].SetRotation( style.m_ColumnToRotation[c] );
		m_HoldGhostArrows[c].SetRotation( style.m_ColumnToRotation[c] );
	}
}


void GhostArrows::Update( float fDeltaTime, float fSongBeat )
{
	m_fSongBeat = fSongBeat;

	for( int c=0; c<m_Style.m_iNumColumns; c++ )
	{
		m_GhostArrows[c].Update( fDeltaTime );
		m_GhostArrowsBright[c].Update( fDeltaTime );
		m_HoldGhostArrows[c].Update( fDeltaTime );

		m_GhostArrows[c].SetBeat( fSongBeat );
		m_GhostArrowsBright[c].SetBeat( fSongBeat );
		m_HoldGhostArrows[c].SetBeat( fSongBeat );
	}
}

void GhostArrows::RenderPrimitives()
{
	for( int c=0; c<m_Style.m_iNumColumns; c++ )
	{
		float fX = ArrowGetXPos( m_PlayerOptions, m_Style, c, 0, m_fSongBeat );
		m_GhostArrows[c].SetX( fX );
		m_GhostArrowsBright[c].SetX( fX );
		m_HoldGhostArrows[c].SetX( fX );

		m_GhostArrows[c].Draw();
		m_GhostArrowsBright[c].Draw();
		m_HoldGhostArrows[c].Draw();
	}
}


void GhostArrows::TapStep( int iCol, TapStepScore score, bool bBright )
{
	ASSERT( iCol >= 0  &&  iCol < m_Style.m_iNumColumns );
	if( bBright )
		m_GhostArrowsBright[iCol].Step( score );
	else
		m_GhostArrows[iCol].Step( score );
}

void GhostArrows::HoldStep( int iCol )
{
	ASSERT( iCol >= 0  &&  iCol < m_Style.m_iNumColumns );
	m_HoldGhostArrows[iCol].Step();
}
