#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: GranularityIndicator.h

 Desc: A graphic displayed in the GranularityIndicator during edit.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "GranularityIndicator.h"
#include "RageUtil.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "ColorArrow.h"
#include "ArrowEffects.h"
#include "GameConstants.h"


GranularityIndicator::GranularityIndicator()
{
	for( int i=0; i<2; i++ )
	{
		m_sprIndicators[i].Load( THEME->GetPathTo(GRAPHIC_EDIT_SNAP_INDICATOR) );
		this->AddActor( &m_sprIndicators[i] );
	}

	m_NoteType = NOTE_4TH;
	D3DXCOLOR color = NOTE_TYPE_TO_COLOR[m_NoteType];
	
	for( i=0; i<2; i++ )
	{
		m_sprIndicators[i].SetDiffuseColor( color );
	}
}

void GranularityIndicator::Load( Style style )
{
	m_Style = style;

	m_sprIndicators[0].StopTweening();
	m_sprIndicators[1].StopTweening();
	m_sprIndicators[0].SetX( -ARROW_SIZE * (m_Style.m_iNumColumns/2 + 0.5f) );
	m_sprIndicators[1].SetX(  ARROW_SIZE * (m_Style.m_iNumColumns/2 + 0.5f) );
}

void GranularityIndicator::PrevSnapMode()
{
	if( m_NoteType == 0 )
		return;
	m_NoteType = NoteType(m_NoteType-1);

	SnapModeChanged();
}

void GranularityIndicator::NextSnapMode()
{
	if( m_NoteType == NUM_NOTE_TYPES-1 )
		return;
	m_NoteType = NoteType(m_NoteType+1);

	SnapModeChanged();
}


void GranularityIndicator::SnapModeChanged()
{
	D3DXCOLOR color = NOTE_TYPE_TO_COLOR[m_NoteType];
	
	for( int i=0; i<2; i++ )
	{
		m_sprIndicators[i].BeginTweening( 0.3f );
		m_sprIndicators[i].SetTweenDiffuseColor( color );
	}
}
