#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: LifeMeterPills.h

 Desc: A graphic displayed in the LifeMeterPills during Dancing.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "LifeMeterPills.h"
#include "ThemeManager.h"


const float LIFE_PERFECT	=	 0.010f;
const float LIFE_GREAT		=	 0.005f;
const float LIFE_GOOD		=	 0.000f;
const float LIFE_BOO		=	-0.015f;
const float LIFE_MISS		=	-0.030f;


const float INITIAL_LIFE_PERCENTAGE	=	0.5f;

const int LIEFMETER_NUM_PILLS		=	17;
const float PILL_OFFSET_Y[LIEFMETER_NUM_PILLS] = {
	0.3f, 0.7f, 1.0f, 0.7f, 0.3f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f	// kind of a sin wave
};


LifeMeterPills::LifeMeterPills()
{
	m_fLifePercentage = INITIAL_LIFE_PERCENTAGE;

	m_sprLifeMeterFrame.Load( THEME->GetPathTo(GRAPHIC_LIFEMETER_FRAME) );
	m_sprLifeMeterFrame.StopAnimating();
	m_sprLifeMeterPills.Load( THEME->GetPathTo(GRAPHIC_LIFEMETER_PILLS) );
	m_sprLifeMeterPills.StopAnimating();

	this->AddActor( &m_sprLifeMeterFrame );
	this->AddActor( &m_sprLifeMeterPills );
}


void LifeMeterPills::ChangeLife( TapStepScore score )
{
	switch( score )
	{
	case TSS_PERFECT:	m_fLifePercentage += LIFE_PERFECT;	break;
	case TSS_GREAT:		m_fLifePercentage += LIFE_GREAT;	break;
	case TSS_GOOD:		m_fLifePercentage += LIFE_GOOD;		break;
	case TSS_BOO:		m_fLifePercentage += LIFE_BOO;		break;
	case TSS_MISS:		m_fLifePercentage += LIFE_MISS;		break;
	}

	m_fLifePercentage = clamp( m_fLifePercentage, 0, 1 );
	
	if( m_fLifePercentage == 1 )
		m_sprLifeMeterFrame.SetEffectCamelion( 5, D3DXCOLOR(0.2f,0.2f,0.2f,1), D3DXCOLOR(1,1,1,1) );
	else if( m_fLifePercentage < 0.25f )
		m_sprLifeMeterFrame.SetEffectCamelion( 5, D3DXCOLOR(1,0.8f,0.8f,1), D3DXCOLOR(1,0.2f,0.2f,1) );
	else
		m_sprLifeMeterFrame.SetEffectNone();

}

float LifeMeterPills::GetLifePercentage() 
{ 
	return m_fLifePercentage; 
}

void LifeMeterPills::RenderPrimitives()
{
	float fBeatPercentage = m_fSongBeat - (int)m_fSongBeat;
	int iOffsetStart = roundf( LIEFMETER_NUM_PILLS*fBeatPercentage );

	m_sprLifeMeterFrame.Draw();

	float iX = m_sprLifeMeterFrame.GetX() - m_sprLifeMeterFrame.GetZoomedWidth()/2 + 27;
	int iNumPills = (int)(m_sprLifeMeterPills.GetNumStates() * m_fLifePercentage);
	int iPillWidth = (int)m_sprLifeMeterPills.GetZoomedWidth();

	for( int i=0; i<iNumPills; i++ )
	{
		m_sprLifeMeterPills.SetState( i );
		m_sprLifeMeterPills.SetX( iX );

		int iOffsetNum = (iOffsetStart - i + LIEFMETER_NUM_PILLS) % LIEFMETER_NUM_PILLS;
		int iOffset = roundf( -PILL_OFFSET_Y[iOffsetNum] * m_fLifePercentage * 8.0f );
		m_sprLifeMeterPills.SetY( (float)iOffset );

		m_sprLifeMeterPills.Draw();

		iX += iPillWidth;
	}
}