/*
-----------------------------------------------------------------------------
 File: PadInput.h

 Desc: Input translated to a player's pad and button.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#ifndef _PADINPUT_H_
#define _PADINPUT_H_


#include "RageInput.h"


enum PadNumber {
	PAD_1 = 0,
	PAD_2,
	NUM_PADS,	// leave this at the end
	PAD_NONE	// this means NULL
};

enum PadButton {
	BUTTON_LEFT	= 0,
	BUTTON_UPLEFT,
	BUTTON_DOWN,
	BUTTON_UP,
	BUTTON_UPRIGHT,
	BUTTON_RIGHT,
	BUTTON_BACK,
	BUTTON_NEXT,
	NUM_PAD_BUTTONS,	// leave this at the end
	BUTTON_NONE
};

const CString g_sPIButtonDescription[NUM_PAD_BUTTONS] = {
	"Left",
	"Up+Left",
	"Down",
	"Up",
	"Up+Right",
	"Right",
	"Back",
	"Next",
};


struct PadInput
{
public:
	PadNumber pad_no;
	PadButton button;

	PadInput() { MakeBlank(); };
	PadInput( PadNumber p, PadButton b ) { pad_no = p; button = b; };
	bool operator== (const PadInput o) const { return (pad_no == o.pad_no) && (button == o.button); };

	bool IsBlank() const { return pad_no == PAD_NONE; };
	void MakeBlank() { pad_no = PAD_NONE; button = BUTTON_NONE; };
	
	CString toString() 
	{
		return ssprintf("%d-%d", pad_no, button );
	}

	bool fromString( CString s )
	{ 
		CStringArray a;
		split( s, "-", a);

		if( a.GetSize() != 2 ) {
			pad_no = PAD_NONE;
			return false;
		}

		pad_no = (PadNumber)atoi( a[0] );
		button = (PadButton)atoi( a[1] );
		return true;
	};

};




#endif