#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 Class: RageSoundSample

 Desc: See header.

 Copyright (c) 2001-2002 by the names listed below.  All rights reserved.
	Chris Danford
-----------------------------------------------------------------------------
*/



#include "RageSoundSample.h"
#include "RageUtil.h"


RageSoundSample::RageSoundSample()
{
	m_hSample = NULL;

	m_hLastChannelPlayed = NULL;

	m_fLastKnownPosition = 0;
}

RageSoundSample::~RageSoundSample()
{
	Unload();
}

void RageSoundSample::Unload()
{
	if( m_hSample )
	{
		Stop();
		BASS_SampleFree( m_hSample );
	}
}
	
bool RageSoundSample::Load( CString sSoundFilePath )
{
	RageLog( "RageSoundSample::Load( %s )", sSoundFilePath );

	RageLog( "Made it here - a" );

	Unload();

	RageLog( "Made it here - b" );

	m_hSample = BASS_SampleLoad( FALSE, (void*)((LPCTSTR)sSoundFilePath), 0, 0, 5, 0 );

	RageLog( "Made it here - c" );
	if( m_hSample == NULL )
		RageError( ssprintf("RageSoundSample::Load: error loading %s (error code %d)", 
											   sSoundFilePath, BASS_ErrorGetCode()) );


	return true;
}

void RageSoundSample::Play()
{
	m_hLastChannelPlayed = BASS_SamplePlay( m_hSample );
}

void RageSoundSample::Pause()
{
	BASS_ChannelPause( m_hLastChannelPlayed );
}

void RageSoundSample::Stop()
{
	BASS_SampleStop( m_hSample );
}

float RageSoundSample::GetLengthSeconds()
{
	BASS_SAMPLE info;
	BASS_SampleGetInfo( m_hSample, &info);

	float fSeconds = info.length/(float)info.freq;
	return fSeconds;
}

float RageSoundSample::GetPositionSeconds()
{
	if( !IsPlaying() )
		return m_fLastKnownPosition;

	DWORD dwPosition = BASS_ChannelGetPosition( m_hLastChannelPlayed );
	float fSeconds = BASS_ChannelBytes2Seconds( m_hLastChannelPlayed, dwPosition );
	
	if( fSeconds == -1 )
		return 0;
	else		
	{
		m_fLastKnownPosition = fSeconds;	// save this so we can report it later when the song is paused
		return fSeconds;
	}
}

bool RageSoundSample::SetPlaybackRate( float fScale )
{
	BASS_SAMPLE info;
	BASS_SampleGetInfo( m_hSample, &info);

	return BASS_ChannelSetAttributes( m_hLastChannelPlayed, DWORD(info.freq * fScale), -1, -101 ) == 1;
}

float RageSoundSample::GetPlaybackRate()
{
	DWORD freq;
	if( ! BASS_ChannelGetAttributes( m_hLastChannelPlayed, &freq, NULL, NULL ) )
		return 1;

	BASS_SAMPLE info;
	BASS_SampleGetInfo( m_hSample, &info);

	return freq / (float)info.freq;
}

bool RageSoundSample::IsPlaying()
{
	return BASS_ChannelIsActive(m_hLastChannelPlayed) ? true : false;
}