/*
-----------------------------------------------------------------------------
 Class: RageSoundSample

 Desc: A wrapper for Bass's SAMPLE type.  This class holds a single sound sample 
	from which and multiple channels (instances) can be generated.

 Copyright (c) 2001-2002 by the names listed below.  All rights reserved.
	Chris Danford
-----------------------------------------------------------------------------
*/

#ifndef _RageSoundSample_H_
#define _RageSoundSample_H_


#include "RageSound.h"


class RageSoundSample
{
public:
	RageSoundSample();
	~RageSoundSample();

	bool Load( CString sSoundFilePath );
	void Unload();

	void Play();
	void Pause();
	void Stop();
	float GetLengthSeconds();
	float GetPositionSeconds();	// position of the most recent channel played
	bool SetPlaybackRate( float fScale );
	float GetPlaybackRate();
	bool IsPlaying();

private:

	HSAMPLE m_hSample;
	float m_fLastKnownPosition;	// Bass will not report a sample position while it's paused,
								// so keep a record of the last position we saw so that we
								// can use this as the return value of GetPositionSeconds()
								// while the sample is paused.

	HCHANNEL m_hLastChannelPlayed;
};




#endif