#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: RageSoundStream.h

 Desc: DirectShow sound (MP3, WMA) support.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include <windows.h>
#include <dshow.h>
#include <tchar.h>
#include <malloc.h>
#include "RageSoundStream.h"
#include "RageUtil.h"



RageSoundStream::RageSoundStream()
{
	m_hStream = NULL;
}

RageSoundStream::~RageSoundStream()
{
	Unload();
}

void RageSoundStream::Unload()
{
	if( m_hStream ) 
	{
		Stop();
		BASS_StreamFree( m_hStream );
		m_hStream = NULL;
	}
}

bool RageSoundStream::Load( CString sFileName )
{
	RageLog( "RageSound::LoadSound( '%s' )", sFileName );

	Unload();

	m_hStream = BASS_StreamCreateFile( FALSE, (void*)((LPCTSTR)sFileName), 0, 0, 0 );
	if( m_hStream == NULL )
		RageError( ssprintf("RageSound::LoadSound: error loading %s (error code %d)", 
								               sFileName, BASS_ErrorGetCode()) );

	// save original frequency because we can't get it back after changing it
	BASS_ChannelGetAttributes( m_hStream, &m_dwOriginalFreq, NULL, NULL );

	return true;
}

void RageSoundStream::Update( float fDeltaTime )
{

}

void RageSoundStream::Play( bool bLoop )
{
	if( FALSE == BASS_StreamPlay( m_hStream, FALSE, bLoop ? BASS_SAMPLE_LOOP : 0 ) )
		RageError( "There was an error playing a sound stream.  Are you sure Load()ed it first?" );
}

void RageSoundStream::Pause()
{
	if( FALSE == BASS_ChannelPause( m_hStream ) )
		RageLog( "WARNING!  Error pausing a sound stream." );
}

void RageSoundStream::Stop()
{
	if( FALSE == BASS_ChannelStop( m_hStream ) )
		RageError( "There was an error stopping a sound stream.  Are you sure this is a valid HSTREAM?" );
}

float RageSoundStream::GetLengthSeconds()
{
	DWORD dwLength = BASS_StreamGetLength( m_hStream ); 
	float fSeconds = BASS_ChannelBytes2Seconds( m_hStream, dwLength );
	return fSeconds;
}

float RageSoundStream::GetPositionSeconds()
{
	DWORD dwPosition = BASS_ChannelGetPosition( m_hStream );
	float fSeconds = BASS_ChannelBytes2Seconds( m_hStream, dwPosition );
	return fSeconds;
}


bool RageSoundStream::SetPlaybackRate( float fScale )
{
	return BASS_ChannelSetAttributes( m_hStream, DWORD(m_dwOriginalFreq * fScale), -1, -101 ) == 1;
}

float RageSoundStream::GetPlaybackRate()
{
	DWORD freq;
	if( ! BASS_ChannelGetAttributes( m_hStream, &freq, NULL, NULL ) )
		return 1;

	return freq / (float)m_dwOriginalFreq;
}


bool RageSoundStream::IsPlaying()
{
	return (0 == BASS_ChannelIsActive(m_hStream));
}

