#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: RandomStream.h

 Desc: Holds multiple sounds samples and can play a random sound easily.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/



#include "RandomStream.h"
#include "RageUtil.h"


RandomStream::RandomStream()
{
	m_iIndexLastPlayed = -1;
}

RandomStream::~RandomStream()
{
	for( int i=0; i<m_pStreams.GetSize(); i++ )
		SAFE_DELETE( m_pStreams[i] );
}


bool RandomStream::LoadSoundDir( CString sDir )
{
	// make sure there's a backslash at the end of this path
	if( sDir[sDir.GetLength()-1] != '\\' )
		sDir += "\\";

	CStringArray arraySoundFiles;
	GetDirListing( sDir + "*.mp3", arraySoundFiles );
	GetDirListing( sDir + "*.wav", arraySoundFiles );

	for( int i=0; i<arraySoundFiles.GetSize(); i++ )
		LoadSound( sDir + arraySoundFiles[i] );

	return true;
}
	
bool RandomStream::LoadRandomStream( CString sSetFilePath )
{
	CStdioFile file;
	if( !file.Open(sSetFilePath, CFile::modeRead|CFile::shareDenyNone) )
		RageError( ssprintf("Error opening sound set file '%s'.", sSetFilePath) );

	
	// Split for the directory of the sound set file.  We'll need it below
	CString sDir, sFileName, sExtension;
	splitrelpath( sSetFilePath, sDir, sFileName, sExtension );


	CString line;
	while( file.ReadString(line) )
	{
		if( line != "" )
			LoadSound( sDir + line );
	}

	return true;
}
	
bool RandomStream::LoadSound( CString sSoundFilePath )
{
	RageLog( "RandomStream::LoadSound( %s )", sSoundFilePath );

	RageLog( "Made it here - A" );

	RageSoundStream* pSS = new RageSoundStream;
	RageLog( "Made it here - B" );
	pSS->Load( sSoundFilePath );


	m_pStreams.Add( pSS );
	
	RageLog( "Made it here - C" );

	return true;
}

void RandomStream::PlayRandom()
{
	// play one of the samples
	if( m_pStreams.GetSize() == 0 )
	{
		RageLog( "WARNING:  Tried to play a RandomStream that has 0 sounds loaded." );
	}
	else
	{
		int iIndexToPlay = rand() % m_pStreams.GetSize();
		m_pStreams[iIndexToPlay]->Play();
		m_iIndexLastPlayed = iIndexToPlay;
	}
}

void RandomStream::Pause()
{
	m_pStreams[m_iIndexLastPlayed]->Pause();
}

void RandomStream::Stop()
{
	if( m_iIndexLastPlayed == -1 )	// nothing is currently playing
		return;

	m_pStreams[m_iIndexLastPlayed]->Stop();
}
