/*
-----------------------------------------------------------------------------
 File: StepsSelector.h

 Desc: A control for two players to select which Steps to use.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/


#ifndef _StepsSelector_H_
#define _StepsSelector_H_


#include "Sprite.h"
#include "Song.h"
#include "ActorFrame.h"
#include "BitmapText.h"
#include "Rectangle.h"
#include "TextBanner.h"
#include "FootMeter.h"
#include "DifficultyIcon.h"
#include "RandomSample.h"
#include "WindowMessage.h"


const WindowMessage	SM_StepsChanged	=	WindowMessage(SM_User+68);	// this should be unique!


class StepsSelector : public ActorFrame
{
public:
	StepsSelector();

	virtual void Update( float fDeltaTime );
	virtual void RenderPrimitives();

	void LoadFromSong( Song* pSong, GameMode mode );

	Steps* GetSelectedSteps( PlayerNumber p )
	{
		if( !m_bEnabled[p] )
			return NULL;
		if( m_iSelection[p] == -1 )
			return NULL;
		if( p > m_pSteps.GetSize()-1 )
			return NULL;
		return m_pSteps[m_iSelection[p]];
	};
	void SelectTheseSteps( PlayerNumber p, Steps* pSteps )
	{
		for( int i=0; i<m_pSteps.GetSize(); i++ )
		{
			if( m_pSteps[i] == pSteps )
			{
				ChangeSelection( p, i );
				return;
			}
		}
	};

	void Enable( PlayerNumber p )	{ m_bEnabled[p] = true;		m_iSelection[p] = 0;	MakeSelectionValid(p);	};
	void Disable( PlayerNumber p )	{ m_bEnabled[p] = false;	m_iSelection[p] = -1; };

	void ChangeSelection( PlayerNumber p, int iNewSelection );
	void Up( PlayerNumber p );
	void Down( PlayerNumber p );

	void MakeSelectionValid( PlayerNumber p )
	{
		if( m_iSelection[p] < 0  ||  m_iSelection[p] > m_pSteps.GetSize()-1 )	// selection out of bounds
			m_iSelection[p] = -1;
		else if( m_pSteps[m_iSelection[p]] == NULL )	// selection is a blank slot
			m_iSelection[p] = -1;

		if( m_iSelection[p] == -1 )
		{
			// find a new, valid selection.  Select the lowest difficulty possible.
			for( int i=0; i<m_pSteps.GetSize(); i++ )
			{
				if( m_pSteps[i] != NULL)
				{
					m_iSelection[p] = i;
					break;
				}
			}		
		}
	};

private:
	void BeforeChange( PlayerNumber p );
	void AfterChange( PlayerNumber p );

	int			m_iSelection[NUM_PLAYERS];
	bool		m_bEnabled[NUM_PLAYERS];


#define MAX_NUM_STEPS_LINES 8
	CArray<Steps*,Steps*>	m_pSteps;
	FootMeter		m_FootMeter[MAX_NUM_STEPS_LINES];
	DifficultyIcon	m_DifficultyIcon[MAX_NUM_STEPS_LINES][NUM_PLAYERS];

	
	RandomSample m_soundChangeSteps;
};


#endif