#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: Style.cpp

 Desc: A data structure that holds the definition of a GameMode.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "Style.h"
#include "D3DXMath.h"


Style GetStyle( GameMode mode )
{
	Style style;


	CMap<TapStep, TapStep, float, float> mapTapStepToRotation;	// arrow facing left is rotation 0
	mapTapStepToRotation[STEP_PAD1_LEFT]	= 0;
	mapTapStepToRotation[STEP_PAD1_UPLEFT]	= D3DX_PI/4.0f;
	mapTapStepToRotation[STEP_PAD1_DOWN]	= -D3DX_PI/2.0f;
	mapTapStepToRotation[STEP_PAD1_UP]		= D3DX_PI/2.0f;
	mapTapStepToRotation[STEP_PAD1_UPRIGHT]	= D3DX_PI*3.0f/4.0f;
	mapTapStepToRotation[STEP_PAD1_RIGHT]	= D3DX_PI;
	mapTapStepToRotation[STEP_PAD2_LEFT]	= mapTapStepToRotation[STEP_PAD1_LEFT];
	mapTapStepToRotation[STEP_PAD2_UPLEFT]	= mapTapStepToRotation[STEP_PAD1_UPLEFT];
	mapTapStepToRotation[STEP_PAD2_DOWN]	= mapTapStepToRotation[STEP_PAD1_DOWN];
	mapTapStepToRotation[STEP_PAD2_UP]		= mapTapStepToRotation[STEP_PAD1_UP];
	mapTapStepToRotation[STEP_PAD2_UPRIGHT]	= mapTapStepToRotation[STEP_PAD1_UPRIGHT];
	mapTapStepToRotation[STEP_PAD2_RIGHT]	= mapTapStepToRotation[STEP_PAD1_RIGHT];


	switch( mode )
	{
	case MODE_SINGLE:
	case MODE_DOUBLE:
	case MODE_SOLO:
		style.m_iNumPlayers = 1;
		break;
	case MODE_VERSUS:
	case MODE_COUPLE:
		style.m_iNumPlayers = 2;
		break;
	default:
		ASSERT( false );	// invalid GameMode
	}


	switch( mode )
	{
	case MODE_SINGLE:
	case MODE_VERSUS:
	case MODE_COUPLE:
		style.m_iNumColumns = 4;		// LEFT, DOWN, UP, RIGHT
		style.m_ColumnToTapStep[0]		= STEP_PAD1_LEFT;
		style.m_ColumnToTapStep[1]		= STEP_PAD1_DOWN;
		style.m_ColumnToTapStep[2]		= STEP_PAD1_UP;
		style.m_ColumnToTapStep[3]		= STEP_PAD1_RIGHT;
		style.m_ColumnToTapStep[4]		= STEP_PAD1_UPLEFT; // Even though this isn't used, Need them here so
		style.m_ColumnToTapStep[5]		= STEP_PAD1_UPRIGHT; // TapStepToColumnNumber doesn't get confused by them
		style.m_ColumnToRotation[0] = mapTapStepToRotation[STEP_PAD1_LEFT];
		style.m_ColumnToRotation[1] = mapTapStepToRotation[STEP_PAD1_DOWN];
		style.m_ColumnToRotation[2] = mapTapStepToRotation[STEP_PAD1_UP];
		style.m_ColumnToRotation[3] = mapTapStepToRotation[STEP_PAD1_RIGHT];
		style.m_ColumnToRotation[4] = mapTapStepToRotation[STEP_PAD1_UPLEFT];
		style.m_ColumnToRotation[5] = mapTapStepToRotation[STEP_PAD1_UPRIGHT];
		break;
	case MODE_SOLO:
		style.m_iNumColumns = 6;		// LEFT, UP+LEFT, DOWN, UP, UP+RIGHT, RIGHT
		style.m_ColumnToTapStep[0]		= STEP_PAD1_LEFT;
		style.m_ColumnToTapStep[1]		= STEP_PAD1_UPLEFT;
		style.m_ColumnToTapStep[2]		= STEP_PAD1_DOWN;
		style.m_ColumnToTapStep[3]		= STEP_PAD1_UP;
		style.m_ColumnToTapStep[4]		= STEP_PAD1_UPRIGHT;
		style.m_ColumnToTapStep[5]		= STEP_PAD1_RIGHT;
		style.m_ColumnToRotation[0] = mapTapStepToRotation[STEP_PAD1_LEFT];
		style.m_ColumnToRotation[1] = mapTapStepToRotation[STEP_PAD1_UPLEFT];
		style.m_ColumnToRotation[2] = mapTapStepToRotation[STEP_PAD1_DOWN];
		style.m_ColumnToRotation[3] = mapTapStepToRotation[STEP_PAD1_UP];
		style.m_ColumnToRotation[4] = mapTapStepToRotation[STEP_PAD1_UPRIGHT];
		style.m_ColumnToRotation[5] = mapTapStepToRotation[STEP_PAD1_RIGHT];
		break;
	case MODE_DOUBLE:
		style.m_iNumColumns = 8;		// 1_LEFT, 1_DOWN, 1_UP, 1_RIGHT, 2_LEFT, 2_DOWN, 2_UP, 2_RIGHT
		style.m_ColumnToTapStep[0]		= STEP_PAD1_LEFT;
		style.m_ColumnToTapStep[1]		= STEP_PAD1_DOWN;
		style.m_ColumnToTapStep[2]		= STEP_PAD1_UP;
		style.m_ColumnToTapStep[3]		= STEP_PAD1_RIGHT;
		style.m_ColumnToTapStep[4]		= STEP_PAD2_LEFT;
		style.m_ColumnToTapStep[5]		= STEP_PAD2_DOWN;
		style.m_ColumnToTapStep[6]		= STEP_PAD2_UP;
		style.m_ColumnToTapStep[7]		= STEP_PAD2_RIGHT;
		style.m_ColumnToTapStep[8]		= STEP_PAD1_UPLEFT;// Even though this isn't used, Need them here so
		style.m_ColumnToTapStep[9]		= STEP_PAD1_UPRIGHT;// TapStepToColumnNumber doesn't get confused by them
		style.m_ColumnToTapStep[10]		= STEP_PAD2_UPLEFT;
		style.m_ColumnToTapStep[11]		= STEP_PAD2_UPRIGHT;
		style.m_ColumnToRotation[0] = mapTapStepToRotation[STEP_PAD1_LEFT];
		style.m_ColumnToRotation[1] = mapTapStepToRotation[STEP_PAD1_DOWN];
		style.m_ColumnToRotation[2] = mapTapStepToRotation[STEP_PAD1_UP];
		style.m_ColumnToRotation[3] = mapTapStepToRotation[STEP_PAD1_RIGHT];
		style.m_ColumnToRotation[4] = mapTapStepToRotation[STEP_PAD2_LEFT];
		style.m_ColumnToRotation[5] = mapTapStepToRotation[STEP_PAD2_DOWN];
		style.m_ColumnToRotation[6] = mapTapStepToRotation[STEP_PAD2_UP];
		style.m_ColumnToRotation[7] = mapTapStepToRotation[STEP_PAD2_RIGHT];
		style.m_ColumnToRotation[8] = mapTapStepToRotation[STEP_PAD1_UPLEFT];
		style.m_ColumnToRotation[9] = mapTapStepToRotation[STEP_PAD1_UPRIGHT];
		style.m_ColumnToRotation[10] = mapTapStepToRotation[STEP_PAD2_UPLEFT];
		style.m_ColumnToRotation[11] = mapTapStepToRotation[STEP_PAD2_UPRIGHT];
		break;
	default:
		ASSERT( false );	// invalid GameMode
	}

	return style;
}