#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: Window.h

 Desc: Class representing a game state.  It also holds Actors.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "Window.h"

Window::Window()
{
}

Window::~Window()
{

}

void Window::Update( float fDeltaTime )
{
	ActorFrame::Update( fDeltaTime );


	// update the times of queued WindowMessages and send if timer has expired
	// The order you remove messages in must be very careful!  Sending a message can 
	// potentially clear all m_QueuedMessages, and set a new state!
	for( int i=0; i<m_QueuedMessages.GetSize(); i++ )
	{
		if( m_QueuedMessages[i].fDelayRemaining <= 0.0f )		// send this sucker!
		{
			this->HandleWindowMessage( m_QueuedMessages[i].SM );
			m_QueuedMessages.RemoveAt( i );
			i--;
		}
		else
		{
			m_QueuedMessages[i].fDelayRemaining -= fDeltaTime;
		}
	}
}


void Window::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	// default input handler used by most menus
	switch( PlayerI.player_no )
	{
	case PLAYER_1:
		switch( PadI.button )
		{
		case BUTTON_UP:			this->Player1Up();		return;
		case BUTTON_DOWN:		this->Player1Down();	return;
		case BUTTON_LEFT:		this->Player1Left();	return;
		case BUTTON_RIGHT:		this->Player1Right();	return;
		case BUTTON_UPRIGHT:	this->Player1UpRight();	return;
		case BUTTON_UPLEFT:		this->Player1UpLeft();	return;
		case BUTTON_BACK:		this->Player1Back();	return;
		case BUTTON_NEXT:		this->Player1Next();	return;
		}
		break;
	case PLAYER_2:
		switch( PadI.button )
		{
		case BUTTON_UP:			this->Player2Up();		return;
		case BUTTON_DOWN:		this->Player2Down();	return;
		case BUTTON_LEFT:		this->Player2Left();	return;
		case BUTTON_RIGHT:		this->Player2Right();	return;
		case BUTTON_UPRIGHT:	this->Player2UpRight();	return;
		case BUTTON_UPLEFT:		this->Player2UpLeft();	return;
		case BUTTON_BACK:		this->Player2Back();	return;
		case BUTTON_NEXT:		this->Player2Next();	return;
		}
		break;
	default:	// if we get here, the pad that pressed the button isn't used in the current game mode.
		switch( PadI.button )
		{
		case BUTTON_UP:			this->Up();			return;
		case BUTTON_DOWN:		this->Down();		return;
		case BUTTON_LEFT:		this->Left();		return;
		case BUTTON_RIGHT:		this->Right();		return;
		case BUTTON_UPRIGHT:	this->UpRight();	return;
		case BUTTON_UPLEFT:		this->UpLeft();		return;
		case BUTTON_BACK:		this->Back();		return;
		case BUTTON_NEXT:		this->Next();		return;
		}
		break;
	}

}


void Window::SendWindowMessage( WindowMessage SM, float fDelay )
{
	assert( fDelay >= 0.0 );

	QueuedWindowMessage QSM;
	QSM.SM = SM;
	QSM.fDelayRemaining = fDelay;
	m_QueuedMessages.Add( QSM );
}