/*
-----------------------------------------------------------------------------
 File: Window.h

 Desc: Class representing a game state.  It also holds Actors.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#ifndef _Window_H_
#define _Window_H_


#include "RageInput.h"
#include "PadInput.h"
#include "RageScreen.h"
#include "PrefsManager.h"
#include "ActorFrame.h"
#include "WindowMessage.h"



class Window : public ActorFrame
{
public:
	Window();
	virtual ~Window();

	// let subclass override if they want
	virtual void Restore() {};
	virtual void Invalidate() {};

	virtual void Update( float fDeltaTime );
	virtual void Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI );
	virtual void HandleWindowMessage( WindowMessage SM ) {};

	void SendWindowMessage( WindowMessage SM, float fDelay );
	void ClearMessageQueue() { m_QueuedMessages.RemoveAll(); };

protected:

	// structure for holding messages sent to a Window
	struct QueuedWindowMessage {
		WindowMessage SM;  
		float fDelayRemaining;
	};
	CArray<QueuedWindowMessage, QueuedWindowMessage&>	m_QueuedMessages;

public:

	// let subclass override if they want
	virtual void Player1Up()		{ this->Up();		};
	virtual void Player1Down()		{ this->Down();		};
	virtual void Player1Left()		{ this->Left();		};
	virtual void Player1Right()		{ this->Right();	};
	virtual void Player1Next()		{ this->Next();		};
	virtual void Player1Back()		{ this->Back();		};
	virtual void Player1UpLeft()	{ this->UpLeft();	};
	virtual void Player1UpRight()	{ this->UpRight();	};

	virtual void Player2Up()		{ this->Up();		};
	virtual void Player2Down()		{ this->Down();		};
	virtual void Player2Left()		{ this->Left();		};
	virtual void Player2Right()		{ this->Right();	};
	virtual void Player2Next()		{ this->Next();		};
	virtual void Player2Back()		{ this->Back();		};
	virtual void Player2UpLeft()	{ this->UpLeft();	};
	virtual void Player2UpRight()	{ this->UpRight();	};

	virtual void Up()	{};
	virtual void Down() {};
	virtual void Left() {};
	virtual void Right() {};
	virtual void Next() {};
	virtual void Back() {};
	virtual void UpLeft() {};
	virtual void UpRight() {};
	
};


#endif