#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowCaution.h

Desc: Screen that displays while resources are being loaded.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/


#include "WindowCaution.h"
#include "ScreenDimensions.h"
#include "WindowMenuSelectStyle.h"
#include "RageTextureManager.h"
#include "ThemeManager.h"




const WindowMessage SM_DoneOpening		= WindowMessage(SM_User-7);
const WindowMessage SM_StartClosing		= WindowMessage(SM_User-8);
const WindowMessage SM_GoToSelectMusic	= WindowMessage(SM_User-9);




WindowCaution::WindowCaution()
{
	m_sprCaution.Load( THEME->GetPathTo(GRAPHIC_CAUTION) );
	m_sprCaution.StretchTo( CRect(0,0,640,480) );
	this->AddActor( &m_sprCaution );
	
	m_Wipe.OpenWipingRight( SM_DoneOpening );
	this->AddActor( &m_Wipe );

	this->SendWindowMessage( SM_StartClosing, 3 );

	m_soundCaution.Load( THEME->GetPathTo(ANNOUNCER_CAUTION) );
}


void WindowCaution::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	if( m_Wipe.IsClosing() )
		return;

	Window::Input( DeviceI, PadI, PlayerI );
}


void WindowCaution::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_StartClosing:
		m_Wipe.CloseWipingRight( SM_GoToSelectMusic );
		break;
	case SM_DoneOpening:
		if( PREFS->m_GameOptions.m_bAnnouncer )
			m_soundCaution.PlayRandom();
		break;
	case SM_GoToSelectMusic:
		WM->SetNewWindow( new WindowMenuSelectStyle );
		break;
	}
}

void WindowCaution::Next()
{
	m_Wipe.CloseWipingRight( SM_GoToSelectMusic );
}
