#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowDancing.h

 Desc: The music plays, the notes scroll, and the Player is pressing buttons.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowDancing.h"
#include "SongManager.h"
#include "WindowManager.h"

#include "WindowMenuSelectMusic.h"
#include "WindowMenuResults.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"

//
// Defines specific to GameWindowTitleMenu
//

const float COLUMN_LEFT_SINGLE		=	(SCREEN_WIDTH*1/4);
const float COLUMN_RIGHT_SINGLE		=	(SCREEN_WIDTH*3/4);
const float COLUMN_LEFT_DOUBLE		=	(COLUMN_LEFT_SINGLE+30);
const float COLUMN_RIGHT_DOUBLE		=	(COLUMN_RIGHT_SINGLE-30);




const float MAX_SECONDS_CAN_BE_OFF_BY	=	0.20f;

const float TIME_BETWEEN_DANCING_COMMENTS	=	15;


// received while STATE_DANCING
const WindowMessage	SM_SongEnded			= WindowMessage(SM_User+102);
const WindowMessage	SM_LifeIs0				= WindowMessage(SM_User+103);


// received while STATE_OUTRO
const WindowMessage	SM_ShowCleared			= WindowMessage(SM_User+111);
const WindowMessage	SM_HideCleared			= WindowMessage(SM_User+112);
const WindowMessage	SM_GoToResults			= WindowMessage(SM_User+113);

const WindowMessage	SM_BeginFailed			= WindowMessage(SM_User+121);
const WindowMessage	SM_ShowFailed			= WindowMessage(SM_User+122);
const WindowMessage	SM_PlayFailComment		= WindowMessage(SM_User+123);
const WindowMessage	SM_HideFailed			= WindowMessage(SM_User+124);
const WindowMessage	SM_GoToSelectSong		= WindowMessage(SM_User+125);



WindowDancing::WindowDancing()
{
	RageLog( "WindowDancing::WindowDancing()" );

	m_DancingState = STATE_INTRO;
	m_fTimeLeftBeforeDancingComment = TIME_BETWEEN_DANCING_COMMENTS;

	m_bHasFailed = false;
	m_pSong = SONGS->m_pCurSong;



	// Get the current style definition (used below)
	Style style = GetStyle( PREFS->m_GameMode );

	// load Steps into Player
	switch( PREFS->m_GameMode )
	{
	case MODE_SINGLE:
	case MODE_SOLO:
	case MODE_VERSUS:
	case MODE_DOUBLE:
		if( PREFS->IsPlayerEnabled( PLAYER_1 ) )
		{
			m_Player[PLAYER_1].Load( 
				style,
				PLAYER_1,
				*SONGS->m_pStepsPlayer[PLAYER_1], 
				PREFS->m_PlayerOptions[PLAYER_1] 
			);
		}
		if( PREFS->IsPlayerEnabled( PLAYER_2 ) )
		{
			m_Player[PLAYER_2].Load( 
				style,
				PLAYER_2,
				*SONGS->m_pStepsPlayer[PLAYER_2], 
				PREFS->m_PlayerOptions[PLAYER_2] 
			);
		}
		break;
	case MODE_COUPLE:
		{
		Steps steps1 = *SONGS->m_pStepsPlayer[PLAYER_1];
		Steps steps2 = *SONGS->m_pStepsPlayer[PLAYER_2];

		steps1.CropToLeftSide();
		steps2.CropToRightSide();

		m_Player[PLAYER_1].Load( style, PLAYER_1, steps1, PREFS->m_PlayerOptions[PLAYER_1] );
		m_Player[PLAYER_2].Load( style, PLAYER_2, steps2, PREFS->m_PlayerOptions[PLAYER_2] );
		
		}
		break;
	}



	m_Background.LoadFromSong( SONGS->m_pCurSong );
	m_Background.SetDiffuseColor( D3DXCOLOR(0.4f,0.4f,0.4f,1) );
	this->AddActor( &m_Background );




	// set player positions
	switch( PREFS->m_GameMode )
	{
	case MODE_SINGLE:
		m_Player[PLAYER_1].SetX( COLUMN_LEFT_SINGLE );
		this->AddActor( &m_Player[PLAYER_1] );
		break;
	case MODE_VERSUS:
	case MODE_COUPLE:
		m_Player[PLAYER_1].SetX( COLUMN_LEFT_SINGLE );
		this->AddActor( &m_Player[PLAYER_1] );
		m_Player[PLAYER_2].SetX( COLUMN_RIGHT_SINGLE );
		this->AddActor( &m_Player[PLAYER_2] );
		break;
	case MODE_DOUBLE:
	case MODE_SOLO:
		m_Player[PLAYER_1].SetX( CENTER_X );
		this->AddActor( &m_Player[PLAYER_1] );
		break;
	}


	m_StarWipe.SetClosed();
	this->AddActor( &m_StarWipe );

	m_textStage.Load( THEME->GetPathTo(FONT_FUTURISTIC) );
	m_textStage.SetXY( CENTER_X, CENTER_Y );
	m_textStage.SetZoom( 2 );
	m_textStage.SetDiffuseColor( D3DXCOLOR(1,1,1,0) );

	CString sNumberSuffix;
	if( ( (PREFS->m_iCurrentStage/10) % 10 ) == 1 )	// in the teens (e.g. 19, 213)
	{
		sNumberSuffix = "th";
	}
	else	// not in the teens
	{
		switch( PREFS->m_iCurrentStage%10 )
		{
		case 1:	sNumberSuffix = "st";	break;
		case 2:	sNumberSuffix = "nd";	break;
		case 3:	sNumberSuffix = "rd";	break;
		default:sNumberSuffix = "th";	break;
		}
	}
	CString sStageText = ssprintf( "%d%s Stage", PREFS->m_iCurrentStage, sNumberSuffix );
	m_textStage.SetText( sStageText );
	this->AddActor( &m_textStage );

	m_sprReady.Load( THEME->GetPathTo(GRAPHIC_READY) );
	m_sprReady.SetXY( CENTER_X, CENTER_Y );
	m_sprReady.SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
	this->AddActor( &m_sprReady );

	m_sprHereWeGo.Load( THEME->GetPathTo(GRAPHIC_HERE_WE_GO) );
	m_sprHereWeGo.SetXY( CENTER_X, CENTER_Y );
	m_sprHereWeGo.SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
	this->AddActor( &m_sprHereWeGo );

	m_sprCleared.Load( THEME->GetPathTo(GRAPHIC_CLEARED) );
	m_sprCleared.SetXY( CENTER_X, CENTER_Y );
	m_sprCleared.SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
	this->AddActor( &m_sprCleared );

	m_sprFailed.Load( THEME->GetPathTo(GRAPHIC_FAILED) );
	m_sprFailed.SetXY( CENTER_X, CENTER_Y );
	m_sprFailed.SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
	this->AddActor( &m_sprFailed );



	m_soundFail.Load(			THEME->GetPathTo(SOUND_FAILED) );
	m_announcerReady.Load(		THEME->GetPathTo(ANNOUNCER_READY) );
	m_announcerHereWeGo.Load(	THEME->GetPathTo(ANNOUNCER_HERE_WE_GO) );
	m_announcerGood.Load(		THEME->GetPathTo(ANNOUNCER_GOOD_COMMENT) );
	m_announcerBad.Load(		THEME->GetPathTo(ANNOUNCER_BAD_COMMENT) );
	m_announcerCleared.Load(	THEME->GetPathTo(ANNOUNCER_CLEARED) );
	m_announcerFailComment.Load(THEME->GetPathTo(ANNOUNCER_FAIL_COMMENT) );
	m_announcerGameOver.Load(	THEME->GetPathTo(ANNOUNCER_GAME_OVER) );

	m_soundAssistTick.Load(		THEME->GetPathTo(SOUND_ASSIST) );



	// Send some messages every have second to we can get the introduction rolling
	for( int i=0; i<30; i++ )
		this->SendWindowMessage( WindowMessage(SM_User+i), i/2.0f );



	m_StarWipe.SetClosed();
}

WindowDancing::~WindowDancing()
{
	RageLog( "WindowDancing::~WindowDancing()" );
	m_soundMusic.Stop();
}


void WindowDancing::Update( float fDeltaTime )
{
	//RageLog( "WindowDancing::Update(%f)", fDeltaTime );
	Window::Update( fDeltaTime );


	float fSongBeat, fBPS;
	float fPositionSeconds = m_soundMusic.GetPositionSeconds();
	
	m_pSong->GetBeatAndBPSFromElapsedTime( fPositionSeconds, fSongBeat, fBPS );

	

	
	//RageLog( "m_fOffsetInBeats = %f, m_fBeatsPerSecond = %f, m_Music.GetPositionSeconds = %f", m_fOffsetInBeats, m_fBeatsPerSecond, m_Music.GetPositionSeconds() );

	float fMaxBeatDifference = fBPS*MAX_SECONDS_CAN_BE_OFF_BY;

	switch( PREFS->m_GameMode )
	{
	case MODE_SINGLE:
	case MODE_SOLO:
	case MODE_DOUBLE:
		m_Player[PLAYER_1].Update( fDeltaTime, fSongBeat, fMaxBeatDifference );
		break;
	case MODE_VERSUS:
	case MODE_COUPLE:
		m_Player[PLAYER_1].Update( fDeltaTime, fSongBeat, fMaxBeatDifference );
		m_Player[PLAYER_2].Update( fDeltaTime, fSongBeat, fMaxBeatDifference );
		break;
	}

	// check for fail
	switch( PREFS->m_SongOptions.m_FailType )
	{
	case SongOptions::FAIL_ARCADE:
	case SongOptions::FAIL_END_OF_SONG:

		if( m_bHasFailed )
		{
			;	// if they have already failed, don't bother checking again
		}
		else
		{
			// check for both players fail
			switch( PREFS->m_GameMode )
			{
			case MODE_SINGLE:
			case MODE_SOLO:
			case MODE_DOUBLE:
				if( m_Player[PLAYER_1].GetLifePercentage() <= 0.3f )	// about to fail!
					m_Background.TurnDangerOn();
				else
					m_Background.TurnDangerOff();

				if( m_Player[PLAYER_1].GetLifePercentage() <= 0.0f )	// dead.  Let the message handler choose to fail or not
				{
					m_bHasFailed = true;
					WM->SendMessageToTopWindow( SM_LifeIs0, 0 );
				}
				break;
			case MODE_VERSUS:
			case MODE_COUPLE:
				if( m_Player[PLAYER_1].GetLifePercentage() <= 0.3f  &&  m_Player[PLAYER_2].GetLifePercentage() <= 0.3f )	// about to fail!
					m_Background.TurnDangerOn();
				else
					m_Background.TurnDangerOff();

				if( m_Player[PLAYER_1].GetLifePercentage() <= 0.0f  &&  m_Player[PLAYER_2].GetLifePercentage() <= 0.0f )	// dead.  Let the message handler choose to fail or not
				{
					m_bHasFailed = true;
					WM->SendMessageToTopWindow( SM_LifeIs0, 0 );
				}
				break;
			}		
		}		
		break;
		break;
	case SongOptions::FAIL_OFF:
		break;
	}


	switch( m_DancingState )
	{
	case STATE_DANCING:
		
		// Check for end of song
		if( m_DancingState == STATE_DANCING  &&
			m_soundMusic.GetLengthSeconds() - m_soundMusic.GetPositionSeconds() < 2 )
			this->SendWindowMessage( SM_SongEnded, 1 );
	
		// Check to see if it's time to play a dancing comment
		if( PREFS->m_GameOptions.m_bAnnouncer )
		{
			m_fTimeLeftBeforeDancingComment -= fDeltaTime;
			if( m_fTimeLeftBeforeDancingComment <= 0 )
			{
				m_fTimeLeftBeforeDancingComment = TIME_BETWEEN_DANCING_COMMENTS;	// reset for the next comment

				if( m_Background.IsDangerOn() )
					m_announcerBad.PlayRandom();
				else
					m_announcerGood.PlayRandom();
			}
		}
	}




	if( PREFS->m_SongOptions.m_AssistType == SongOptions::ASSIST_TICK )
	{
		// 
		// play assist ticks
		//
		// Sound cards have a latency between when a sample is Play()ed and when the sound
		// will start coming out the speaker.  Compensate for this by boosting
		// fPositionSeconds ahead
		fPositionSeconds += (SOUND->GetPlayLatency()+0.06f) * m_soundMusic.GetPlaybackRate();	// HACK:  Add 0.06 seconds to make them play a tiny bit earlier
		m_pSong->GetBeatAndBPSFromElapsedTime( fPositionSeconds, fSongBeat, fBPS );

		int iIndexNow = BeatToStepIndexNotRounded( fSongBeat );
		static int iIndexLastCrossed = 0;

		bool bAnyoneHasANote = false;	// set this to true if any player has a note at one of the indicies we crossed

		for( int i=iIndexLastCrossed+1; i<=iIndexNow; i++ )  // for each index we crossed since the last update
		{
			for( int p=0; p<NUM_PLAYERS; p++ )
			{
				if( !PREFS->IsPlayerEnabled( (PlayerNumber)p ) )
					continue;		// skip

				bAnyoneHasANote |= m_Player[p].IsThereANoteAtIndex( i );
			}
		}

		if( bAnyoneHasANote )
			m_soundAssistTick.PlayRandom();


		iIndexLastCrossed = iIndexNow;
	}

}


void WindowDancing::RenderPrimitives()
{
	Window::RenderPrimitives();
}


void WindowDancing::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	//RageLog( "WindowDancing::Input()" );


	float fSongBeat, fBPS;
	m_pSong->GetBeatAndBPSFromElapsedTime( m_soundMusic.GetPositionSeconds(), fSongBeat, fBPS );

	
	///////////////////////////
	// handle device inputs
	///////////////////////////
	if( DeviceI.device == DEVICE_KEYBOARD )
	{
		/*
		switch( DeviceI.button )
		{
		case DIK_PRIOR:
			m_pSong->SetBeatOffsetInSeconds( m_pSong->GetBeatOffsetInSeconds()+(1/fBPS) );
			m_pSong->SetChangedSinceLastSave();
			break;
		case DIK_NEXT:
			m_pSong->SetBeatOffsetInSeconds( m_pSong->GetBeatOffsetInSeconds()-(1/fBPS) );
			m_pSong->SetChangedSinceLastSave();
			break;
		case DIK_MINUS:
		case DIK_SUBTRACT:
			m_pSong->SetBeatOffsetInSeconds( m_pSong->GetBeatOffsetInSeconds()+0.01f );
			m_pSong->SetChangedSinceLastSave();
			return;
		case DIK_EQUALS:
		case DIK_ADD:
			m_pSong->SetBeatOffsetInSeconds( m_pSong->GetBeatOffsetInSeconds()-0.01f );
			m_pSong->SetChangedSinceLastSave();
			return;
		}
		*/
	}

	///////////////////////////
	// handle pad inputs
	///////////////////////////
	switch( PadI.button )
	{
	case BUTTON_BACK:
		if( m_DancingState == STATE_DANCING 
		 && !m_bHasFailed )
		{
			m_bHasFailed = true;
			WM->SendMessageToTopWindow( SM_BeginFailed, 0 );
		}
		else
			; // do not let user go back!
		return;
	case BUTTON_NEXT:
		// do nothing
		return;
	}

	float fMaxBeatsCanBeOffBy = MAX_SECONDS_CAN_BE_OFF_BY * fBPS;

	switch( PlayerI.player_no )
	{
		case PLAYER_1:	
			m_Player[PLAYER_1].HandlePlayerStep( fSongBeat, PlayerI.step, fMaxBeatsCanBeOffBy ); 
			return;
		case PLAYER_2:	
			m_Player[PLAYER_2].HandlePlayerStep( fSongBeat, PlayerI.step, fMaxBeatsCanBeOffBy ); 
			return;
	}

}

void WindowDancing::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
		// received while STATE_INTRO
	case SM_User+0:
		m_textStage.SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_textStage.SetZoomX( 2 );
		m_textStage.SetZoomY( 0 );
		m_textStage.BeginTweening( 0.5f );
		m_textStage.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_textStage.SetTweenZoom( 2 );
		break;
	case SM_User+1:
		break;
	case SM_User+2:
		m_soundMusic.Load( m_pSong->GetMusicPath() );
		m_textStage.BeginTweening( 0.5f );
		m_textStage.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_textStage.SetTweenZoomY( 0 );
		break;
	case SM_User+3:
		break;
	case SM_User+4:
		m_StarWipe.OpenWipingRight(SM_None);
		break;
	case SM_User+5:
		break;
	case SM_User+6:
		m_sprReady.StartFocusing();
		if( PREFS->m_GameOptions.m_bAnnouncer )
			m_announcerReady.PlayRandom();
		break;
	case SM_User+7:
		break;
	case SM_User+8:
		m_sprReady.StartBlurring();
		break;
	case SM_User+9:
		m_sprHereWeGo.StartFocusing();
		if( PREFS->m_GameOptions.m_bAnnouncer )
			m_announcerHereWeGo.PlayRandom();
		m_Background.SetDiffuseColor( D3DXCOLOR(0.8f,0.8f,0.8f,1) );
		m_soundMusic.Play();
		m_soundMusic.SetPlaybackRate( PREFS->m_SongOptions.m_fMusicRate );
		break;
	case SM_User+10:
		break;
	case SM_User+11:
		break;
	case SM_User+12:
		m_sprHereWeGo.StartBlurring();
		m_DancingState = STATE_DANCING;		// STATE CHANGE!  Now the user is allowed to press Back
		break;
	case SM_User+13:
		break;
	case SM_User+14:
		break;

		// received while STATE_DANCING
	case SM_LifeIs0:
		if( PREFS->m_SongOptions.m_FailType == SongOptions::FAIL_ARCADE )	// fail them now!
			this->SendWindowMessage( SM_BeginFailed, 0 );
		m_DancingState = STATE_OUTRO;
		break;
	case SM_SongEnded:
		if( m_DancingState == STATE_OUTRO )	// dancing already ended
			return;		// ignore

		if( m_bHasFailed )	// fail them
		{
			this->SendWindowMessage( SM_BeginFailed, 0 );
		}
		else	// cleared
		{
			m_StarWipe.CloseWipingRight( SM_ShowCleared );
			if( PREFS->m_GameOptions.m_bAnnouncer )
				m_announcerCleared.PlayRandom();	// crowd cheer
		}
		m_DancingState = STATE_OUTRO;
		break;

		// received while STATE_OUTRO
	case SM_ShowCleared:
		m_sprCleared.StartFocusing();
		WM->SendMessageToTopWindow( SM_HideCleared, 2.5 );
		break;
	case SM_HideCleared:
		m_sprCleared.StartBlurring();
		WM->SendMessageToTopWindow( SM_GoToResults, 1 );
		break;
	case SM_GoToResults:
		// send score summaries to the PREFS object so WindowResults can grab it.
		PREFS->scoreSummaryPlayer[PLAYER_1] = m_Player[PLAYER_1].GetScoreSummary();
		PREFS->scoreSummaryPlayer[PLAYER_2] = m_Player[PLAYER_2].GetScoreSummary();
		WM->SetNewWindow( new WindowMenuResults );
		break;


	case SM_BeginFailed:
		m_DancingState = STATE_OUTRO;
		m_soundMusic.Pause();
		m_StarWipe.CloseWipingRight( SM_None );
		this->SendWindowMessage( SM_ShowFailed, 0.2f );
		break;
	case SM_ShowFailed:
		if( PREFS->m_GameOptions.m_bAnnouncer )
			m_soundFail.PlayRandom();

		// make the background invisible so we don't waste mem bandwidth drawing it
		m_Background.BeginTweening( 1 );
		m_Background.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );

		m_sprFailed.SetZoom( 4 );
		m_sprFailed.BeginBlurredTweening( 0.8f, TWEEN_BIAS_END );
		m_sprFailed.SetTweenZoom( 0.5f );			// zoom out
		m_sprFailed.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0.7f) );	// and fade in
		m_sprFailed.BeginTweeningQueued( 0.3f );
		m_sprFailed.SetTweenZoom( 1.1f );			// bounce
		m_sprFailed.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0.7f) );	// and fade in
		m_sprFailed.BeginTweeningQueued( 0.2f );
		m_sprFailed.SetTweenZoom( 1.0f );			// come to rest
		m_sprFailed.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0.7f) );	// and fade in

		WM->SendMessageToTopWindow( SM_PlayFailComment, 1.5f );
		WM->SendMessageToTopWindow( SM_HideFailed, 3.0f );
		break;
	case SM_PlayFailComment:
		if( PREFS->m_GameOptions.m_bAnnouncer )
			m_announcerFailComment.PlayRandom();
		break;
	case SM_HideFailed:
		m_sprFailed.StopTweening();
		m_sprFailed.BeginTweening(1.0f);
		m_sprFailed.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );

		WM->SendMessageToTopWindow( SM_GoToSelectSong, 1.5f );
		break;
	case SM_GoToSelectSong:
		WM->SetNewWindow( new WindowMenuSelectMusic );
		break;
	}


}
