/*
-----------------------------------------------------------------------------
 File: WindowDancing.h

 Desc: The music plays, the notes scroll, and the Player is pressing buttons.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "Window.h"
#include "Sprite.h"
#include "TransitionStarWipe.h"
#include "TransitionFade.h"
#include "BitmapText.h"
#include "Player.h"
#include "RandomSample.h"
#include "RandomStream.h"
#include "FocusingSprite.h"
#include "RageMusic.h"
#include "MotionBlurSprite.h"
#include "Background.h"


class WindowDancing : public Window
{
public:
	WindowDancing();
	virtual ~WindowDancing();
	virtual void Update( float fDeltaTime );
	virtual void RenderPrimitives();
	virtual void Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI );
	virtual void HandleWindowMessage( WindowMessage SM );

	enum DancingState { 
		STATE_INTRO = 0, // not allowed to press Back
		STATE_DANCING,
		STATE_OUTRO,	// not allowed to press Back
		NUM_DANCING_STATES
	};


private:

	DancingState	m_DancingState;

	float			m_fTimeLeftBeforeDancingComment;	// this counter is only running while STATE_DANCING

	Song*		m_pSong;
	bool		m_bHasFailed;

	Background		m_Background;

	TransitionStarWipe	m_StarWipe;

	BitmapText		m_textStage;
	FocusingSprite	m_sprReady;
	FocusingSprite	m_sprHereWeGo;
	FocusingSprite	m_sprCleared;
	MotionBlurSprite	m_sprFailed;

	Player		m_Player[NUM_PLAYERS];

	RandomStream	m_soundFail;
	RandomStream	m_announcerReady;
	RandomStream	m_announcerHereWeGo;
	RandomSample	m_announcerGood;		// these are samples because we play them often
	RandomSample	m_announcerBad;			// these are samples because we play them often
	RandomStream	m_announcerCleared;
	RandomStream	m_announcerFailComment;
	RandomStream	m_announcerGameOver;

	RandomSample	m_soundAssistTick;

	RageSoundStream		m_soundMusic;


};



