#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowEdit.h

 Desc: The music plays, the notes scroll, and the Player is pressing buttons.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowEdit.h"
#include "PrefsManager.h"
#include "SongManager.h"
#include "WindowManager.h"
#include "WindowMenuSelectMusic.h"
#include "WindowMenuResults.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "WindowEditMenu.h"
#include "WindowEditOptions.h"
#include "GameConstants.h"


//
// Defines specific to GameWindowTitleMenu
//


const float MAX_SECONDS_CAN_BE_OFF_BY	=	0.20f;
const float GRAY_ARROW_Y					= ARROW_SIZE * 1.5;

const float DEBUG_X			=	SCREEN_LEFT + 10;
const float DEBUG_Y			=	CENTER_Y-100;

const float HELP_X				= SCREEN_RIGHT - 10;
const float HELP_Y				= DEBUG_Y;

const float MENU_WIDTH		=	110;
const float EDIT_CENTER_X	=	CENTER_X + ( MENU_WIDTH - 30 ) / 2;

const float EDIT_GRAY_Y		=	CENTER_Y - 0.5f * (float)ARROW_SIZE;

const WindowMessage SM_GoToPrevState		=	WindowMessage(SM_User+1);
const WindowMessage SM_GoToNextState		=	WindowMessage(SM_User+2);


WindowEdit::WindowEdit()
{
	RageLog( "WindowEdit::WindowEdit()" );

	m_fBeat = 0.0f;
	m_fTrailingBeat = m_fBeat;
	
	m_Style = GetStyle(PREFS->m_GameMode);

	m_PlayerOptions.m_fArrowScrollSpeed = 1;
	m_PlayerOptions.m_ColorType = PlayerOptions::COLOR_NOTE;
	m_PlayerOptions.m_bShowMeasureBars = true;

	m_Steps = *SONGS->m_pStepsPlayer[PLAYER_1];

	m_sprBackground.Load( THEME->GetPathTo( GRAPHIC_EDIT_BACKGROUND ) );
	m_sprBackground.StretchTo( CRect(SCREEN_LEFT,SCREEN_TOP,SCREEN_RIGHT,SCREEN_BOTTOM) );
	this->AddActor( &m_sprBackground );


	m_GranularityIndicator.SetXY( EDIT_CENTER_X, EDIT_GRAY_Y );
	m_GranularityIndicator.Load( m_Style );
	this->AddActor( &m_GranularityIndicator );

	m_GrayArrows.SetXY( EDIT_CENTER_X, EDIT_GRAY_Y );
	m_GrayArrows.Load( m_PlayerOptions, m_Style );
	this->AddActor( &m_GrayArrows );

	m_ColorArrowField.SetXY( EDIT_CENTER_X, EDIT_GRAY_Y );
	m_ColorArrowField.Load( m_Style, m_Steps, m_PlayerOptions, 3.5, 4.5 );
	this->AddActor( &m_ColorArrowField );

	m_Fade.SetClosed();
	this->AddActor( &m_Fade );


	m_soundChangeLine.Load( THEME->GetPathTo(SOUND_EDIT_CHANGE_LINE) );
	m_soundChangeSnap.Load( THEME->GetPathTo(SOUND_EDIT_CHANGE_SNAP) );


	m_Fade.OpenWipingRight();
}

WindowEdit::~WindowEdit()
{
	RageLog( "WindowEdit::~WindowEdit()" );
	m_soundMusic.Stop();
}


void WindowEdit::Update( float fDeltaTime )
{
	//RageLog( "WindowEdit::Update(%f)", fDeltaTime );
	Window::Update( fDeltaTime );

	if( m_fTrailingBeat != m_fBeat )
	{
		float fOffset = m_fBeat-m_fTrailingBeat;
		float fSign = fOffset/fabsf(fOffset);

		float fBeatsToMove = fDeltaTime * 10;

		if( fabsf(fOffset) < fBeatsToMove )
			m_fTrailingBeat = m_fBeat;
		else
			m_fTrailingBeat += fBeatsToMove * fSign;
	}

//	float fSongBeat, fBPS;
///	float fPositionSeconds = m_soundMusic.GetPositionSeconds();

//	fPositionSeconds += 0.08f;	// HACK:  The assist ticks are playing too late, so make them play a tiny bit earlier
//	m_pSong->GetBeatAndBPSFromElapsedTime( fPositionSeconds, fSongBeat, fBPS );
//	RageLog( "fPositionSeconds = %f, fSongBeat = %f, fBPS = %f", fPositionSeconds, fSongBeat, fBPS );


	m_ColorArrowField.Update( fDeltaTime, m_fTrailingBeat );

	int iIndexNow = BeatToStepIndexNotRounded( m_fBeat );	

}


void WindowEdit::RenderPrimitives()
{
	Window::RenderPrimitives();
}


void WindowEdit::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	//RageLog( "WindowEdit::Input()" );


	///////////////////////////
	// handle pad inputs
	///////////////////////////

	if( DeviceI.device == DEVICE_KEYBOARD )
	{
		switch( DeviceI.button )
		{
		case DIK_1:
		case DIK_2:
		case DIK_3:
		case DIK_4:
		case DIK_5:
		case DIK_6:
		case DIK_7:
		case DIK_8:
		case DIK_9:
			{
			int iColIndex = DeviceI.button - DIK_1;
			if( iColIndex < m_Style.m_iNumColumns )	// this button is in the range of columns for this style
			{
				int iStepIndex = BeatToStepIndex( m_fBeat );
				m_ColorArrowField.m_TapSteps[iStepIndex] ^= m_Style.m_ColumnToTapStep[iColIndex];	// XOR to add or remove note
			}
			}
			break;
		case DIK_ESCAPE:
			WM->AddWindowToTop( new WindowEditOptions );
			break;
		case DIK_UP:
			m_fBeat -= NOTE_TYPE_TO_BEAT[m_GranularityIndicator.GetSnapMode()];
			if( m_fBeat < 0.0f ) 
				m_fBeat = 0;
			m_soundChangeLine.PlayRandom();
			break;
		case DIK_DOWN:
			m_fBeat += NOTE_TYPE_TO_BEAT[m_GranularityIndicator.GetSnapMode()];
			if( m_fBeat > MAX_BEATS-1 ) 
				m_fBeat = MAX_BEATS-1;
			m_soundChangeLine.PlayRandom();
			break;
		case DIK_PRIOR:
			m_GranularityIndicator.PrevSnapMode();
			OnSnapModeChange();
			break;
		case DIK_NEXT:
			m_GranularityIndicator.NextSnapMode();
			OnSnapModeChange();
			break;
		}
	}
}

void WindowEdit::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_GoToPrevState:
		WM->SetNewWindow( new WindowEditMenu );
		break;
	case SM_GoToNextState:
		WM->SetNewWindow( new WindowEditMenu );
		break;
	}


}

void WindowEdit::OnSnapModeChange()
{
	m_soundChangeSnap.PlayRandom();
			
	NoteType nt = m_GranularityIndicator.GetSnapMode();
	int iStepIndex = BeatToStepIndex( m_fBeat );
	int iElementsPerNoteType = BeatToStepIndex( NOTE_TYPE_TO_BEAT[nt] );
	int iStepIndexHangover = iStepIndex % iElementsPerNoteType;
	m_fBeat -= StepIndexToBeat( iStepIndexHangover );
}
