#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowEditMenu.h

 Desc: The main title screen and menu.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowEditMenu.h"
#include "SongManager.h"
#include "WindowManager.h"
#include "WindowTitleMenu.h"
#include "WindowEdit.h"
#include "ScreenDimensions.h"
#include "RageUtil.h"
#include "ThemeManager.h"
#include "WindowPrompt.h"


//
// Defines specific to WindowEditMenu
//

const float SONG_X		=	CENTER_X;
const float SONG_Y		=	CENTER_Y-100;

const float GAME_MODE_X	=	CENTER_X;
const float GAME_MODE_Y	=	CENTER_Y-050;

const float STEPS_X		=	CENTER_X;
const float STEPS_Y		=	CENTER_Y-000;

const float EXPLANATION_X	=	CENTER_X;
const float EXPLANATION_Y	=	CENTER_Y+100;


const WindowMessage SM_GoToPrevState		=	WindowMessage(SM_User+1);
const WindowMessage SM_GoToNextState		=	WindowMessage(SM_User+2);


WindowEditMenu::WindowEditMenu() : 
	WindowMenu( THEME->GetPathTo(GRAPHIC_EDIT_BACKGROUND), 
				THEME->GetPathTo(GRAPHIC_SELECT_MUSIC_TOP_EDGE),
				ssprintf("%s %s change music    NEXT to continue", CString(char(1)), CString(char(2)) )
				)
{
	RageLog( "WindowEditMenu::WindowEditMenu()" );


	// data structures
	m_SelectedRow = ROW_SONG;


	m_pSongs.Copy( SONGS->m_pSongs );
	SortSongPointerArrayByTitle( m_pSongs );
	m_iSelectedSong = 0;

	// select the current song
	for( int i=0; i<m_pSongs.GetSize(); i++ )
	{
		if( m_pSongs[i] == SONGS->m_pCurSong )
		{
			m_iSelectedSong = i;
			break;
		}
	}


	m_iSelectedGameMode = MODE_SINGLE;


	GetSelectedSong()->GetStepsThatMatchGameMode( GetSelectedGameMode(), m_pSteps );
	SortStepsArrayByDifficultyClass( m_pSteps );
	m_pSteps.Add( NULL );		// marker for "(NEW)"
	m_iSelectedSteps = 0;

	// select the current steps
	for( i=0; i<m_pSteps.GetSize(); i++ )
	{
		if( m_pSteps[i] == SONGS->m_pStepsPlayer[PLAYER_1] )
		{
			m_iSelectedSteps = i;
			break;
		}
	}



	m_textSong.Load( THEME->GetPathTo(FONT_OUTLINE) );
	m_textSong.SetXY( SONG_X, SONG_Y );
	m_textSong.SetDiffuseColor( D3DXCOLOR(0.7f,0.7f,0.7f,1) );
	this->AddActor( &m_textSong );

	m_textGameMode.Load( THEME->GetPathTo(FONT_OUTLINE) );
	m_textGameMode.SetXY( GAME_MODE_X, GAME_MODE_Y );
	m_textGameMode.SetDiffuseColor( D3DXCOLOR(0.7f,0.7f,0.7f,1) );
	this->AddActor( &m_textGameMode );

	m_textSteps.Load( THEME->GetPathTo(FONT_OUTLINE) );
	m_textSteps.SetXY( STEPS_X, STEPS_Y );
	m_textSteps.SetDiffuseColor( D3DXCOLOR(0.7f,0.7f,0.7f,1) );
	this->AddActor( &m_textSteps );


	AfterRowChange();
	OnSongChange();


	m_textExplanation.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textExplanation.SetXY( EXPLANATION_X, EXPLANATION_Y );
	m_textExplanation.SetText( ssprintf("This mode will allow you to\nedit an existing or\n create a new steps pattern.") );
	m_textExplanation.SetZoom( 0.7f );
	this->AddActor( &m_textExplanation );

	m_Fade.SetOpened();
	this->AddActor( &m_Fade);


	m_soundChangeMusic.Load( THEME->GetPathTo(SOUND_SWITCH_MUSIC) );
	m_soundSelect.Load( THEME->GetPathTo(SOUND_SELECT) );

	MUSIC->Load( THEME->GetPathTo(SOUND_TRAINING_MUSIC) );
	MUSIC->Play( true );

}


WindowEditMenu::~WindowEditMenu()
{
	RageLog( "WindowEditMenu::~WindowEditMenu()" );
}


void WindowEditMenu::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	RageLog( "WindowEditMenu::Input()" );

	
	Window::Input( DeviceI, PadI, PlayerI );
}

void WindowEditMenu::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_GoToPrevState:
		WM->SetNewWindow( new WindowTitleMenu );
		break;
	case SM_GoToNextState:
		WM->SetNewWindow( new WindowEdit );
		break;
	}
}
	
void WindowEditMenu::BeforeRowChange()
{
	m_textSong.SetEffectNone();
	m_textGameMode.SetEffectNone();
	m_textSteps.SetEffectNone();
}

void WindowEditMenu::AfterRowChange()
{
	switch( m_SelectedRow )
	{
	case ROW_SONG:		m_textSong.SetEffectGlowing();		break;
	case ROW_GAME_MODE:	m_textGameMode.SetEffectGlowing();	break;
	case ROW_STEPS:		m_textSteps.SetEffectGlowing();		break;
	default:		ASSERT(false);
	}
}

void WindowEditMenu::OnSongChange()
{
	m_iSelectedSong = clamp( m_iSelectedSong, 0, m_pSongs.GetSize()-1 );

	m_textSong.SetText( GetSelectedSong()->GetTitle() );

	OnGameModeChange();
}

void WindowEditMenu::OnGameModeChange()
{
	m_iSelectedGameMode = clamp( m_iSelectedGameMode, 0, NUM_GAMEMODES-1 );

	switch( GetSelectedGameMode() )
	{
	case MODE_SINGLE:
		m_textGameMode.SetText( "Single" );
		break;
	case MODE_VERSUS:
		m_textGameMode.SetText( "Versus" );
		break;
	case MODE_DOUBLE:
		m_textGameMode.SetText( "Double" );
		break;
	case MODE_COUPLE:
		m_textGameMode.SetText( "Couple" );
		break;
	case MODE_SOLO:
		m_textGameMode.SetText( "Solo" );
		break;
	default:
		ASSERT(false);
	}

	m_pSteps.RemoveAll();
	GetSelectedSong()->GetStepsThatMatchGameMode( GetSelectedGameMode(), m_pSteps );
	SortStepsArrayByDifficultyClass( m_pSteps );
	m_pSteps.Add( NULL );		// marker for "(NEW)"
	m_iSelectedSteps = 0;


	OnStepsChange();
}

void WindowEditMenu::OnStepsChange()
{
	m_iSelectedSteps = clamp( m_iSelectedSteps, 0, NUM_GAMEMODES-1 );

	if( GetSelectedSteps() == NULL )
		m_textSteps.SetText( "(NEW)" );
	else
		m_textSteps.SetText( GetSelectedSteps()->m_sDescription );

}

void WindowEditMenu::Up()
{
	if( m_SelectedRow == 0 )	// can't go up any further
		return;	

	BeforeRowChange();
	m_SelectedRow = SelectedRow(m_SelectedRow-1);
	AfterRowChange();
}

void WindowEditMenu::Down()
{
	if( m_SelectedRow == NUM_ROWS-1 )	// can't go down any further
		return;	

	BeforeRowChange();
	m_SelectedRow = SelectedRow(m_SelectedRow+1);
	AfterRowChange();
}

void WindowEditMenu::Left()
{
	switch( m_SelectedRow )
	{
	case ROW_SONG:
		if( m_iSelectedSong == 0 )	// can't go left any further
			return;
		m_iSelectedSong--;
		OnSongChange();
		break;
	case ROW_GAME_MODE:
		if( m_iSelectedGameMode == 0 )	// can't go left any further
			return;
		m_iSelectedGameMode--;
		OnGameModeChange();
		break;
	case ROW_STEPS:
		if( m_iSelectedSteps == 0 )	// can't go left any further
			return;
		m_iSelectedSteps--;
		OnStepsChange();
		break;
	default:
		ASSERT(false);
	}
}

void WindowEditMenu::Right()
{
	switch( m_SelectedRow )
	{
	case ROW_SONG:
		if( m_iSelectedSong == m_pSongs.GetSize()-1 )	// can't go right any further
			return;
		m_iSelectedSong++;
		OnSongChange();
		break;
	case ROW_GAME_MODE:
		if( m_iSelectedGameMode == NUM_GAMEMODES-1 )	// can't go right any further
			return;
		m_iSelectedGameMode++;
		OnGameModeChange();
		break;
	case ROW_STEPS:
		if( m_iSelectedSteps == m_pSteps.GetSize()-1 )	// can't go right any further
			return;
		m_iSelectedSteps++;
		OnStepsChange();
		break;
	default:
		ASSERT(false);
	}
}

void WindowEditMenu::Next()
{
	WindowMenu::Next();

	MUSIC->Stop();

	SONGS->m_pCurSong = GetSelectedSong();
	PREFS->m_GameMode = GetSelectedGameMode();
	SONGS->m_pStepsPlayer[PLAYER_1] = GetSelectedSteps();

	m_soundSelect.PlayRandom();
	WindowMenu::TweenOffScreen();
	m_Fade.CloseWipingRight( SM_GoToNextState );
}

void WindowEditMenu::Back()
{	
	WindowMenu::Back();


	MUSIC->Stop();

	WindowMenu::TweenOffScreen();
	m_Fade.CloseWipingLeft( SM_GoToPrevState );
}
