/*
-----------------------------------------------------------------------------
 File: WindowEditMenu.h

 Desc: The main title screen and menu.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowMenu.h"
#include "Sprite.h"
#include "ColorArrow.h"
#include "BitmapText.h"
#include "TransitionFadeWipeWithLogo.h"
#include "TransitionFade.h"
#include "RandomSample.h"
#include "Banner.h"
#include "TextBanner.h"
#include "RandomSample.h"
#include "TransitionInvisible.h"



class WindowEditMenu : public WindowMenu
{
public:
	WindowEditMenu();
	virtual ~WindowEditMenu();

	virtual void Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI );
	virtual void HandleWindowMessage( WindowMessage SM );

private:
	void BeforeRowChange();
	void AfterRowChange();

	void OnSongChange();
	void OnGameModeChange();
	void OnStepsChange();

	Song* GetSelectedSong() { return m_pSongs[m_iSelectedSong]; };
	GameMode GetSelectedGameMode() { return (GameMode)m_iSelectedGameMode; };
	Steps* GetSelectedSteps() { return m_pSteps[m_iSelectedSteps]; };
	
	void Up();
	void Down();
	void Left();
	void Right();
	void Back();
	void Next();


	enum SelectedRow { ROW_SONG, ROW_GAME_MODE, ROW_STEPS, NUM_ROWS };
	SelectedRow m_SelectedRow;


	CArray<Song*, Song*> m_pSongs;
	int m_iSelectedSong;		// index into m_pSongs
	BitmapText m_textSong;

	int m_iSelectedGameMode;	// index into enum GameMode
	BitmapText m_textGameMode;

	CArray<Steps*, Steps*> m_pSteps;
	int m_iSelectedSteps;		// index into m_pSteps
	BitmapText m_textSteps;


	BitmapText m_textExplanation;

	TransitionFade		m_Fade;

	RandomSample m_soundChangeMusic;
	RandomSample m_soundSelect;

};



