#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowEditOptions.h

 Desc: Area for testing.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/


#include "WindowEditOptions.h"
#include "WindowManager.h"
#include "RageMusic.h"
#include "WindowTitleMenu.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"


CString g_sOptionsText[WindowEditOptions::NUM_OPTIONS] = { "Return To Edit", "Finished Editing" };

const float OPTIONS_START_Y = 200;
const float OPTIONS_GAP_Y = 50;


WindowEditOptions::WindowEditOptions()
{
	m_EditOption = EditOptions(0);

	m_Fade.SetTransitionTime( 0.5f );
	m_Fade.SetColor( D3DXCOLOR(0,0,0,0.7f) );
	m_Fade.SetOpened();
	m_Fade.CloseWipingRight();
	m_Fade.SetZ(-2);
	this->AddActor( &m_Fade );

	for( int i=0; i<NUM_OPTIONS; i++ )
	{
		m_textOptions[i].Load( THEME->GetPathTo(FONT_OUTLINE) );
		m_textOptions[i].SetText( g_sOptionsText[i] );
		m_textOptions[i].SetXY( CENTER_X, OPTIONS_START_Y + OPTIONS_GAP_Y*i );
		m_textOptions[i].SetZ(-2);
		this->AddActor( &m_textOptions[i] );
	}
	
	AfterChange();

	m_soundSelect.Load( THEME->GetPathTo(SOUND_SELECT) );
}

void WindowEditOptions::Update( float fDeltaTime )
{
	Window::Update( fDeltaTime );
}

void WindowEditOptions::RenderPrimitives()
{
	Window::RenderPrimitives();
}

void WindowEditOptions::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	if( m_Fade.IsClosing() )
		return;

	Window::Input( DeviceI, PadI, PlayerI );
}

void WindowEditOptions::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_DoneClosingWipingLeft:
		break;
	case SM_DoneClosingWipingRight:
		break;
	case SM_DoneOpeningWipingLeft:
		break;
	case SM_DoneOpeningWipingRight:
		WM->PopTopWindow();
		break;
	}
}

void WindowEditOptions::BeforeChange()
{
	for( int i=0; i<NUM_OPTIONS; i++ )
	{
		m_textOptions[i].SetEffectNone();
	}
}

void WindowEditOptions::AfterChange()
{
	m_textOptions[m_EditOption].SetEffectGlowing();
}

void WindowEditOptions::Left()
{

}

void WindowEditOptions::Right()
{

}

void WindowEditOptions::Up()
{
	if( m_EditOption == 0 )	// can't go up any further
		return;

	BeforeChange();
	m_EditOption = EditOptions(m_EditOption-1);
	AfterChange();
}

void WindowEditOptions::Down()
{
	if( m_EditOption == NUM_OPTIONS-1 )	// can't go down any further
		return;

	BeforeChange();
	m_EditOption = EditOptions(m_EditOption+1);
	AfterChange();
}

void WindowEditOptions::Next()
{
	m_Fade.OpenWipingRight( SM_DoneOpeningWipingRight );

	m_soundSelect.PlayRandom();
}

void WindowEditOptions::Back()
{

}
