#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowGameOptions.cpp

 Desc: Select a song.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowGameOptions.h"
#include <assert.h>
#include "RageTextureManager.h"
#include "RageUtil.h"
#include "RageMusic.h"
#include "WindowManager.h"
#include "PrefsManager.h"
#include "WindowOptions.h"
#include "WindowTitleMenu.h"
#include "ScreenDimensions.h"
#include "StepMania.h"
#include "ThemeManager.h"



OptionLine g_GameOptionsLines[] = {
	{ "Mode",			2, {"Fullscreen", "Windowed"} },
	{ "Resolution",		7, {"320", "400", "512", "640", "800", "1024", "1280" } },
	{ "Display",		2, {"16bit", "32bit"} },
	{ "Texture",		2, {"16bit", "32bit"} },
	{ "Tex Filter",		2, {"OFF","ON"} },
	{ "Shadows",		2, {"OFF","ON (for faster computers)"} },
	{ "Ignore Axes",	2, {"OFF","ON (for DirectPad and NTPad)"} },
	{ "Show FPS",		2, {"OFF","ON"} },
	{ "Movie Vis",		2, {"OFF","ON (for faster computers)"} },
	{ "SkipCaution",	2, {"OFF","ON"} },
	{ "Announcer",		2, {"OFF","ON"} },
};
enum GAMEOPTION {
	GO_WINDOWED = 0,
	GO_RESOLUTION,
	GO_DISPLAY_COLOR,
	GO_TEXTURE_COLOR,
	GO_TEXTURE_FILTER,
	GO_SHADOWS,
	GO_IGNORE_AXES,
	GO_SHOWFPS,
	GO_VISUALS,
	GO_SKIP_CAUTION,
	GO_ANNOUNCER,
	NUM_GAME_OPTIONS_LINES
};

//const int NUM_OPTIONS_LINES = sizeof(g_GameOptionsLines)/sizeof(OptionLine);

WindowGameOptions::WindowGameOptions() :
	WindowOptions(
		THEME->GetPathTo(GRAPHIC_PLAYER_OPTIONS_BACKGROUND),
		THEME->GetPathTo(GRAPHIC_PLAYER_OPTIONS_TOP_EDGE)
		)
{
	RageLog( "WindowGameOptions::WindowGameOptions()" );

	Init( 
		INPUTMODE_BOTH, 
		g_GameOptionsLines, 
		NUM_GAME_OPTIONS_LINES
		);
}

void WindowGameOptions::ImportOptions()
{
	GameOptions &go = PREFS->m_GameOptions;

	m_Options[0][GO_WINDOWED]		= (go.m_bWindowed ? 1:0 );
	switch( go.m_iResolution )
	{
	case 320:		m_Options[0][GO_RESOLUTION] = 0;		break;
	case 400:		m_Options[0][GO_RESOLUTION] = 1;		break;
	case 512:		m_Options[0][GO_RESOLUTION] = 2;		break;
	case 640:		m_Options[0][GO_RESOLUTION] = 3;		break;
	case 800:		m_Options[0][GO_RESOLUTION] = 4;		break;
	case 1024:		m_Options[0][GO_RESOLUTION] = 5;		break;
	case 1280:		m_Options[0][GO_RESOLUTION] = 6;		break;
	default:		m_Options[0][GO_RESOLUTION] = 3;		break;
	}
	switch( go.m_iDisplayColor )
	{
	case 16:		m_Options[0][GO_DISPLAY_COLOR] = 0;		break;
	case 32:		m_Options[0][GO_DISPLAY_COLOR] = 1;		break;
	}
	switch( go.m_iTextureColor )
	{
	case 16:		m_Options[0][GO_TEXTURE_COLOR] = 0;		break;
	case 32:		m_Options[0][GO_TEXTURE_COLOR] = 1;		break;
	}
	m_Options[0][GO_TEXTURE_FILTER]	= (go.m_bFilterTextures ? 1:0 );
	m_Options[0][GO_SHADOWS]		= (go.m_bShadows ? 1:0 );
	m_Options[0][GO_IGNORE_AXES]	= (go.m_bIgnoreJoyAxes ? 1:0 );
	m_Options[0][GO_SHOWFPS]		= (go.m_bShowFPS ? 1:0 );
	m_Options[0][GO_VISUALS]		= (go.m_bUseRandomVis? 1:0 );
	m_Options[0][GO_SKIP_CAUTION]	= (go.m_bSkipCaution ? 1:0 );
	m_Options[0][GO_ANNOUNCER]		= (go.m_bAnnouncer? 1:0 );
}

void WindowGameOptions::ExportOptions()
{
	GameOptions &go = PREFS->m_GameOptions;

	go.m_bWindowed		= (m_Options[0][GO_WINDOWED] == 1);
	switch( m_Options[0][GO_RESOLUTION] )
	{
	case 0:		go.m_iResolution = 320;		break;
	case 1:		go.m_iResolution = 400;		break;
	case 2:		go.m_iResolution = 512;		break;
	case 3:		go.m_iResolution = 640;		break;
	case 4:		go.m_iResolution = 800;		break;
	case 5:		go.m_iResolution = 1024;	break;
	case 6:		go.m_iResolution = 1280;	break;
	}
	switch( m_Options[0][GO_DISPLAY_COLOR] )
	{
	case 0:		go.m_iDisplayColor = 16;	break;
	case 1:		go.m_iDisplayColor = 32;	break;
	}
	switch( m_Options[0][GO_TEXTURE_COLOR] )
	{
	case 0:		go.m_iTextureColor = 16;	break;
	case 1:		go.m_iTextureColor = 32;	break;
	}
	go.m_bFilterTextures= (m_Options[0][GO_TEXTURE_FILTER] == 1);
	go.m_bShadows		= (m_Options[0][GO_SHADOWS] == 1);
	go.m_bIgnoreJoyAxes	= (m_Options[0][GO_IGNORE_AXES] == 1);
	go.m_bShowFPS		= (m_Options[0][GO_SHOWFPS] == 1);
	go.m_bUseRandomVis	= (m_Options[0][GO_VISUALS] == 1);
	go.m_bSkipCaution	= (m_Options[0][GO_SKIP_CAUTION] == 1);
	go.m_bAnnouncer		= (m_Options[0][GO_ANNOUNCER] == 1);

	// put the options into effect
	SwitchDisplayMode(); 
}

void WindowGameOptions::GoToPrevState()
{
	WM->SetNewWindow( new WindowTitleMenu );
}

void WindowGameOptions::GoToNextState()
{
	WM->SetNewWindow( new WindowTitleMenu );
}





