#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowIntroCovers.h

 Desc: Intro with some flash graphics.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowIntroCovers.h"
#include "WindowManager.h"
#include "RageMusic.h"
#include "WindowTitleMenu.h"
#include "ScreenDimensions.h"


const CString TEXTURE_CD_COVER[NUM_CD_COVERS] = {
	"Themes\\Default\\Textures\\CD Cover - DDR.png",
	"Themes\\Default\\Textures\\CD Cover - DDR 2nd Mix.png",
	"Themes\\Default\\Textures\\CD Cover - DDR Club Append Vol1.png",
	"Themes\\Default\\Textures\\CD Cover - Dancing Stage.png",
	"Themes\\Default\\Textures\\CD Cover - DDR 3rd Mix.png",
	"Themes\\Default\\Textures\\CD Cover - DDR Club Append Vol2.png",
	"Themes\\Default\\Textures\\CD Cover - DDR Disney Rave.png",
	"Themes\\Default\\Textures\\CD Cover - Dreams Come True.png",
	"Themes\\Default\\Textures\\CD Cover - DDR Best Hits.png"
};

const float COVER_WIDTH			=	256;
const float HALF_COVER_WIDTH	=	(COVER_WIDTH/2);


const float FLOAT_LEFT_POINT_X	=	(SCREEN_WIDTH*3/10);
const float FLOAT_MID_POINT_X	=	(SCREEN_WIDTH*5/10);
const float FLOAT_RIGHT_POINT_X	=	(SCREEN_WIDTH*7/10);
const float FLOAT_TOP_POINT_Y	=	(SCREEN_HEIGHT*3/10);
const float FLOAT_MID_POINT_Y	=	(SCREEN_HEIGHT*5/10);
const float FLOAT_BOT_POINT_Y	=	(SCREEN_HEIGHT*7/10);


const float GRID_LEFT_POINT_X	=	(CENTER_X-HALF_COVER_WIDTH);
const float GRID_MID_POINT_X	=	(CENTER_X);
const float GRID_RIGHT_POINT_X	=	(CENTER_X+HALF_COVER_WIDTH);
const float GRID_TOP_POINT_Y	=	(CENTER_Y-HALF_COVER_WIDTH);
const float GRID_MID_POINT_Y	=	(CENTER_Y);
const float GRID_BOT_POINT_Y	=	(CENTER_Y+HALF_COVER_WIDTH);

const CString SOUND_BG_MUSIC	=		"Sounds\\CD Cover music.mp3";


const WindowMessage SM_StepsHaveEnded	=	WindowMessage(SM_User-1);
const WindowMessage SM_GoToTitleMenu	=	WindowMessage(SM_User-2);
const WindowMessage SM_GoToResults		=	WindowMessage(SM_User-3);



WindowIntroCovers::WindowIntroCovers()
{
	m_sprBG.Load( "Textures\\dancers.avi" );
	m_sprBG.StretchTo( CRect(0,480,640,0) );
	m_sprBG.SetDiffuseColor( D3DXCOLOR(1,1,1,0) );

	for( int i=0; i<NUM_CD_COVERS; i++ ) {
		m_CDCover[i].Load( TEXTURE_CD_COVER[i] );
		m_CDCover[i].SetXY( -1000, -1000 );
		m_CDCover[i].SetZoom( 2 );
	}

	for( i=0; i<20*2; i++ )
		WM->SendMessageToTopWindow( WindowMessage(SM_User+i),	i/2.0f );

	m_TransStars.SetOpened();

	m_Starburst.SetColor( D3DXCOLOR(1,1,1,1) );
	m_Starburst.SetOpened();

	m_Fade.SetColor( D3DXCOLOR(1,1,1,1) );
	m_Fade.SetOpened();

//	MUSIC->Load( SOUND_BG_MUSIC );
//	MUSIC->Play();
}

WindowIntroCovers::~WindowIntroCovers()
{

}

void WindowIntroCovers::Update( float fDeltaTime )
{
	m_sprBG.Update( fDeltaTime );
	for( int i=0; i<NUM_CD_COVERS; i++ )
		m_CDCover[i].Update( fDeltaTime );

	m_TransStars.Update( fDeltaTime );
	m_Starburst.Update( fDeltaTime );
	m_Fade.Update( fDeltaTime );
}

void WindowIntroCovers::RenderPrimitives()
{
	m_sprBG.Draw();
	for( int i=0; i<NUM_CD_COVERS; i++ )
		m_CDCover[i].Draw();

	m_Starburst.Draw();
	m_Fade.Draw();
	m_TransStars.Draw();
}

void WindowIntroCovers::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	if( m_TransStars.IsClosing() ) 
		return;

	Window::Input( DeviceI, PadI, PlayerI );
}

void WindowIntroCovers::Next()
{
	m_TransStars.CloseWipingRight( SM_GoToTitleMenu );
}

void WindowIntroCovers::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_DoneClosingWipingLeft:
		break;
	case SM_GoToTitleMenu:
		MUSIC->Stop();
		WM->SetNewWindow( new WindowTitleMenu );
		break;
	case SM_DoneOpeningWipingLeft:
		break;
	case SM_DoneOpeningWipingRight:
		break;
	case SM_User+1:
		m_Starburst.SetOpened();
		m_Starburst.SetTransitionTime( 1.0f );
		m_Starburst.CloseWipingRight();
		break;
	case SM_User+2:
		break;
	case SM_User+3:
		m_Starburst.SetOpened();
		m_sprBG.SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_Fade.OpenWipingRight();
		break;
	case SM_User+4:
		break;
		/*
	case SM_User+5:
		m_CDCover[0].SetXY( FLOAT_LEFT_POINT_X, FLOAT_BOT_POINT_Y );
		m_CDCover[0].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[0].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+6:
		m_CDCover[0].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );
		m_CDCover[1].SetXY( FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y );
		m_CDCover[1].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[1].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+7:
		m_CDCover[1].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );
		m_CDCover[2].SetXY( FLOAT_RIGHT_POINT_X, FLOAT_TOP_POINT_Y );
		m_CDCover[2].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[2].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+8:
		m_CDCover[2].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );
		m_CDCover[3].SetXY( FLOAT_RIGHT_POINT_X, FLOAT_BOT_POINT_Y );
		m_CDCover[3].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[3].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+9:
		m_CDCover[3].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );

		m_CDCover[4].SetXY( FLOAT_LEFT_POINT_X, FLOAT_BOT_POINT_Y );
		m_CDCover[4].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[4].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+10:
		m_CDCover[4].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );
		m_CDCover[5].SetXY( FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y );
		m_CDCover[5].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[5].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+11:
		m_CDCover[5].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );
		m_CDCover[6].SetXY( FLOAT_RIGHT_POINT_X, FLOAT_TOP_POINT_Y );
		m_CDCover[6].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[6].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+12:
		m_CDCover[6].TweenTo( 1.0f, FLOAT_RIGHT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );
		m_CDCover[7].SetXY( FLOAT_RIGHT_POINT_X, FLOAT_BOT_POINT_Y );
		m_CDCover[7].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[7].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+13:
		m_CDCover[7].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_BOT_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );

		m_CDCover[8].SetXY( FLOAT_LEFT_POINT_X, FLOAT_BOT_POINT_Y );
		m_CDCover[8].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[8].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+14:
		m_CDCover[8].TweenTo( 1.0f, FLOAT_LEFT_POINT_X, FLOAT_TOP_POINT_Y, 2.0f, 0.0f, D3DXCOLOR(1,1,1,0) );

		break;

	case SM_User+15:
		m_CDCover[0].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[0].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[0].TweenTo( 1.0f, GRID_LEFT_POINT_X, GRID_TOP_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+16:
		m_CDCover[1].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[1].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[1].TweenTo( 1.0f, GRID_MID_POINT_X, GRID_TOP_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+17:
		m_CDCover[2].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[2].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[2].TweenTo( 1.0f, GRID_RIGHT_POINT_X, GRID_TOP_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+18:
		m_CDCover[3].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[3].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[3].TweenTo( 1.0f, GRID_LEFT_POINT_X, GRID_MID_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+19:
		m_CDCover[4].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[4].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[4].TweenTo( 1.0f, GRID_MID_POINT_X, GRID_MID_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+20:
		m_CDCover[5].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[5].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[5].TweenTo( 1.0f, GRID_RIGHT_POINT_X, GRID_MID_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+21:
		m_CDCover[6].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[6].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[6].TweenTo( 1.0f, GRID_LEFT_POINT_X, GRID_BOT_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+22:
		m_CDCover[7].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[7].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[7].TweenTo( 1.0f, GRID_MID_POINT_X, GRID_BOT_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;
	case SM_User+23:
		m_CDCover[8].SetXY( CENTER_X, CENTER_Y );
		m_CDCover[8].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_CDCover[8].TweenTo( 1.0f, GRID_RIGHT_POINT_X, GRID_BOT_POINT_Y, 1.0f, D3DX_PI*2, D3DXCOLOR(1,1,1,1) );
		break;

	case SM_User+25:
		int i;
		for( i=0; i<NUM_CD_COVERS; i++ )
			m_CDCover[i].SetEffectVibrating();
		break;
	case SM_User+28:
		for( i=0; i<NUM_CD_COVERS; i++ ) {
			m_CDCover[i].SetEffectNone();
			m_CDCover[i].TweenTo( 1.0f, 
								  m_CDCover[i].GetX(), 
								  m_CDCover[i].GetY(),
								  3.0f, 
								  D3DX_PI*2, 
								  D3DXCOLOR(1,1,1,0) 
								  );
		}
		break;
	case SM_User+30:
		m_sprBG.BeginTweening( 2.0f );
		m_sprBG.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );
		break;
	case SM_User+34:
		if( !m_TransStars.IsClosing() )
			WM->SetNewWindow( new WindowTitleMenu );
		break;
*/
	}
}
