/*
-----------------------------------------------------------------------------
 File: WindowManager.h

 Desc: Manages the game windows.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#ifndef _WindowManager_H_
#define _WindowManager_H_


#include "RageInput.h"
#include "PadInput.h"
#include "Song.h"
#include "Steps.h"
#include "Window.h"
#include "BitmapText.h"


const int MAX_NUM_SYSTEM_MESSAGE_LINES	=	5;


class WindowManager
{
public:
	WindowManager();
	~WindowManager();

	// pass these messages along to the current state
	void Restore();
	void Invalidate();
	void Update( float fDeltaTime );
	void Draw();
	void Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI );

	void SetNewWindow( Window *pNewWindow );
	void AddWindowToTop( Window *pNewWindow );
	void PopTopWindow();

	void SystemMessage( CString sMessage );

	void SendMessageToTopWindow( WindowMessage SM, float fDelay );


private:
	CArray<Window*, Window*&> m_WindowStack;
	CArray<Window*, Window*&> m_WindowsToDelete;


	BitmapText m_textFPS;
	BitmapText m_textSystemMessage[MAX_NUM_SYSTEM_MESSAGE_LINES];
};


extern WindowManager*	WM;	// global and accessable from anywhere in our program

#endif