#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowMenu.h

 Desc: Base class for menu Windows.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowMenu.h"
#include <assert.h>
#include "RageTextureManager.h"
#include "RageUtil.h"
#include "RageMusic.h"
#include "WindowManager.h"
#include "WindowDancing.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"




const float HEADER_X	=	CENTER_X;
const float HEADER_Y	=	50;
const float HELP_X		=	CENTER_X;
const float HELP_Y		=	SCREEN_HEIGHT-25;


const WindowMessage SM_PlaySample			= WindowMessage(SM_User-4);
const WindowMessage SM_GoToPrevState		= WindowMessage(SM_User-5);
const WindowMessage SM_GoToNextState		= WindowMessage(SM_User-6);
const WindowMessage SM_JustPressedNext		= WindowMessage(SM_User-7);


WindowMenu::WindowMenu( CString sBackgroundPath, CString sTopEdgePath, CString sHelpText )
{
	RageLog( "WindowMenu::WindowMenu()" );


	m_sprBG.Load( sBackgroundPath );
	m_sprBG.StretchTo( CRect(0,0,SCREEN_WIDTH,SCREEN_HEIGHT) );
	m_sprBG.TurnShadowOff();
	this->AddActor( &m_sprBG );

	m_sprTopEdge.Load( sTopEdgePath );
	m_sprTopEdge.TurnShadowOff();
	m_sprTopEdge.SetZ( -1 );
	this->AddActor( &m_sprTopEdge );

	RageLog("Middle of WindowMenu()");

	m_sprBottomEdge.Load( THEME->GetPathTo(GRAPHIC_MENU_BOTTOM_EDGE) );
	m_sprBottomEdge.TurnShadowOff();
	m_sprBottomEdge.SetZ( -1 );
	this->AddActor( &m_sprBottomEdge );

	m_textHelp.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textHelp.SetXY( HELP_X, HELP_Y );
	m_textHelp.SetZ( -1 );
	m_textHelp.SetText( sHelpText );
	m_textHelp.SetZoom( 0.5f );
	m_textHelp.SetEffectBlinking();
	this->AddActor( &m_textHelp );

	
	m_soundSwoosh.Load( THEME->GetPathTo(SOUND_MENU_SWOOSH) );
	m_soundBack.Load( THEME->GetPathTo(SOUND_MENU_BACK) );


	TweenOnScreen();


	m_soundSwoosh.PlayRandom();

	RageLog("End of WindowMenu()");
}


void WindowMenu::TweenOnScreen()
{
	m_sprTopEdge.SetXY( CENTER_X, -m_sprTopEdge.GetZoomedHeight() );
	m_sprTopEdge.BeginTweening( MENU_TWEEN_TIME );
	m_sprTopEdge.SetTweenY( m_sprTopEdge.GetZoomedHeight()/2 );

	m_sprBottomEdge.SetXY( CENTER_X, SCREEN_HEIGHT + m_sprBottomEdge.GetZoomedHeight() );
	m_sprBottomEdge.BeginTweening( MENU_TWEEN_TIME );
	m_sprBottomEdge.SetTweenY( SCREEN_HEIGHT - m_sprBottomEdge.GetZoomedHeight()/2 );

	m_sprBG.SetDiffuseColor( D3DXCOLOR(0,0,0,1) );
	m_sprBG.BeginTweening( MENU_TWEEN_TIME );
	m_sprBG.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );

	float fOriginalZoomY = m_textHelp.GetZoomY();
	m_textHelp.SetZoomY( 0 );
	m_textHelp.BeginTweening( MENU_TWEEN_TIME );
	m_textHelp.SetTweenZoomY( fOriginalZoomY );
}

void WindowMenu::TweenOffScreen()
{
	m_sprTopEdge.BeginTweening( MENU_TWEEN_TIME );
	m_sprTopEdge.SetTweenY( -m_sprTopEdge.GetZoomedHeight() );

	m_sprBottomEdge.BeginTweening( MENU_TWEEN_TIME );
	m_sprBottomEdge.SetTweenY( SCREEN_HEIGHT + m_sprTopEdge.GetZoomedHeight() );

	m_sprBG.SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
	m_sprBG.BeginTweening( MENU_TWEEN_TIME );
	m_sprBG.SetTweenDiffuseColor( D3DXCOLOR(0,0,0,1) );

	m_textHelp.BeginTweening( MENU_TWEEN_TIME );
	m_textHelp.SetTweenZoomY( 0 );
}


void WindowMenu::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	// default input handler
	Window::Input( DeviceI, PadI, PlayerI );
}

void WindowMenu::HandleWindowMessage( WindowMessage SM )
{
	//switch( SM )
	//{
	//}

}


void WindowMenu::Back()
{
	TweenOffScreen();
	
	m_soundBack.PlayRandom();
}


void WindowMenu::Next()
{
	TweenOffScreen();
	
	m_soundSwoosh.PlayRandom();
}


