#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowMenuResults.h

 Desc: Shows the user their score after dancing has ended.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowMenuResults.h"
#include <assert.h>
#include "WindowMenuSelectMusic.h"
#include "SongManager.h"
#include "WindowManager.h"
#include "RageUtil.h"
#include "ScreenDimensions.h"
#include "Steps.h"
#include "ThemeManager.h"




const float HEADER_X		=	CENTER_X;
const float HEADER_Y		=	50;
const float LETTER_Y		=	HEADER_Y + 40;
const float SCORES_START_Y	=	LETTER_Y + 82;
const float SCORES_SPACING	=	33;
const float TOTAL_SCORE_Y	=	SCREEN_HEIGHT-55;
const float HELP_X			=	CENTER_X;
const float HELP_Y			=	(SCREEN_HEIGHT-35);

const float PLAYER_X[NUM_PLAYERS] = { SCREEN_WIDTH*4/16, SCREEN_WIDTH*12/16 };
const float GRADE_X[NUM_PLAYERS] = { SCREEN_WIDTH*3/16, SCREEN_WIDTH*13/16 };




WindowMenuResults::WindowMenuResults() : 
	WindowMenu( THEME->GetPathTo(GRAPHIC_RESULT_BACKGROUND), 
				THEME->GetPathTo(GRAPHIC_RESULT_TOP_EDGE),
				"Press NEXT to continue."
				)
{
	RageLog( "WindowMenuResults::WindowMenuResults()" );

	int i, p;	// for counting

	m_MiniBackground.LoadFromSong( SONGS->m_pCurSong );
	m_MiniBackground.SetXY( CENTER_X, 100 );
	m_MiniBackground.SetZ( -1 );
	m_MiniBackground.SetZoom( 2 );
	m_MiniBackground.BeginTweening( MENU_TWEEN_TIME );
	m_MiniBackground.SetTweenZoom( 1 );
	this->AddActor( &m_MiniBackground );



	for( i=0; i<NUM_RESULT_LINES; i++ ) 
	{
		m_sprCategory[i].Load( THEME->GetPathTo(GRAPHIC_JUDGEMENT) );
		m_sprCategory[i].StopAnimating();
		int iFrameNo;
		switch( i )
		{
		case 0:	iFrameNo = 0;	break;
		case 1:	iFrameNo = 1;	break;
		case 2:	iFrameNo = 2;	break;
		case 3:	iFrameNo = 3;	break;
		case 4:	iFrameNo = 4;	break;
		case 5:	iFrameNo = 5;	break;
		case 6:	iFrameNo = 7;	break;
		case 7:	iFrameNo = 8;	break;
		}
		m_sprCategory[i].SetState( iFrameNo );
		m_sprCategory[i].SetXY( CENTER_X, SCORES_START_Y + i*SCORES_SPACING );
		m_sprCategory[i].SetZoom( 0.7f );
		m_sprCategory[i].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );	// invisible
		this->AddActor( &m_sprCategory[i] );

		for( int j=0; j<NUM_PLAYERS; j++ ) 
		{
			m_textScores[i][j].Load( THEME->GetPathTo(FONT_BOLD_NUMBERS) );
			m_textScores[i][j].SetXY( PLAYER_X[j]-30, SCORES_START_Y + i*SCORES_SPACING );
			m_textScores[i][j].SetText( "    " );
			m_textScores[i][j].SetZoom( 0.6f );
			m_textScores[i][j].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );	// invisible
			this->AddActor( &m_textScores[i][j] );

			m_textPercentages[i][j].Load( THEME->GetPathTo(FONT_NORMAL) );
			m_textPercentages[i][j].SetZoom( 0.6f );
			m_textPercentages[i][j].SetXY( PLAYER_X[j]+40, SCORES_START_Y + i*SCORES_SPACING + 2 );
			m_textPercentages[i][j].SetText( "  0%" );
			m_textPercentages[i][j].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );	// invisible
			this->AddActor( &m_textPercentages[i][j] );
		}
	}

	for( p=0; p<NUM_PLAYERS; p++ )
	{
		m_textNewRecord[p].Load( THEME->GetPathTo(FONT_FUTURISTIC) );
		m_textNewRecord[p].SetXY( PLAYER_X[p], TOTAL_SCORE_Y-30 );
		m_textNewRecord[p].SetText( "It's a new record!" );
		m_textNewRecord[p].SetZoomY( 0.5f );
		m_textNewRecord[p].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );	// invisible
		this->AddActor( &m_textNewRecord[p] );

		m_ScoreFrame[p].Load( THEME->GetPathTo(GRAPHIC_SCORE_FRAME) );
		m_ScoreFrame[p].SetXY( PLAYER_X[p], TOTAL_SCORE_Y );
		m_ScoreFrame[p].SetZoom( 0.8f );
		m_ScoreFrame[p].SetZ( -1 );
		this->AddActor( &m_ScoreFrame[p] );

		m_TotalScore[p].Load( THEME->GetPathTo(FONT_BOLD_NUMBERS) );
		m_TotalScore[p].SetXY( PLAYER_X[p], TOTAL_SCORE_Y );
		m_TotalScore[p].SetText( "         " );
		m_TotalScore[p].SetZoom( 0.8f );
		m_TotalScore[p].SetZ( -1 );
		this->AddActor( &m_TotalScore[p] );
		
		m_Grades[p].SetXY( GRADE_X[p], LETTER_Y );
		m_Grades[p].SetZ( -2 );
		m_Grades[p].SetZoom( 1.0f );
		m_Grades[p].SetDiffuseColor( D3DXCOLOR(1,1,1,0) );	// invisible
		this->AddActor( &m_Grades[p] );

		m_bNewRecord[p] = false;
	}

	

	PREFS->m_iCurrentStage++;



	for( p=0; p<NUM_PLAYERS; p++ )
	{
		if( !PREFS->IsPlayerEnabled( (PlayerNumber)p ) )
			continue;	// skip

		ScoreSummary &SS = PREFS->scoreSummaryPlayer[p];
		

		Steps* pSteps = SONGS->m_pStepsPlayer[p];
		int iNumSteps = pSteps == NULL ? 0 : pSteps->GetNumTapSteps();
		int iNumHoldSteps = pSteps == NULL ? 0 : pSteps->GetNumHoldSteps();
		if( iNumHoldSteps == 0 )
			iNumHoldSteps = 1;		// so we don't divide by 0

		float fPercentPerfects	= SS.perfect	/ (float)iNumSteps;
		float fPercentGreats	= SS.great		/ (float)iNumSteps;
		float fPercentGoods		= SS.good		/ (float)iNumSteps;
		float fPercentBoos		= SS.boo		/ (float)iNumSteps;
		float fPercentMissess	= SS.miss		/ (float)iNumSteps;
		float fPercentMaxCombo	= SS.max_combo	/ (float)iNumSteps;
		float fPercentOK		= SS.ok			/ (float)iNumHoldSteps;
		float fPercentNG		= SS.ng			/ (float)iNumHoldSteps;

		Grade grade;
		if(	SS.perfect == iNumSteps )							grade = GRADE_AAA;
		else if( fPercentPerfects + fPercentGreats > 0.99f )	grade = GRADE_AA;
		else if( fPercentPerfects + fPercentGreats > 0.95f )	grade = GRADE_A;
		else if( fPercentPerfects + fPercentGreats > 0.85f )	grade = GRADE_B;
		else if( fPercentPerfects + fPercentGreats > 0.70f )	grade = GRADE_C;
		else if( fPercentPerfects + fPercentGreats > 0.50f )	grade = GRADE_D;
		else													grade = GRADE_E;


		m_textScores[0][p].SetText( ssprintf("%4.1d", SS.perfect) );
		m_textScores[1][p].SetText( ssprintf("%4.1d", SS.great) );
		m_textScores[2][p].SetText( ssprintf("%4.1d", SS.good) );
		m_textScores[3][p].SetText( ssprintf("%4.1d", SS.boo) );
		m_textScores[4][p].SetText( ssprintf("%4.1d", SS.miss) );
		m_textScores[5][p].SetText( ssprintf("%4.1d", SS.max_combo) );
		m_textScores[6][p].SetText( ssprintf("%4.1d", SS.ok) );

		m_textPercentages[0][p].SetText( ssprintf("%3.0f%%", 100*fPercentPerfects) );
		m_textPercentages[1][p].SetText( ssprintf("%3.0f%%", 100*fPercentGreats) );
		m_textPercentages[2][p].SetText( ssprintf("%3.0f%%", 100*fPercentGoods) );
		m_textPercentages[3][p].SetText( ssprintf("%3.0f%%", 100*fPercentBoos) );
		m_textPercentages[4][p].SetText( ssprintf("%3.0f%%", 100*fPercentMissess) );
		m_textPercentages[5][p].SetText( ssprintf("%3.0f%%", 100*fPercentMaxCombo) );
		m_textPercentages[6][p].SetText( ssprintf("%3.0f%%", 100*fPercentOK) );

		m_TotalScore[p].SetText( ssprintf("%9.0f", SS.score) );

		m_Grades[p].SetGrade( grade );

		
		// update song stats
		if( PREFS )
		{
			if( !PREFS->IsPlayerEnabled( (PlayerNumber)p ) )
				continue;	// not joined.  skip

			Steps* pSteps = SONGS->m_pStepsPlayer[p];
			pSteps->m_iNumTimesPlayed++;

			if( SS.max_combo > pSteps->m_iMaxCombo )
				pSteps->m_iMaxCombo = SS.max_combo;

			if( SS.score > pSteps->m_iTopScore )
			{
				pSteps->m_iTopScore = (int)SS.score;
				m_bNewRecord[p] = true;
			}

			if( grade > pSteps->m_TopGrade )
				pSteps->m_TopGrade = grade;

		}
	}
	



	
	for( i=0; i<20*4; i++ )
		this->SendWindowMessage( WindowMessage(SM_User+i),	i/8.0f+0.5f );


	// fix this to play a sound according to the grade

	m_SoundResultComment.Load( THEME->GetPathTo(ANNOUNCER_RESULT_A) );
	m_SoundResultComment.PlayRandom();

	m_wipe.SetZ( -2 );
	m_wipe.OpenWipingRight();
	this->AddActor( &m_wipe );
}

WindowMenuResults::~WindowMenuResults()
{
	RageLog( "WindowMenuResults::~WindowMenuResults()" );

}

void WindowMenuResults::TweenOffScreen()
{
	m_MiniBackground.BeginTweening( MENU_TWEEN_TIME );
	m_MiniBackground.SetTweenZoomY( 0 );

	for( int i=0; i<NUM_RESULT_LINES; i++ ) 
	{
		m_sprCategory[i].BeginTweening( MENU_TWEEN_TIME );
		m_sprCategory[i].SetTweenZoomY( 0 );

		for( int j=0; j<NUM_PLAYERS; j++ ) 
		{
			m_textScores[i][j].BeginTweening( MENU_TWEEN_TIME );
			m_textScores[i][j].SetTweenZoomY( 0 );

			m_textPercentages[i][j].BeginTweening( MENU_TWEEN_TIME );
			m_textPercentages[i][j].SetTweenZoomY( 0 );
		}
	}

	for( int p=0; p<NUM_PLAYERS; p++ )
	{
		m_textNewRecord[p].BeginTweening( MENU_TWEEN_TIME );
		m_textNewRecord[p].SetTweenZoomY( 0 );

		m_ScoreFrame[p].BeginTweening( MENU_TWEEN_TIME );
		m_ScoreFrame[p].SetTweenZoomY( 0 );

		m_TotalScore[p].BeginTweening( MENU_TWEEN_TIME );
		m_TotalScore[p].SetTweenZoomY( 0 );

		m_Grades[p].BeginTweening( MENU_TWEEN_TIME );
		m_Grades[p].SetTweenZoomY( 0 );
	}
}


void WindowMenuResults::Update( float fDeltaTime )
{
	WindowMenu::Update( fDeltaTime );
}

void WindowMenuResults::RenderPrimitives()
{
	WindowMenu::RenderPrimitives();
}

void WindowMenuResults::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	RageLog( "WindowMenuResults::Input()" );

	if( m_wipe.IsClosing() )
		return;

	WindowMenu::Input( DeviceI, PadI, PlayerI );

}

void WindowMenuResults::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_DoneClosingWipingLeft:
		break;
	case SM_DoneClosingWipingRight:
		WM->SetNewWindow( new WindowMenuSelectMusic );
		break;
	default:
		if( SM >= SM_User )
		{
			int iQuarterSecondsFromStart = SM - SM_User;

			switch( iQuarterSecondsFromStart )
			{
			case 0:
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
				{
					int iIndex = iQuarterSecondsFromStart;
					m_sprCategory[iQuarterSecondsFromStart-0].SetZoom( 1.6f );
					m_sprCategory[iQuarterSecondsFromStart-0].BeginTweening( 0.5f );
					m_sprCategory[iQuarterSecondsFromStart-0].SetTweenZoom( 0.8f );
					m_sprCategory[iQuarterSecondsFromStart-0].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
					break;
				}
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
			case 12:
			case 13:
				int p;
				for( p=0; p<NUM_PLAYERS; p++ )
				{
					int iIndex = iQuarterSecondsFromStart - 7;
					m_textScores[iIndex][p].SetZoom( 1.2f );
					m_textScores[iIndex][p].BeginTweening( 0.5f );
					m_textScores[iIndex][p].SetTweenZoom( 0.6f );
					m_textScores[iIndex][p].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );

					m_textPercentages[iIndex][p].SetZoom( 1.2f );
					m_textPercentages[iIndex][p].BeginTweening( 0.5f );
					m_textPercentages[iIndex][p].SetTweenZoom( 0.6f );
					m_textPercentages[iIndex][p].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
				}
				break;
			case 20:
				for( p=0; p<NUM_PLAYERS; p++ )
				{
					if( !PREFS->IsPlayerEnabled( PlayerNumber(p) ) )
						continue;

					m_Grades[p].SetZoom( 2.0f );
					m_Grades[p].BeginTweening( 0.5f );
					m_Grades[p].SetTweenZoom( 1.0f );
					m_Grades[p].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
					m_Grades[p].SetEffectGlowing();
				}
			case 22:
				for( p=0; p<NUM_PLAYERS; p++ )
				{
					if( !PREFS->IsPlayerEnabled( PlayerNumber(p) ) )
						continue;

					if( m_bNewRecord[p] )
					{
						m_textNewRecord[p].SetZoom( 1.0f );
						m_textNewRecord[p].BeginTweening( 0.5f );
						m_textNewRecord[p].SetTweenZoom( 0.5f );
						m_textNewRecord[p].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
						m_textNewRecord[p].SetEffectCamelion();
					}
				}
				break;
			}

		}
	}
}

void WindowMenuResults::Back()
{
	Next();
}

void WindowMenuResults::Next()
{
	WindowMenu::Next();

	TweenOffScreen();

	m_wipe.CloseWipingRight( SM_DoneClosingWipingRight );
}

