#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowMenuSelectMusic.cpp

 Desc: Testing the Window class.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/


#include "WindowMenuSelectMusic.h"
#include "WindowManager.h"
#include "PrefsManager.h"
#include "SongManager.h"
#include "RageMusic.h"
#include "WindowTitleMenu.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "WindowDancing.h"
#include "WindowPrompt.h"
#include "WindowPlayerOptions.h"


const float BPM_X		= CENTER_X-100;
const float BPM_Y		= 90;

const float SCORE_X[NUM_PLAYERS]	=	{ 152, 181 };
const float SCORE_Y[NUM_PLAYERS]	=	{ CENTER_Y+100, CENTER_Y+134 };

const float GRADE_X[NUM_PLAYERS]	=	{ SCORE_X[0]+115, SCORE_X[1]+115 };
const float GRADE_Y[NUM_PLAYERS]	=	{ SCORE_Y[0], SCORE_Y[1] };

const float STEPS_X		= 155;
const float STEPS_Y		= 223;

const float WHEEL_X		= CENTER_X+142;
const float WHEEL_Y		= CENTER_Y+5;

const float MINI_BACKGROUND_X		= 159;
const float MINI_BACKGROUND_Y		= 151;

const float TIP_X	=	160;
const float TIP_Y	=	SCREEN_HEIGHT-60;


const float TWEEN_TIME		= 0.35f;


const WindowMessage SM_GoToPrevState		=	WindowMessage(SM_User+1);
const WindowMessage SM_GoToNextState		=	WindowMessage(SM_User+2);
const WindowMessage SM_ConfirmChange		=	WindowMessage(SM_User+3);



WindowMenuSelectMusic::WindowMenuSelectMusic() : 
	WindowMenu( THEME->GetPathTo(GRAPHIC_SELECT_MUSIC_BACKGROUND), 
				THEME->GetPathTo(GRAPHIC_SELECT_MUSIC_TOP_EDGE),
				ssprintf("%s %s change music    %s change sort", CString(char(1)), CString(char(2)), CString(char(6)) )
				)

{
	RageLog( "WindowMenuSelectMusic::WindowMenuSelectMusic()" );


	m_StepsSelector.SetXY( STEPS_X, STEPS_Y );
	m_StepsSelector.SetZoom( 0.4f );
	for( int p=0; p<NUM_PLAYERS; p++ )
	{
		if( PREFS->IsPlayerEnabled( (PlayerNumber)p ) )
			m_StepsSelector.Enable( (PlayerNumber)p );
	}
	this->AddActor( &m_StepsSelector );


	m_MusicWheel.SetXY( WHEEL_X, WHEEL_Y );
	this->AddActor( &m_MusicWheel );


	m_BPMDisplay.SetXY( BPM_X, BPM_Y );
	m_BPMDisplay.SetZoom( 0.7f );
	this->AddActor( &m_BPMDisplay );
	

	m_MiniBackground[0].SetXY( MINI_BACKGROUND_X, MINI_BACKGROUND_Y );
	m_MiniBackground[1].SetXY( MINI_BACKGROUND_X, MINI_BACKGROUND_Y );
	this->AddActor( &m_MiniBackground[0] );
	this->AddActor( &m_MiniBackground[1] );


	for( p=0; p<NUM_PLAYERS; p++ )
	{
		m_TopGrade[p].SetXY( GRADE_X[p], GRADE_Y[p] );
		m_TopGrade[p].SetZoom( 0.4f );
		
		m_TopScore[p].SetXY( SCORE_X[p], SCORE_Y[p] );
		m_TopScore[p].SetZoom( 0.75f );

		if( PREFS->IsPlayerEnabled( (PlayerNumber)p ) )
		{
			this->AddActor( &m_TopGrade[p] );
			this->AddActor( &m_TopScore[p] );
		}
	}

	m_TipDisplay.SetXY( TIP_X, TIP_Y );
	m_TipDisplay.SetZ( -2 );
	m_TipDisplay.SetZoom( 0.6f );

	CStringArray arrayTips;
	arrayTips.Add( ssprintf( "Press %s %s to change difficulty", CString(3), CString(4) ) );
	arrayTips.Add( ssprintf( "Press %s %s to change music", CString(1), CString(2) ) );
	arrayTips.Add( ssprintf( "Press %s to switch sorts", CString(6) ) );
	arrayTips.Add( ssprintf( "Press NEXT on music to select it" ) );
	arrayTips.Add( ssprintf( "Press NEXT on group to expand it" ) );
	m_TipDisplay.SetTips( arrayTips );
	this->AddActor( &m_TipDisplay );


	this->AddActor( &m_Fade );

	m_textHoldForOptions.Load( THEME->GetPathTo(FONT_FUTURISTIC) );
	m_textHoldForOptions.SetXY( CENTER_X, CENTER_Y );
	m_textHoldForOptions.SetText( "hold NEXT for options" );
	m_textHoldForOptions.SetZoom( 1 );
	m_textHoldForOptions.SetZoomY( 0 );
	m_textHoldForOptions.SetDiffuseColor( D3DXCOLOR(1,1,1,0) );
	this->AddActor( &m_textHoldForOptions );


	m_soundSelect.Load( THEME->GetPathTo(SOUND_SELECT) );
	m_soundMusicComment.Load( THEME->GetPathTo(ANNOUNCER_MUSIC_COMMENT) );

	AfterMusicChange();

	// select the last played steps
	for( p=0; p<NUM_PLAYERS; p++ )
	{
		m_StepsSelector.SelectTheseSteps( (PlayerNumber)p, SONGS->m_pStepsPlayer[p] );
	}

	AfterStepsChange();

	TweenOnScreen();


	if( SONGS->m_pCurSong != NULL  &&  SONGS->m_pCurSong->HasChangedSinceLastSave() )
	{
		this->SendWindowMessage( SM_ConfirmChange, 0.1f );
	}
}


WindowMenuSelectMusic::~WindowMenuSelectMusic()
{
	RageLog( "WindowMenuSelectMusic::~WindowMenuSelectMusic()" );

}

void WindowMenuSelectMusic::TweenOnScreen()
{
	float fOriginalZoomY;

	fOriginalZoomY = m_StepsSelector.GetZoomY();
	m_StepsSelector.BeginTweening( MENU_TWEEN_TIME );
	m_StepsSelector.SetTweenZoomY( fOriginalZoomY );

	fOriginalZoomY = m_BPMDisplay.GetZoomY();
	m_BPMDisplay.BeginTweening( MENU_TWEEN_TIME );
	m_BPMDisplay.SetTweenZoomY( fOriginalZoomY );


	for( int p=0; p<NUM_PLAYERS; p++ )
	{
		fOriginalZoomY = m_TopGrade[p].GetZoomY();
		m_TopGrade[p].BeginTweening( MENU_TWEEN_TIME );
		m_TopGrade[p].SetTweenZoomY( fOriginalZoomY );

		fOriginalZoomY = m_TopScore[p].GetZoomY();
		m_TopScore[p].BeginTweening( MENU_TWEEN_TIME );
		m_TopScore[p].SetTweenZoomY( fOriginalZoomY );
	}


	fOriginalZoomY = m_MiniBackground[0].GetZoomY();
	m_MiniBackground[0].BeginTweening( MENU_TWEEN_TIME );
	m_MiniBackground[0].SetTweenZoomY( fOriginalZoomY );
	
	fOriginalZoomY = m_MiniBackground[1].GetZoomY();
	m_MiniBackground[1].BeginTweening( MENU_TWEEN_TIME );
	m_MiniBackground[1].SetTweenZoomY( fOriginalZoomY );


	fOriginalZoomY = m_TipDisplay.GetZoomY();
	m_TipDisplay.BeginTweening( MENU_TWEEN_TIME );
	m_TipDisplay.SetTweenZoomY( fOriginalZoomY );

	m_MusicWheel.TweenOnScreen();
}

void WindowMenuSelectMusic::TweenOffScreen()
{
	m_StepsSelector.BeginTweening( MENU_TWEEN_TIME );
	m_StepsSelector.SetTweenZoomY( 0 );

	m_BPMDisplay.BeginTweening( MENU_TWEEN_TIME );
	m_BPMDisplay.SetTweenZoomY( 0 );

	for( int p=0; p<NUM_PLAYERS; p++ )
	{
		m_TopGrade[p].BeginTweening( MENU_TWEEN_TIME );
		m_TopGrade[p].SetTweenZoomY( 0 );

		m_TopScore[p].BeginTweening( MENU_TWEEN_TIME );
		m_TopScore[p].SetTweenZoomY( 0 );
	}

	m_MiniBackground[0].BeginTweening( MENU_TWEEN_TIME );
	m_MiniBackground[0].SetTweenZoomY( 0 );
	
	m_MiniBackground[1].BeginTweening( MENU_TWEEN_TIME );
	m_MiniBackground[1].SetTweenZoomY( 0 );

	m_TipDisplay.BeginTweening( MENU_TWEEN_TIME );
	m_TipDisplay.SetTweenZoomY( 0 );

	m_MusicWheel.TweenOffScreen();
}


void WindowMenuSelectMusic::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	RageLog( "WindowMenuSelectMusic::Input()" );

	if( m_Fade.IsClosing() )
		return;		// ignore


	for( int p=0; p<NUM_PLAYERS; p++ )	// foreach player
	{
		if( PlayerI.player_no == p  &&  PadI.button == BUTTON_UP )
			m_StepsSelector.Up( (PlayerNumber)p );
		if( PlayerI.player_no == p  &&  PadI.button == BUTTON_DOWN )
			m_StepsSelector.Down( (PlayerNumber)p );
	}	


	Window::Input( DeviceI, PadI, PlayerI );	// default input handler
}

void WindowMenuSelectMusic::HandleWindowMessage( WindowMessage SM )
{
	WindowMenu::HandleWindowMessage( SM );

	static bool bAnswer = false;

	switch( SM )
	{

	case SM_ConfirmChange:
		bAnswer = true;
		WM->AddWindowToTop( 
			new WindowPrompt(
				ssprintf( 
					"You have changed the offset of\n\n"
					"'%s'.\n\n"
					"Would you like to save changes\n"
					"to song file(s)?", 
					SONGS->m_pCurSong->GetTitle() 
				),
				PROMPT_YES_NO,
				&bAnswer
			)
		);
		break;
	case SM_RegainingInputFocus:
		if( bAnswer == true )
		{
			SONGS->m_pCurSong->SaveOffsetChangeToDisk();
		}
		break;


	case SM_GoToPrevState:
		WM->SetNewWindow( new WindowTitleMenu );
		break;
	case SM_GoToNextState:

		// find out if the Next button is being held down on any of the pads
		bool bIsHoldingNext;
		bIsHoldingNext = false;
		int pad;
		for( pad=0; pad<NUM_PADS; pad++ )
		{
			PadInput pi( (PadNumber)pad, BUTTON_NEXT );
			if( PREFS->IsButtonDown( pi ) )
				bIsHoldingNext = true;
		}

		if( bIsHoldingNext )
		{
			WM->SetNewWindow( new WindowPlayerOptions );
		}
		else
		{
			MUSIC->Stop();

			WM->SetNewWindow( new WindowDancing );
		}
		break;
	case SM_PlaySongSample:
		PlayMusicSample();
		break;
	case SM_StepsChanged:
		AfterStepsChange();
		break;
	}
}

void WindowMenuSelectMusic::Left()
{
	BeforeMusicChange();

	m_MusicWheel.PrevMusic();
	
	AfterMusicChange();
}


void WindowMenuSelectMusic::Right()
{
	BeforeMusicChange();

	m_MusicWheel.NextMusic();

	AfterMusicChange();
}

void WindowMenuSelectMusic::Next()
{
	// this needs to check whether valid steps are selected!

	if( m_MusicWheel.Select() )	// we selected a song
	{
		if( !m_MusicWheel.GetSelectedSong()->HasMusic() )
		{
			WM->AddWindowToTop( new WindowPrompt( "ERROR:\n \nThis song does not have a music file\n and cannot be played.", PROMPT_OK) );
			return;
		}

		for( int p=0; p<NUM_PLAYERS; p++ )
		{
			Steps* pSteps = m_StepsSelector.GetSelectedSteps( (PlayerNumber)p );

			//ASSERT( pSteps != NULL  ||  !PREFS->IsPlayerEnabled( (PlayerNumber)p ) );
			
			SONGS->m_pStepsPlayer[p] = pSteps;
		}


		TweenOffScreen();
		
		WindowMenu::Next();

		m_soundMusicComment.PlayRandom();
		m_soundSelect.PlayRandom();

		// show "hold NEXT for options"
		m_textHoldForOptions.BeginTweeningQueued( 0.25f );	// fade in
		m_textHoldForOptions.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_textHoldForOptions.SetTweenZoomY( 1 );
		m_textHoldForOptions.BeginTweeningQueued( 1.0f );	// sleep
		m_textHoldForOptions.BeginTweeningQueued( 0.25f );	// fade out
		m_textHoldForOptions.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );
		m_textHoldForOptions.SetTweenZoomY( 0 );

		m_Fade.SetTransitionTime( 1.5f );	// give enough time for m_textHoldForOptions to complete its tweens
		m_Fade.CloseWipingRight( SM_GoToNextState );
	}

	// else do nothing
}

void WindowMenuSelectMusic::UpRight()
{
	WindowMenu::UpRight();

	m_MusicWheel.NextSort();
}

void WindowMenuSelectMusic::Back()
{
	WindowMenu::Back();

	MUSIC->Stop();

	TweenOffScreen();

	m_Fade.CloseWipingRight( SM_GoToPrevState );
}

void WindowMenuSelectMusic::BeforeMusicChange()
{
	m_MiniBackground[1].Load( m_MiniBackground[0].GetTexturePath() );

	m_MiniBackground[1].SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
	m_MiniBackground[1].BeginTweening( TWEEN_TIME );
	m_MiniBackground[1].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );
}

void WindowMenuSelectMusic::AfterMusicChange()
{
	Song* pSong = m_MusicWheel.GetSelectedSong();
	
	SONGS->m_pCurSong = pSong;


	if( pSong == NULL )		// a section is selected
	{
		m_StepsSelector.LoadFromSong( NULL, PREFS->m_GameMode );


		CString sGroup = m_MusicWheel.GetSelectedSection();
		CString sPath = SONGS->GetBannerPathFromGroup( sGroup );

		if( sPath == "" )
			m_MiniBackground[0].LoadFromSong( NULL );
		else
			m_MiniBackground[0].Load( sPath );

		m_BPMDisplay.SetBPMRange( 0, 0 );
	}
	else			// a song is selected
	{	
		m_StepsSelector.LoadFromSong( pSong, PREFS->m_GameMode );
		
		m_MiniBackground[0].LoadFromSong( pSong );
		
		float fMinBPM, fMaxBPM;
		pSong->GetMinMaxBPM( fMinBPM, fMaxBPM );
		m_BPMDisplay.SetBPMRange( fMinBPM, fMaxBPM );
	}
}

void WindowMenuSelectMusic::AfterStepsChange()
{
	for( int p=0; p<NUM_PLAYERS; p++ )	// foreach player
	{
		if( m_StepsSelector.GetSelectedSteps( (PlayerNumber)p ) )
		{
			Grade grade = m_StepsSelector.GetSelectedSteps( (PlayerNumber)p )->m_TopGrade;
			int iTopScore = m_StepsSelector.GetSelectedSteps( (PlayerNumber)p )->m_iTopScore;
			m_TopGrade[p].SetGrade( grade );
			m_TopScore[p].SetScore( (float)iTopScore );
		}
		else
		{
			m_TopGrade[p].SetGrade( GRADE_NO_DATA );
			m_TopScore[p].SetScore( 0 );
		}
	}
}

void WindowMenuSelectMusic::PlayMusicSample()
{
	//RageLog( "WindowSelectSong::PlaySongSample()" );

	MUSIC->Stop();

	Song* pSong = m_MusicWheel.GetSelectedSong();
	if( pSong == NULL )
		return;

	CString sSongToPlay = pSong->GetMusicPath();
 
	if( pSong->HasMusic() )
	{
		MUSIC->Load( sSongToPlay );
		MUSIC->Play( true );
	}
}


