#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowMenuSelectStyle.cpp

 Desc: Testing the Window class.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/


#include "WindowMenuSelectStyle.h"
#include "WindowManager.h"
#include "PrefsManager.h"
#include "RageMusic.h"
#include "WindowTitleMenu.h"
#include "WindowCaution.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "WindowMenuSelectMusic.h"
#include "WindowSandbox.h"


const float PAD_X[NUM_STYLE_PADS] = {
	CENTER_X-250,
	CENTER_X-125,
	CENTER_X-20,
	CENTER_X+125,
	CENTER_X+250,
};
const float PAD_Y[NUM_STYLE_PADS] = {
	CENTER_Y+60,
	CENTER_Y-30,
	CENTER_Y+70,
	CENTER_Y-30,
	CENTER_Y+60,
};
const float DANCER_X[NUM_STYLE_DANCERS] = {
	PAD_X[0],
	PAD_X[1]-35,
	PAD_X[1]+35,
	PAD_X[2],
	PAD_X[3]-35,
	PAD_X[3]+35,
	PAD_X[4],
};
const float DANCER_Y[NUM_STYLE_DANCERS] = {
	PAD_Y[0],
	PAD_Y[1]+20,
	PAD_Y[1]-20,
	PAD_Y[2],
	PAD_Y[3]+20,
	PAD_Y[3]-20,
	PAD_Y[4],
};

CString TEXT_EXPLANATION1[NUM_GAMEMODES] = {
	"1 Player",
	"2 Players",
	"1 Player",
	"2 Players",
	"1 Player"
};
CString TEXT_EXPLANATION2[NUM_GAMEMODES] = {
	"Dance using 4 panels on one pad",
	"Each uses 4 panels on one pad",
	"Dance using 8 panels on two pads",
	"Each uses 4 panels on one pad",
	"Dance using 6 panels on one pad"
};

const float EXPLANATION1_ZOOM_X		=	1;
const float EXPLANATION1_ZOOM_Y		=	1;
const float EXPLANATION2_ZOOM_X		=	0.65f;
const float EXPLANATION2_ZOOM_Y		=	0.7f;

const float ICON_X		= 140;
const float ICON_Y		= SCREEN_HEIGHT - 85;

const float EXPLANATION1_X		= ICON_X+110;
const float EXPLANATION1_Y		= ICON_Y+5;
const float EXPLANATION2_X		= EXPLANATION1_X;
const float EXPLANATION2_Y		= ICON_Y+32;

const float HELP_X		= CENTER_X;
const float HELP_Y		= SCREEN_HEIGHT-20;

const float TWEEN_TIME		= 0.35f;

const D3DXCOLOR COLOR_P1_SELECTED = D3DXCOLOR(0.4f,1.0f,0.8f,1);
const D3DXCOLOR COLOR_P2_SELECTED = D3DXCOLOR(1.0f,0.5f,0.2f,1);
const D3DXCOLOR COLOR_P1_NOT_SELECTED = COLOR_P1_SELECTED*0.5f + D3DXCOLOR(0,0,0,0.5f);
const D3DXCOLOR COLOR_P2_NOT_SELECTED = COLOR_P2_SELECTED*0.5f + D3DXCOLOR(0,0,0,0.5f);


const WindowMessage SM_GoToPrevState		=	WindowMessage(SM_User + 1);
const WindowMessage SM_GoToNextState		=	WindowMessage(SM_User + 2);
const WindowMessage SM_UpdateAnimations		=	WindowMessage(SM_User + 3);
const WindowMessage SM_TweenExplanation2	=	WindowMessage(SM_User + 4);


WindowMenuSelectStyle::WindowMenuSelectStyle() : 
	WindowMenu( THEME->GetPathTo(GRAPHIC_SELECT_STYLE_BACKGROUND), 
				THEME->GetPathTo(GRAPHIC_SELECT_STYLE_TOP_EDGE),
				ssprintf("Use %s %s to select, then press NEXT", CString(char(1)), CString(char(2)) )
				)

{
	RageLog( "WindowMenuSelectStyle::WindowMenuSelectStyle()" );

	m_GameMode = MODE_SINGLE;


	for( int i=0; i<NUM_STYLE_PADS; i++ )
	{
		CString sPadGraphicPath;
		switch( i )
		{
		case 0:
			sPadGraphicPath = THEME->GetPathTo(GRAPHIC_PAD_SINGLE);	
			break;
		case 4:
			sPadGraphicPath = THEME->GetPathTo(GRAPHIC_PAD_SOLO);	
			break;
		case 1:	
		case 2:	
		case 3:
			sPadGraphicPath = THEME->GetPathTo(GRAPHIC_PAD_DOUBLE);	
			break;
		}

		m_sprPad[i].Load( sPadGraphicPath );
		m_sprPad[i].SetXY( PAD_X[i], PAD_Y[i] );
		m_sprPad[i].SetZoom( 1 );
		this->AddActor( &m_sprPad[i] );
	}


	for( i=0; i<NUM_STYLE_DANCERS; i++ )
	{
		CString sDancerGraphicPath;
		D3DXCOLOR colorDiffuse;
		switch( i )
		{
		case 0:	sDancerGraphicPath = THEME->GetPathTo(GRAPHIC_DANCER_P1);	colorDiffuse = COLOR_P1_SELECTED;		SetX(-0.1f); 	break;
		case 1:	sDancerGraphicPath = THEME->GetPathTo(GRAPHIC_DANCER_P1);	colorDiffuse = COLOR_P1_NOT_SELECTED;	SetX(-0.1f); 	break;
		case 2:	sDancerGraphicPath = THEME->GetPathTo(GRAPHIC_DANCER_P2);	colorDiffuse = COLOR_P2_NOT_SELECTED;	SetX( 0.0f); 	break;
		case 3:	sDancerGraphicPath = THEME->GetPathTo(GRAPHIC_DANCER_P1);	colorDiffuse = COLOR_P1_NOT_SELECTED;	SetX(-0.1f); 	break;
		case 4:	sDancerGraphicPath = THEME->GetPathTo(GRAPHIC_DANCER_P1);	colorDiffuse = COLOR_P1_NOT_SELECTED;	SetX(-0.1f); 	break;
		case 5:	sDancerGraphicPath = THEME->GetPathTo(GRAPHIC_DANCER_P2);	colorDiffuse = COLOR_P2_NOT_SELECTED;	SetX( 0.0f); 	break;
		case 6:	sDancerGraphicPath = THEME->GetPathTo(GRAPHIC_DANCER_P1);	colorDiffuse = COLOR_P1_NOT_SELECTED;	SetX(-0.1f); 	break;
		}

		m_sprDancer[i].Load( sDancerGraphicPath );
		m_sprDancer[i].SetDiffuseColor( colorDiffuse );
		m_sprDancer[i].SetVertAlign( Actor::align_top );
		m_sprDancer[i].SetXY( DANCER_X[i], DANCER_Y[i] );
		m_sprDancer[i].SetZoom( 2 );
		m_sprDancer[i].StopAnimating();
		this->AddActor( &m_sprDancer[i] );
	}


	m_sprStyleIcon.Load( THEME->GetPathTo(GRAPHIC_STYLE_ICONS) );
	m_sprStyleIcon.TurnShadowOff();
	m_sprStyleIcon.StopAnimating();
	m_sprStyleIcon.SetXY( ICON_X, ICON_Y );
	this->AddActor( &m_sprStyleIcon );


	m_textExplanation1.Load( THEME->GetPathTo(FONT_FUTURISTIC) );
	m_textExplanation1.SetDiffuseColor( D3DXCOLOR(0,0.7f,0,1) );
	m_textExplanation1.SetXY( EXPLANATION1_X, EXPLANATION1_Y );
	m_textExplanation1.SetZ( -1 );
	m_textExplanation1.SetZoomX( EXPLANATION1_ZOOM_X );
	m_textExplanation1.SetZoomY( EXPLANATION1_ZOOM_Y );
	m_textExplanation1.SetHorizAlign( BitmapText::align_left );
	this->AddActor( &m_textExplanation1 );

	m_textExplanation2.Load( THEME->GetPathTo(FONT_FUTURISTIC) );
	m_textExplanation2.SetDiffuseColor( D3DXCOLOR(0,0.7f,0,1) );
	m_textExplanation2.SetXY( EXPLANATION2_X, EXPLANATION2_Y );
	m_textExplanation2.SetZ( -1 );
	m_textExplanation2.SetZoomX( EXPLANATION2_ZOOM_X );
	m_textExplanation2.SetZoomY( EXPLANATION2_ZOOM_Y );
	m_textExplanation2.SetHorizAlign( BitmapText::align_left );
	this->AddActor( &m_textExplanation2 );

	this->AddActor( &m_Fade );


	m_soundChange.Load( THEME->GetPathTo(SOUND_SWITCH_STYLE) );
	m_soundSelect.Load( THEME->GetPathTo(SOUND_SELECT) );



	MUSIC->Load( THEME->GetPathTo(SOUND_TRAINING_MUSIC) );
	MUSIC->Play( true );

	m_soundChange.PlayRandom();
	TweenOnScreen();
}


WindowMenuSelectStyle::~WindowMenuSelectStyle()
{
	RageLog( "WindowMenuSelectStyle::~WindowMenuSelectStyle()" );

}


void WindowMenuSelectStyle::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	RageLog( "WindowMenuSelectStyle::Input()" );

	if( m_Fade.IsClosing() )
		return;

	Window::Input( DeviceI, PadI, PlayerI );	// default input handler
}

void WindowMenuSelectStyle::HandleWindowMessage( WindowMessage SM )
{
	WindowMenu::HandleWindowMessage( SM );

	switch( SM )
	{
	case SM_GoToPrevState:
		MUSIC->Stop();
		WM->SetNewWindow( new WindowTitleMenu );
		break;
	case SM_GoToNextState:
		MUSIC->Stop();
		WM->SetNewWindow( new WindowMenuSelectMusic );
		break;
	case SM_TweenExplanation2:
		m_textExplanation2.BeginTweening( 0.7f );
		m_textExplanation2.SetTweenZoomX( EXPLANATION2_ZOOM_X );
		break;
	case SM_UpdateAnimations:
		for( int i=0; i<NUM_STYLE_DANCERS; i++ )
			m_sprDancer[i].StopAnimating();
		switch( m_GameMode )
		{
		case MODE_SINGLE:	m_sprDancer[0].StartAnimating();										break;
		case MODE_VERSUS:	m_sprDancer[1].StartAnimating();	m_sprDancer[2].StartAnimating();	break;
		case MODE_DOUBLE:	m_sprDancer[3].StartAnimating();										break;
		case MODE_COUPLE:	m_sprDancer[4].StartAnimating();	m_sprDancer[5].StartAnimating();	break;
		case MODE_SOLO:		m_sprDancer[6].StartAnimating();										break;
		}
		break;
	}
}

void WindowMenuSelectStyle::Left()
{
	if( m_GameMode == MODE_SINGLE )		// can't go left any further
		return;

	BeforeChange();
	m_GameMode = GameMode(m_GameMode-1);
	m_soundChange.Stop();
	m_soundChange.PlayRandom();
	AfterChange();
}


void WindowMenuSelectStyle::Right()
{
	if( m_GameMode == MODE_SOLO )		// can't go right any further
		return;

	BeforeChange();
	m_GameMode = GameMode(m_GameMode+1);
	m_soundChange.Stop();
	m_soundChange.PlayRandom();
	AfterChange();
}

void WindowMenuSelectStyle::Next()
{
	PREFS->m_GameMode = m_GameMode;


	this->ClearMessageQueue();

	WindowMenu::Next();

	m_soundSelect.PlayRandom();

	m_Fade.CloseWipingRight( SM_GoToNextState );

	TweenOffScreen();
}

void WindowMenuSelectStyle::Back()
{
	MUSIC->Stop();

	WindowMenu::Back();

	m_Fade.CloseWipingRight( SM_GoToPrevState );

	TweenOffScreen();
}

void WindowMenuSelectStyle::BeforeChange()
{
	switch( m_GameMode )
	{
	case MODE_SINGLE:
		m_sprPad[0].BeginTweening( TWEEN_TIME );
		m_sprPad[0].SetTweenDiffuseColor( D3DXCOLOR(0.5f,0.5f,0.5f,1) );
		m_sprDancer[0].BeginTweening( TWEEN_TIME );
		m_sprDancer[0].SetTweenDiffuseColor( COLOR_P1_NOT_SELECTED );
		break;
	case MODE_VERSUS:
		m_sprPad[1].BeginTweening( TWEEN_TIME );
		m_sprPad[1].SetTweenDiffuseColor( D3DXCOLOR(0.5f,0.5f,0.5f,1) );
		m_sprDancer[1].BeginTweening( TWEEN_TIME );
		m_sprDancer[1].SetTweenDiffuseColor( COLOR_P1_NOT_SELECTED );
		m_sprDancer[2].BeginTweening( TWEEN_TIME );
		m_sprDancer[2].SetTweenDiffuseColor( COLOR_P2_NOT_SELECTED );
		break;
	case MODE_DOUBLE:
		m_sprPad[2].BeginTweening( TWEEN_TIME );
		m_sprPad[2].SetTweenDiffuseColor( D3DXCOLOR(0.5f,0.5f,0.5f,1) );
		m_sprDancer[3].BeginTweening( TWEEN_TIME );
		m_sprDancer[3].SetTweenDiffuseColor( COLOR_P1_NOT_SELECTED );
		break;
	case MODE_COUPLE:
		m_sprPad[3].BeginTweening( TWEEN_TIME );
		m_sprPad[3].SetTweenDiffuseColor( D3DXCOLOR(0.5f,0.5f,0.5f,1) );
		m_sprDancer[4].BeginTweening( TWEEN_TIME );
		m_sprDancer[4].SetTweenDiffuseColor( COLOR_P1_NOT_SELECTED );
		m_sprDancer[5].BeginTweening( TWEEN_TIME );
		m_sprDancer[5].SetTweenDiffuseColor( COLOR_P2_NOT_SELECTED );
		break;
	case MODE_SOLO:
		m_sprPad[4].BeginTweening( TWEEN_TIME );
		m_sprPad[4].SetTweenDiffuseColor( D3DXCOLOR(0.5f,0.5f,0.5f,1) );
		m_sprDancer[6].BeginTweening( TWEEN_TIME );
		m_sprDancer[6].SetTweenDiffuseColor( COLOR_P1_NOT_SELECTED );
		break;
	}


}

void WindowMenuSelectStyle::AfterChange()
{
	this->ClearMessageQueue();

	m_textExplanation1.SetText( TEXT_EXPLANATION1[m_GameMode] );
	m_textExplanation1.SetZoomX( 0 );
	m_textExplanation1.BeginTweening( 0.6f );
	m_textExplanation1.SetTweenZoomX( EXPLANATION1_ZOOM_X );

	m_textExplanation2.SetText( TEXT_EXPLANATION2[m_GameMode] );
	m_textExplanation2.StopTweening();
	m_textExplanation2.SetZoomX( 0 );

	switch( m_GameMode )
	{
	case MODE_SINGLE:
		m_sprPad[0].BeginTweening( TWEEN_TIME );
		m_sprPad[0].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_sprDancer[0].BeginTweening( TWEEN_TIME );
		m_sprDancer[0].SetTweenDiffuseColor( COLOR_P1_SELECTED );
		m_sprDancer[0].StartAnimating();
		m_sprStyleIcon.SetState( 0 );
		break;
	case MODE_VERSUS:
		m_sprPad[1].BeginTweening( TWEEN_TIME );
		m_sprPad[1].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_sprDancer[1].BeginTweening( TWEEN_TIME );
		m_sprDancer[1].SetTweenDiffuseColor( COLOR_P1_SELECTED );
		m_sprDancer[1].StartAnimating();
		m_sprDancer[2].BeginTweening( TWEEN_TIME );
		m_sprDancer[2].SetTweenDiffuseColor( COLOR_P2_SELECTED );
		m_sprDancer[2].StartAnimating();
		m_sprStyleIcon.SetState( 1 );
		break;
	case MODE_DOUBLE:
		m_sprPad[2].BeginTweening( TWEEN_TIME );
		m_sprPad[2].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_sprDancer[3].BeginTweening( TWEEN_TIME );
		m_sprDancer[3].SetTweenDiffuseColor( COLOR_P1_SELECTED );
		m_sprDancer[3].StartAnimating();
		m_sprStyleIcon.SetState( 2 );
		break;
	case MODE_COUPLE:
		m_sprPad[3].BeginTweening( TWEEN_TIME );
		m_sprPad[3].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_sprDancer[4].BeginTweening( TWEEN_TIME );
		m_sprDancer[4].SetTweenDiffuseColor( COLOR_P1_SELECTED );
		m_sprDancer[4].StartAnimating();
		m_sprDancer[5].BeginTweening( TWEEN_TIME );
		m_sprDancer[5].SetTweenDiffuseColor( COLOR_P2_SELECTED );
		m_sprDancer[5].StartAnimating();
		m_sprStyleIcon.SetState( 3 );
		break;
	case MODE_SOLO:
		m_sprPad[4].BeginTweening( TWEEN_TIME );
		m_sprPad[4].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,1) );
		m_sprDancer[6].BeginTweening( TWEEN_TIME );
		m_sprDancer[6].SetTweenDiffuseColor( COLOR_P1_SELECTED );
		m_sprDancer[6].StartAnimating();
		m_sprStyleIcon.SetState( 4 );
		break;
	}

	this->SendWindowMessage( SM_TweenExplanation2, 1 );
	this->SendWindowMessage( SM_UpdateAnimations, TWEEN_TIME );
}

void WindowMenuSelectStyle::TweenOffScreen()
{
	for( int i=0; i<NUM_STYLE_DANCERS; i++ )
	{
		m_sprDancer[i].BeginTweening( MENU_TWEEN_TIME );
		m_sprDancer[i].SetTweenZoomY( 0 );
	}

	for( i=0; i<NUM_STYLE_PADS; i++ )
	{
		m_sprPad[i].BeginTweening( MENU_TWEEN_TIME );
		m_sprPad[i].SetTweenZoomY( 0 );
	}

	m_sprStyleIcon.BeginTweening( MENU_TWEEN_TIME );
	m_sprStyleIcon.SetTweenZoomY( 0 );

	m_textExplanation1.BeginTweening( MENU_TWEEN_TIME );
	m_textExplanation1.SetTweenZoomX( 0 );

	m_textExplanation2.BeginTweening( MENU_TWEEN_TIME );
	m_textExplanation2.SetTweenZoomX( 0 );

	m_textHelp.BeginTweening( MENU_TWEEN_TIME );
	m_textHelp.SetTweenZoomY( 0 );
}
  

void WindowMenuSelectStyle::TweenOnScreen() 
{
	for(int i=0; i<NUM_STYLE_DANCERS; i++ )
	{
		float fOrigDancerZoomY = m_sprDancer[i].GetZoomY();
		m_sprDancer[i].SetZoomY( 0 );
		m_sprDancer[i].BeginTweening( MENU_TWEEN_TIME );
		m_sprDancer[i].SetTweenZoomY( fOrigDancerZoomY );
	}

	for( i=0; i<NUM_STYLE_PADS; i++ )
	{
		float fOrigPadZoomY = m_sprPad[i].GetZoomY();
		m_sprPad[i].SetZoomY( 0 );
		m_sprPad[i].BeginTweening( MENU_TWEEN_TIME );
		m_sprPad[i].SetTweenZoomY( fOrigPadZoomY );
	}

	m_sprStyleIcon.SetZoomY( 0 );
	m_sprStyleIcon.BeginTweening( MENU_TWEEN_TIME );
	m_sprStyleIcon.SetTweenZoomY( 1 );

	float fOrigExplanation1ZoomX = m_textExplanation1.GetZoomX();
	m_textExplanation1.SetZoomX( 0 );
	m_textExplanation1.BeginTweening( 0.6f );
	m_textExplanation1.SetTweenZoomX( EXPLANATION1_ZOOM_X );

	float fOrigExplanation2ZoomX = m_textExplanation2.GetZoomX();
	m_textExplanation2.SetZoomX( 0 );
	m_textExplanation2.SetZoomX( 0 );

	m_sprStyleIcon.SetState( 0 );
	m_textExplanation1.SetText( TEXT_EXPLANATION1[m_GameMode] );
	m_textExplanation2.SetText( TEXT_EXPLANATION2[m_GameMode] );

	this->SendWindowMessage( SM_TweenExplanation2, 1 );
	this->SendWindowMessage( SM_UpdateAnimations, TWEEN_TIME );

}
