#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowOptions.h

 Desc: Select a song.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowOptions.h"
#include <assert.h>
#include "RageTextureManager.h"
#include "RageUtil.h"
#include "RageMusic.h"
#include "WindowManager.h"
#include "PrefsManager.h"
#include "WindowDancing.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "GameConstants.h"




const float HEADER_X	=		CENTER_X;
const float HEADER_Y	=		50;
const float HELP_X		=		CENTER_X;
const float HELP_Y		=		SCREEN_HEIGHT-35;
const float ITEM_GAP_X	=		12;


const WindowMessage SM_PlaySample			= WindowMessage(SM_User-4);
const WindowMessage SM_GoToPrevState		= WindowMessage(SM_User-5);
const WindowMessage SM_GoToNextState		= WindowMessage(SM_User-6);
const WindowMessage SM_JustPressedNext		= WindowMessage(SM_User-7);


WindowOptions::WindowOptions( CString sBackgroundPath, CString sTopEdgePath ) :
	WindowMenu( 
		sBackgroundPath, 
		sTopEdgePath, 
		ssprintf("%s %s to change line   %s %s change sort  then press NEXT", CString(char(3)), CString(char(4)), CString(char(1)), CString(char(2)) )
	)
{
	RageLog( "WindowOptions::WindowOptions()" );

	//m_SoundChange.Load(	THEME->GetPathTo(SOUND_CHANGE) );
	m_SoundNext.Load(	THEME->GetPathTo(SOUND_SELECT) );

	m_Player1SelectedHighlight.SetDiffuseColor( PLAYER_COLOR[PLAYER_1] );
	m_Player2SelectedHighlight.SetDiffuseColor( PLAYER_COLOR[PLAYER_2] );

	for( int i=0; i<MAX_OPTION_LINES; i++ )
	{
		m_Options[0][i] = 0;
		m_Options[1][i] = 0;

		m_Player1OptionsUnderscore[i].SetDiffuseColor( PLAYER_COLOR[PLAYER_1] );
		m_Player2OptionsUnderscore[i].SetDiffuseColor( PLAYER_COLOR[PLAYER_2] );
	}
	m_Player1CurrentRow = 0;
	m_Player2CurrentRow = 0;

	m_Wipe.OpenWipingRight(SM_None);
}

void WindowOptions::Init( InputMode im, OptionLine optionLines[], int iNumOptionLines )
{
	RageLog( "WindowOptions::Set()" );


	m_InputMode = im;
	m_OptionLines = optionLines;
	m_iNumOptionLines = iNumOptionLines;


	this->ImportOptions();
	InitTextOptions();
	PositionUnderlines();
	PositionHighlights();
}

WindowOptions::~WindowOptions()
{
	RageLog( "WindowOptions::~WindowOptions()" );

}

void WindowOptions::InitTextOptions()
{
	// init m_textOptions from optionLines
	for( int i=0; i<m_iNumOptionLines; i++ )	// foreach options line
	{
		OptionLine &optline = m_OptionLines[i];

		//float fYOffsetFromCenter =  - (m_iNumOptionLines-1)/2.0f );
		float fYPosition = 100.0f + 40*i;
		float fXLeftEdge = 80;		// increment this as we draw each line

		m_textOptionLineTitles[i].Load( THEME->GetPathTo(FONT_OUTLINE) );
		m_textOptionLineTitles[i].SetText( optline.szTitle );
		m_textOptionLineTitles[i].SetXY( fXLeftEdge, fYPosition );
		m_textOptionLineTitles[i].SetZoom( 0.75f );
		m_textOptionLineTitles[i].SetVertAlign( Actor::align_middle );
		fXLeftEdge += 70;

		// init all text in this line and count the width of the line
		float fLineWidth = 0;
		for( int j=0; j<optline.iNumOptions; j++ )	// for each option on this line
		{
			m_textOptions[i][j].Load( THEME->GetPathTo(FONT_NORMAL) );
			m_textOptions[i][j].SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
			m_textOptions[i][j].SetText( optline.szOptionsText[j] );
			m_textOptions[i][j].SetZoom( 0.65f );
		}
		
		// set the XY position of each item in the line
		for( j=0; j<optline.iNumOptions; j++ )	// for each option on this line
		{
			float fXOffsetFromCenter = 100 * ( j - (optline.iNumOptions-1)/2.0f );
			float fItemWidth = m_textOptions[i][j].GetWidestLineWidthInSourcePixels() * 0.80f;
			float fXPosition = fXLeftEdge + fItemWidth/2.0f;
			m_textOptions[i][j].SetXY( fXPosition, fYPosition );
			fXLeftEdge += fItemWidth + ITEM_GAP_X;
		}
	}
}


const int RECTANGLE_BORDER = 2;
const int RECTANGLE_HEIGHT = 26;

void WindowOptions::PositionUnderlines()
{
	int iOptionInRow;
	float width, x, y;

	// Set the position of the underscores showing the current choice for each option line.
	for( int i=0; i<m_iNumOptionLines; i++ )	// foreach options line
	{
		iOptionInRow = m_Options[0][i];
		width = m_textOptions[i][iOptionInRow].GetWidestLineWidthInSourcePixels() * 0.80f;
		x = m_textOptions[i][iOptionInRow].GetX();
		y = m_textOptions[i][iOptionInRow].GetY();
		width += RECTANGLE_BORDER*4;
		x -= RECTANGLE_BORDER;
		y += RECTANGLE_BORDER;
		y += RECTANGLE_HEIGHT/2;	// bump it down so it looks like an underline
		m_Player1OptionsUnderscore[i].SetZoomX( width );
		m_Player1OptionsUnderscore[i].SetZoomY( RECTANGLE_BORDER*2 );
		m_Player1OptionsUnderscore[i].SetXY( x, y );
		
		if( m_InputMode == INPUTMODE_2PLAYERS )
		{
			iOptionInRow = m_Options[1][i];
			width = m_textOptions[i][iOptionInRow].GetWidestLineWidthInSourcePixels() * 0.80f;
			x = m_textOptions[i][iOptionInRow].GetX();
			y = m_textOptions[i][iOptionInRow].GetY();
			width += RECTANGLE_BORDER*4;
			x += RECTANGLE_BORDER;
			y -= RECTANGLE_BORDER;
			y += RECTANGLE_HEIGHT/2;	// bump it down so it looks like an underline
			m_Player2OptionsUnderscore[i].SetZoomX( width );
			m_Player2OptionsUnderscore[i].SetZoomY( RECTANGLE_BORDER*2 );
			m_Player2OptionsUnderscore[i].SetXY( x, y );
		}
	}
}


void WindowOptions::PositionHighlights()
{
	int iOptionInRow;
	float width, x, y;

	// Set the position of the highlight showing the current option the user is changing.
	iOptionInRow = m_Options[0][m_Player1CurrentRow];
	width = m_textOptions[m_Player1CurrentRow][iOptionInRow].GetWidestLineWidthInSourcePixels() * 0.80f;
	x = m_textOptions[m_Player1CurrentRow][iOptionInRow].GetX();
	y = m_textOptions[m_Player1CurrentRow][iOptionInRow].GetY();
	width += RECTANGLE_BORDER*4;
	x -= RECTANGLE_BORDER;
	y += RECTANGLE_BORDER;
	m_Player1SelectedHighlight.SetZoomX( width );
	m_Player1SelectedHighlight.SetZoomY( RECTANGLE_HEIGHT + RECTANGLE_BORDER*2 );
	m_Player1SelectedHighlight.SetXY( x, y );

	if( m_InputMode == INPUTMODE_2PLAYERS )
	{
		// Set the position of the highlight showing the current option the user is changing.
		iOptionInRow = m_Options[1][m_Player2CurrentRow];
		width = m_textOptions[m_Player2CurrentRow][iOptionInRow].GetWidestLineWidthInSourcePixels() * 0.80f;
		x = m_textOptions[m_Player2CurrentRow][iOptionInRow].GetX();
		y = m_textOptions[m_Player2CurrentRow][iOptionInRow].GetY();
		width += RECTANGLE_BORDER*4;
		x += RECTANGLE_BORDER;
		y -= RECTANGLE_BORDER;
		m_Player2SelectedHighlight.SetZoomX( width );
		m_Player2SelectedHighlight.SetZoomY( RECTANGLE_HEIGHT + RECTANGLE_BORDER*2 );
		m_Player2SelectedHighlight.SetXY( x, y );
	}
}

void WindowOptions::TweenP1Highlight()
{
	int iOptionInRow;
	float width, x, y;

	// Set the position of the highlight showing the current option the user is changing.
	iOptionInRow = m_Options[0][m_Player1CurrentRow];
	width = m_textOptions[m_Player1CurrentRow][iOptionInRow].GetWidestLineWidthInSourcePixels() * 0.80f;
	x = m_textOptions[m_Player1CurrentRow][iOptionInRow].GetX();
	y = m_textOptions[m_Player1CurrentRow][iOptionInRow].GetY();
	width += RECTANGLE_BORDER*4;
	x -= RECTANGLE_BORDER;
	y += RECTANGLE_BORDER;
	m_Player1SelectedHighlight.BeginTweening( 0.2f );
	m_Player1SelectedHighlight.SetTweenZoomX( width );
	m_Player1SelectedHighlight.SetTweenZoomY( RECTANGLE_HEIGHT + RECTANGLE_BORDER*2 );
	m_Player1SelectedHighlight.SetTweenXY( x, y );
}

void WindowOptions::TweenP2Highlight()
{
	int iOptionInRow;
	float width, x, y;

	// Set the position of the highlight showing the current option the user is changing.
	iOptionInRow = m_Options[1][m_Player2CurrentRow];
	width = m_textOptions[m_Player2CurrentRow][iOptionInRow].GetWidestLineWidthInSourcePixels() * 0.80f;
	x = m_textOptions[m_Player2CurrentRow][iOptionInRow].GetX();
	y = m_textOptions[m_Player2CurrentRow][iOptionInRow].GetY();
	width += RECTANGLE_BORDER*4;
	x += RECTANGLE_BORDER;
	y -= RECTANGLE_BORDER;
	m_Player2SelectedHighlight.BeginTweening( 0.2f );
	m_Player2SelectedHighlight.SetTweenZoomX( width );
	m_Player2SelectedHighlight.SetTweenZoomY( RECTANGLE_HEIGHT + RECTANGLE_BORDER*2 );
	m_Player2SelectedHighlight.SetTweenXY( x, y );
}

void WindowOptions::Update( float fDeltaTime )
{
	WindowMenu::Update( fDeltaTime );

	//RageLog( "WindowOptions::Update(%f)", fDeltaTime );

	m_Player1SelectedHighlight.Update( fDeltaTime );
	m_Player2SelectedHighlight.Update( fDeltaTime );

	for( int i=0; i<m_iNumOptionLines; i++ )	// foreach options line
	{
		OptionLine &optline = m_OptionLines[i];

		m_textOptionLineTitles[i].Update( fDeltaTime );
		for( int j=0; j<optline.iNumOptions; j++ )	// for each option on this line
		{
			m_textOptions[i][j].Update( fDeltaTime );
		}

		m_Player1OptionsUnderscore[i].Update( fDeltaTime );
		m_Player2OptionsUnderscore[i].Update( fDeltaTime );
	}

	m_Wipe.Update( fDeltaTime );
}

void WindowOptions::RenderPrimitives()
{
	WindowMenu::RenderPrimitives();

	m_Player1SelectedHighlight.Draw();
	if( m_InputMode == INPUTMODE_2PLAYERS )
		m_Player2SelectedHighlight.Draw();

	for( int i=0; i<m_iNumOptionLines; i++ )	// foreach options line
	{
		m_textOptionLineTitles[i].Draw();

		m_Player1OptionsUnderscore[i].Draw();
		if( m_InputMode == INPUTMODE_2PLAYERS )
			m_Player2OptionsUnderscore[i].Draw();

		OptionLine &optline = m_OptionLines[i];
		for( int j=0; j<optline.iNumOptions; j++ )	// for each option on this line
		{
			m_textOptions[i][j].Draw();
		}

	}

	m_Wipe.Draw();
}

void WindowOptions::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	if( m_Wipe.IsClosing() )
		return;

	// default input handler
	WindowMenu::Input( DeviceI, PadI, PlayerI );
}

void WindowOptions::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_GoToPrevState:
		this->ExportOptions();
		this->GoToPrevState();
		break;
	case SM_GoToNextState:
		this->ExportOptions();
		this->GoToNextState();
		break;
	}
}


void WindowOptions::OnChange()
{
	PositionUnderlines();
}


void WindowOptions::Back()
{
	WindowMenu::Back();

	m_Wipe.CloseWipingLeft( SM_GoToPrevState );
}


void WindowOptions::Next()
{
	WindowMenu::Next();

	m_SoundNext.PlayRandom();
	m_Wipe.CloseWipingRight( SM_GoToNextState );
}

void WindowOptions::Player1Left() 
{
	if( m_Options[0][m_Player1CurrentRow] == 0 )	// can't go left any more
		return;

	m_Options[0][m_Player1CurrentRow]--;
	
	TweenP1Highlight();
	OnChange();
}

void WindowOptions::Player1Right() 
{
	if( m_Options[0][m_Player1CurrentRow] == m_OptionLines[m_Player1CurrentRow].iNumOptions-1 )	// can't go right any more
		return;
	
	m_Options[0][m_Player1CurrentRow]++;
	
	TweenP1Highlight();
	OnChange();
}

void WindowOptions::Player1Up() 
{
	if( m_Player1CurrentRow == 0 )	
		m_Player1CurrentRow = m_iNumOptionLines-1; // wrap around
	else
		m_Player1CurrentRow--;
	
	TweenP1Highlight();
	OnChange();
}

void WindowOptions::Player1Down() 
{
	if( m_Player1CurrentRow == m_iNumOptionLines-1 )	
		m_Player1CurrentRow = 0; // wrap around
	else
		m_Player1CurrentRow++;

	TweenP1Highlight();
	OnChange();
}


void WindowOptions::Player2Left() 
{
	switch( m_InputMode )
	{
	case INPUTMODE_P1_ONLY:
		return;
	case INPUTMODE_BOTH:
		Player1Left();
		return;
	case INPUTMODE_2PLAYERS:
		break;	// fall through
	}

	if( m_Options[1][m_Player2CurrentRow] == 0 )	// can't go left any more
		return;

	m_Options[1][m_Player2CurrentRow]--;
	
	TweenP2Highlight();
	OnChange();
}

void WindowOptions::Player2Right() 
{
	switch( m_InputMode )
	{
	case INPUTMODE_P1_ONLY:
		return;
	case INPUTMODE_BOTH:
		Player1Right();
		return;
	case INPUTMODE_2PLAYERS:
		break;	// fall through
	}

	if( m_Options[1][m_Player2CurrentRow] == m_OptionLines[m_Player2CurrentRow].iNumOptions-1 )	// can't go right any more
		return;
	
	m_Options[1][m_Player2CurrentRow]++;
	
	TweenP2Highlight();
	OnChange();
}

void WindowOptions::Player2Up() 
{
	switch( m_InputMode )
	{
	case INPUTMODE_P1_ONLY:
		return;
	case INPUTMODE_BOTH:
		Player1Up();
		return;
	case INPUTMODE_2PLAYERS:
		break;	// fall through
	}

	if( m_Player2CurrentRow == 0 )	// can't go up any more
		return;

	m_Player2CurrentRow--;
	
	TweenP2Highlight();
	OnChange();
}

void WindowOptions::Player2Down() 
{
	switch( m_InputMode )
	{
	case INPUTMODE_P1_ONLY:
		return;
	case INPUTMODE_BOTH:
		Player1Down();
		return;
	case INPUTMODE_2PLAYERS:
		break;	// fall through
	}

	if( m_Player2CurrentRow == m_iNumOptionLines-1 )	// can't go down any more
		return;

	m_Player2CurrentRow++;

	TweenP2Highlight();
	OnChange();
}



