/*
-----------------------------------------------------------------------------
 File: WindowOptions.h

 Desc: Select a song.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/
#ifndef _WindowOptions_H_
#define _WindowOptions_H_



#include "Window.h"
#include "Sprite.h"
#include "BitmapText.h"
#include "RandomSample.h"
#include "TransitionFade.h"
#include "Rectangle.h"
#include "WindowMenu.h"


const int MAX_OPTION_LINES = 20;
const int MAX_OPTIONS_PER_LINE = 20;


// used to pass menu info into this class
struct OptionLine {
	LPCTSTR szTitle;
	int iNumOptions;
	LPCTSTR szOptionsText[10];
};

enum InputMode { INPUTMODE_P1_ONLY, INPUTMODE_2PLAYERS, INPUTMODE_BOTH };	// both means both players control the same cursor


class WindowOptions : public WindowMenu
{
public:
	WindowOptions( CString sBackgroundPath, CString sTopEdgePath );
	void Init( InputMode im, OptionLine optionLines[], int iNumOptionLines );
	virtual ~WindowOptions();
	virtual void Update( float fDeltaTime );
	virtual void RenderPrimitives();
	virtual void Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI );
	virtual void HandleWindowMessage( WindowMessage SM );

protected:
	virtual void ImportOptions() = 0;
	virtual void ExportOptions() = 0;
	void InitTextOptions();
	void PositionUnderlines();
	void PositionHighlights();
	void TweenP1Highlight();
	void TweenP2Highlight();
	void OnChange();

	void Back();
	void Next();

	virtual void GoToNextState() = 0;
	virtual void GoToPrevState() = 0;

	void Player1Left();
	void Player1Right();
	void Player1Up();
	void Player1Down();

	void Player2Left();
	void Player2Right();
	void Player2Up();
	void Player2Down();

	InputMode m_InputMode;

//	typedef OptionLine* POptionLine[];

	OptionLine* m_OptionLines;
	int m_iNumOptionLines;


	BitmapText	m_textOptionLineTitles[MAX_OPTION_LINES];

	BitmapText	m_textOptions[MAX_OPTION_LINES][MAX_OPTIONS_PER_LINE];	// this array has to be big enough to hold all of the options

	int m_Options[NUM_PLAYERS][MAX_OPTION_LINES];

	int m_Player1CurrentRow;
	int m_Player2CurrentRow;

	RectangleActor m_Player1OptionsUnderscore[MAX_OPTION_LINES];
	RectangleActor m_Player2OptionsUnderscore[MAX_OPTION_LINES];

	RectangleActor m_Player1SelectedHighlight;
	RectangleActor m_Player2SelectedHighlight;

	RandomSample m_SoundChange;
	RandomSample m_SoundNext;

	TransitionFade		m_Wipe;
};




#endif