#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowPlayerOptions.h

 Desc: Select a song.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowPlayerOptions.h"
#include <assert.h>
#include "RageTextureManager.h"
#include "RageUtil.h"
#include "RageMusic.h"
#include "WindowManager.h"
#include "WindowDancing.h"
#include "WindowSongOptions.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "WindowMenuSelectMusic.h"



OptionLine g_PlayerOptionsLines[] = {
	{ "Speed", 6, {"x1","x1.5","x2","x3","x5","x8"} },	
	{ "Effect", 6, {"OFF","BOOST","WAVE", "DRUNK", "DIZZY", "SPACE"} },	
	{ "Appear", 4, {"VISIBLE","HIDDEN","SUDDEN","STEALTH"} },	
	{ "Turn", 5, {"OFF","MIRROR","LEFT","RIGHT","SHUFFLE"} },	
	{ "Little", 2, {"OFF","ON"} },	
	{ "Scroll", 2, {"STANDARD","REVERSE"} },	
	{ "Color", 4, {"ARCADE","NOTE","FLAT","PLAIN"} },	
	{ "Freezes", 2, {"OFF","ON"} },	
};
const int NUM_PLAYER_OPTIONS_LINES = sizeof(g_PlayerOptionsLines)/sizeof(OptionLine);



WindowPlayerOptions::WindowPlayerOptions() :
	WindowOptions(
		THEME->GetPathTo(GRAPHIC_PLAYER_OPTIONS_BACKGROUND),
		THEME->GetPathTo(GRAPHIC_PLAYER_OPTIONS_TOP_EDGE)
		)
{
	RageLog( "WindowPlayerOptions::WindowPlayerOptions()" );
	
	Init( 
		INPUTMODE_2PLAYERS, 
		g_PlayerOptionsLines, 
		NUM_PLAYER_OPTIONS_LINES
		);
}


void WindowPlayerOptions::ImportOptions()
{
	for( int p=0; p<NUM_PLAYERS; p++ )
	{
		PlayerOptions &po = PREFS->m_PlayerOptions[p];

		
		if(		 po.m_fArrowScrollSpeed == 1.0f )
			m_Options[p][0] = 0;
		else if( po.m_fArrowScrollSpeed == 1.5f )
			m_Options[p][0] = 1;
		else if( po.m_fArrowScrollSpeed == 2.0f )
			m_Options[p][0] = 2;
		else if( po.m_fArrowScrollSpeed == 3.0f )
			m_Options[p][0] = 3;
		else if( po.m_fArrowScrollSpeed == 5.0f )
			m_Options[p][0] = 4;
		else if( po.m_fArrowScrollSpeed == 8.0f )
			m_Options[p][0] = 5;
		else
			m_Options[p][0] = 0;


		m_Options[p][1] = po.m_EffectType;

		m_Options[p][2] = po.m_AppearanceType;

		m_Options[p][3] = po.m_TurnType;

		m_Options[p][4] = po.m_bLittle ? 1 : 0;
		
		m_Options[p][5] = po.m_bReverseScroll ? 1 : 0 ;

		m_Options[p][6] = po.m_ColorType;

		m_Options[p][7] = po.m_bAllowFreezeArrows ? 1 : 0;
	}
}

void WindowPlayerOptions::ExportOptions()
{
	for( int p=0; p<NUM_PLAYERS; p++ )
	{
		PlayerOptions &po = PREFS->m_PlayerOptions[p];

		switch( m_Options[p][0] )
		{
		case 0:	po.m_fArrowScrollSpeed = 1.0f;	break;
		case 1:	po.m_fArrowScrollSpeed = 1.5f;	break;
		case 2:	po.m_fArrowScrollSpeed = 2.0f;	break;
		case 3:	po.m_fArrowScrollSpeed = 3.0f;	break;
		case 4:	po.m_fArrowScrollSpeed = 5.0f;	break;
		case 5:	po.m_fArrowScrollSpeed = 8.0f;	break;
		}


		po.m_EffectType = (PlayerOptions::EffectType)m_Options[p][1];

		po.m_AppearanceType = (PlayerOptions::AppearanceType)m_Options[p][2];

		po.m_TurnType = (PlayerOptions::TurnType)m_Options[p][3];

		po.m_bLittle = m_Options[p][4] == 1;
		// removed this variable//po.m_bFlat	 = m_Options[p][4] == 2;
		
		po.m_bReverseScroll	= (m_Options[p][5] == 1);

		po.m_ColorType = (PlayerOptions::ColorType)m_Options[p][6];

		po.m_bAllowFreezeArrows = (m_Options[p][7] == 1);
	}
}

void WindowPlayerOptions::GoToPrevState()
{
	WM->SetNewWindow( new WindowMenuSelectMusic );
}

void WindowPlayerOptions::GoToNextState()
{
	/*
	// find out if the Next button is being held down on any of the pads
	bool bIsHoldingNext;
	bIsHoldingNext = false;
	for( int pad=0; pad<NUM_PADS; pad++ )
	{
		PadInput pi( (PadNumber)pad, BUTTON_NEXT );
		if( GAMEINFO->IsButtonDown( pi ) )
			bIsHoldingNext = true;
	}
	*/
	//if( bIsHoldingNext )
		WM->SetNewWindow( new WindowSongOptions );
	//else
	//	WM->SetNewWindow( new WindowDancing );
}



