#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowPrompt.h

 Desc: Area for testing.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/


#include "WindowPrompt.h"
#include "PrefsManager.h"
#include "WindowManager.h"
#include "RageMusic.h"
#include "WindowTitleMenu.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"

WindowPrompt::WindowPrompt( CString sText, PromptType pt, bool* pbAnswer )
{

	static bool bThrowAway;

	if( pbAnswer == NULL )
		pbAnswer = &bThrowAway;


	m_PromptType = pt;
	m_pbAnswer = pbAnswer;

	ASSERT( !(pt == PROMPT_YES_NO  &&  pt == NULL) );

	m_Fade.SetTransitionTime( 0.5f );
	m_Fade.SetColor( D3DXCOLOR(0,0,0,0.7f) );
	m_Fade.SetOpened();
	m_Fade.CloseWipingRight();
	m_Fade.SetZ(-2);
	this->AddActor( &m_Fade );

	CStringArray arrayTextLines;
	split( sText, "\n", arrayTextLines );
	for( int i=0; i<arrayTextLines.GetSize(); i++ )
	{
		m_textQuestion[i].Load( THEME->GetPathTo(FONT_OUTLINE) );
		m_textQuestion[i].SetText( arrayTextLines[i] );
		m_textQuestion[i].SetXY( CENTER_X, CENTER_Y-50 + i*27 - arrayTextLines.GetSize()*27/2 );
		m_textQuestion[i].SetZ(-2);
		this->AddActor( &m_textQuestion[i] );
	}

	m_rectAnswerBox.SetDiffuseColor( D3DXCOLOR(0.5f,0.5f,1.0f,0.7f) );
	m_rectAnswerBox.SetXY( m_textAnswer[*m_pbAnswer].GetX(), m_textAnswer[*m_pbAnswer].GetY() );
	m_rectAnswerBox.SetZoomX( m_textAnswer[*m_pbAnswer].GetWidestLineWidthInSourcePixels()+10 );
	m_rectAnswerBox.SetZoomY( 30 );
	m_rectAnswerBox.SetZ(-2);
	this->AddActor( &m_rectAnswerBox );

	m_textAnswer[0].Load( THEME->GetPathTo(FONT_OUTLINE) );
	m_textAnswer[1].Load( THEME->GetPathTo(FONT_OUTLINE) );
	m_textAnswer[0].SetY( CENTER_Y+120 );
	m_textAnswer[1].SetY( CENTER_Y+120 );
	m_textAnswer[0].SetZ(-2);
	m_textAnswer[1].SetZ(-2);
	this->AddActor( &m_textAnswer[0] );
	this->AddActor( &m_textAnswer[1] );

	

	switch( m_PromptType )
	{
	case PROMPT_OK:
		m_textAnswer[0].SetText( "OK" );
		m_textAnswer[0].SetX( CENTER_X );
		break;
	case PROMPT_YES_NO:
		m_textAnswer[0].SetText( "NO" );
		m_textAnswer[1].SetText( "YES" );
		m_textAnswer[0].SetX( CENTER_X+50 );
		m_textAnswer[1].SetX( CENTER_X-50 );
		break;
	}

	m_rectAnswerBox.SetXY( m_textAnswer[*m_pbAnswer].GetX(), m_textAnswer[*m_pbAnswer].GetY() );
	m_rectAnswerBox.SetZoomX( m_textAnswer[*m_pbAnswer].GetWidestLineWidthInSourcePixels()+10 );

	m_textAnswer[*m_pbAnswer].SetEffectGlowing();


	m_soundSelect.Load( THEME->GetPathTo(SOUND_SELECT) );
}

void WindowPrompt::Update( float fDeltaTime )
{
	Window::Update( fDeltaTime );
}

void WindowPrompt::RenderPrimitives()
{
	Window::RenderPrimitives();
}

void WindowPrompt::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	if( m_Fade.IsOpening() )
		return;

	Window::Input( DeviceI, PadI, PlayerI );
}

void WindowPrompt::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_DoneClosingWipingLeft:
		break;
	case SM_DoneClosingWipingRight:
		break;
	case SM_DoneOpeningWipingLeft:
		break;
	case SM_DoneOpeningWipingRight:
		WM->PopTopWindow();
		break;
	}
}

void WindowPrompt::Left()
{
	if( m_PromptType == PROMPT_OK )
		return;

	Right();
}

void WindowPrompt::Right()
{
	if( m_PromptType == PROMPT_OK )
		return;

	m_textAnswer[*m_pbAnswer].SetEffectNone();
	*m_pbAnswer = !*m_pbAnswer;
	m_textAnswer[*m_pbAnswer].SetEffectGlowing();

	m_rectAnswerBox.BeginTweening( 0.2f );
	m_rectAnswerBox.SetTweenXY( m_textAnswer[*m_pbAnswer].GetX(), m_textAnswer[*m_pbAnswer].GetY() );
	m_rectAnswerBox.SetTweenZoomX( m_textAnswer[*m_pbAnswer].GetWidestLineWidthInSourcePixels()+10 );

}

void WindowPrompt::Next()
{
	m_Fade.OpenWipingRight( SM_DoneOpeningWipingRight );

	m_textTitle.BeginTweening( 0.2f );
	m_textTitle.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );

	for( int i=0; i<NUM_QUESTION_LINES; i++ )
	{
		m_textQuestion[i].BeginTweening( 0.2f );
		m_textQuestion[i].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );
	}

	m_rectAnswerBox.BeginTweening( 0.2f );
	m_rectAnswerBox.SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );

	m_textAnswer[*m_pbAnswer].SetEffectNone();

	m_textAnswer[0].BeginTweening( 0.2f );
	m_textAnswer[0].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );
	m_textAnswer[1].BeginTweening( 0.2f );
	m_textAnswer[1].SetTweenDiffuseColor( D3DXCOLOR(1,1,1,0) );


	m_soundSelect.PlayRandom();
}

void WindowPrompt::Back()
{

}
