#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowSongOptions.h

 Desc: Select a song.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowSongOptions.h"
#include <assert.h>
#include "RageTextureManager.h"
#include "RageUtil.h"
#include "RageMusic.h"
#include "WindowManager.h"
#include "WindowMenuSelectMusic.h"
#include "WindowDancing.h"
#include "ScreenDimensions.h"
#include "WindowPlayerOptions.h"



OptionLine g_SongOptionsLines[] = {
	{ "Fail", 3, {"ARCADE","END OF SONG","OFF"} },	
	{ "Assist", 2, {"OFF","TICK"} },	
	{ "Rate", 7, {"70%","80%","90%","100%","110%","120%","130%"} },	
	{ "Pitch", 7, {"-1.5","-1","-0.5","+0","+0.5","+1","+1.5"} },	
};
const int NUM_SONG_OPTIONS_LINES = sizeof(g_SongOptionsLines)/sizeof(OptionLine);

enum SONGOPTION {
	SO_FAIL = 0,
	SO_ASSIST,
	SO_RATE,
	SO_PITCH
};



WindowSongOptions::WindowSongOptions() :
	WindowOptions(
		THEME->GetPathTo(GRAPHIC_MUSIC_OPTIONS_BACKGROUND),
		THEME->GetPathTo(GRAPHIC_MUSIC_OPTIONS_TOP_EDGE)
		)
{
	RageLog( "WindowSongOptions::WindowSongOptions()" );

	Init( INPUTMODE_BOTH, 
		g_SongOptionsLines, 
		NUM_SONG_OPTIONS_LINES
		);
}

void WindowSongOptions::ImportOptions()
{
	SongOptions &so = PREFS->m_SongOptions;

	m_Options[0][SO_FAIL] = so.m_FailType;
	m_Options[0][SO_ASSIST] = so.m_AssistType;

	if(		 so.m_fMusicRate == 0.70f )		m_Options[0][SO_RATE] = 0;
	else if( so.m_fMusicRate == 0.80f )		m_Options[0][SO_RATE] = 1;
	else if( so.m_fMusicRate == 0.90f )		m_Options[0][SO_RATE] = 2;
	else if( so.m_fMusicRate == 1.00f )		m_Options[0][SO_RATE] = 3;
	else if( so.m_fMusicRate == 1.10f )		m_Options[0][SO_RATE] = 4;
	else if( so.m_fMusicRate == 1.20f )		m_Options[0][SO_RATE] = 5;
	else if( so.m_fMusicRate == 1.30f )		m_Options[0][SO_RATE] = 6;
	else									m_Options[0][SO_RATE] = 3;

	m_Options[0][SO_PITCH] = 3;
}

void WindowSongOptions::ExportOptions()
{
	SongOptions &so = PREFS->m_SongOptions;

	so.m_FailType = (SongOptions::FailType)m_Options[0][SO_FAIL];
	so.m_AssistType = (SongOptions::AssistType)m_Options[0][SO_ASSIST];

	switch( m_Options[0][SO_RATE] )
	{
	case 0:	so.m_fMusicRate = 0.70f;	break;
	case 1:	so.m_fMusicRate = 0.80f;	break;
	case 2:	so.m_fMusicRate = 0.90f;	break;
	case 3:	so.m_fMusicRate = 1.00f;	break;
	case 4:	so.m_fMusicRate = 1.10f;	break;
	case 5:	so.m_fMusicRate = 1.20f;	break;
	case 6:	so.m_fMusicRate = 1.30f;	break;
	default:	ASSERT( false );
	}


}

void WindowSongOptions::GoToPrevState()
{
	WM->SetNewWindow( new WindowPlayerOptions );
}

void WindowSongOptions::GoToNextState()
{
	MUSIC->Stop();

	WM->SetNewWindow( new WindowDancing );
}




