#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowSynchronize.h

 Desc: The music plays, the notes scroll, and the Player is pressing buttons.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowSynchronize.h"
#include "SongManager.h"
#include "WindowManager.h"

#include "WindowMenuSelectMusic.h"
#include "WindowMenuResults.h"
#include "ScreenDimensions.h"
#include "ThemeManager.h"
#include "WindowSynchronizeMenu.h"

//
// Defines specific to GameWindowTitleMenu
//


const float MAX_SECONDS_CAN_BE_OFF_BY	=	0.20f;
const float GRAY_ARROW_Y					= ARROW_SIZE * 1.5;

const float DEBUG_X			=	SCREEN_LEFT + 10;
const float DEBUG_Y			=	CENTER_Y-100;

const float HELP_X				= SCREEN_RIGHT - 10;
const float HELP_Y				= DEBUG_Y;

const WindowMessage	SM_StartMusic			= WindowMessage(SM_User+101);
const WindowMessage	SM_MusicEnded			= WindowMessage(SM_User+102);
const WindowMessage SM_GoToPrevState		=	WindowMessage(SM_User+1);
const WindowMessage SM_GoToNextState		=	WindowMessage(SM_User+2);


WindowSynchronize::WindowSynchronize()
{
	RageLog( "WindowSynchronize::WindowSynchronize()" );

	m_pSong = SONGS->m_pCurSong;


	m_Background.LoadFromSong( SONGS->m_pCurSong, true );
	m_Background.SetDiffuseColor( D3DXCOLOR(0.4f,0.4f,0.4f,1) );
	this->AddActor( &m_Background );


	m_textDebug.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textDebug.SetXY( DEBUG_X, DEBUG_Y );
	m_textDebug.SetHorizAlign( Actor::align_left );
	m_textDebug.SetZoom( 0.5f );
	m_textDebug.SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
	this->AddActor( &m_textDebug );


	m_textHelp.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textHelp.SetXY( HELP_X, HELP_Y );
	m_textHelp.SetHorizAlign( Actor::align_right );
	m_textHelp.SetZoom( 0.5f );
	m_textHelp.SetText( 
		ssprintf(
			"Keys:\n"
			"Insert - decrease offset by 0.01 seconds\n"
			"Delete - increase offset by 0.01 seconds\n"
			"Home - decrease offset by 0.10 seconds\n"
			"End - increase offset by 0.10 seconds\n"
			"Page Up - decrease offset by 1 beat\n"
			"Page Down - increase offset by 1 beat\n"
			"Escape - leave synchronize mode\n"
		)
	);
	this->AddActor( &m_textHelp );


	m_Player.SetXY( CENTER_X, 0 );
	CArray<Steps*, Steps*> arraySteps; 
	SONGS->m_pCurSong->GetStepsThatMatchGameMode( MODE_SINGLE, arraySteps );
	SortStepsArrayByDifficultyClass( arraySteps );
	Steps* pSteps = arraySteps[arraySteps.GetSize()-1];	// hardest steps available
	PlayerOptions po;
	po.m_fArrowScrollSpeed = 2;
	m_Player.Load( GetStyle(MODE_SINGLE), PLAYER_1, *pSteps, po );
	this->AddActor( &m_Player );


	m_Fade.SetClosed();
	this->AddActor( &m_Fade );



	m_soundMusic.Load( m_pSong->GetMusicPath() );


	m_soundAssistTick.Load( THEME->GetPathTo(SOUND_ASSIST) );

	this->SendWindowMessage( SM_StartMusic, 0.5f );

	m_Fade.OpenWipingRight();
}

WindowSynchronize::~WindowSynchronize()
{
	RageLog( "WindowSynchronize::~WindowSynchronize()" );
	m_soundMusic.Stop();
}


void WindowSynchronize::Update( float fDeltaTime )
{
	//RageLog( "WindowSynchronize::Update(%f)", fDeltaTime );
	Window::Update( fDeltaTime );


	float fSongBeat, fBPS;
	float fPositionSeconds = m_soundMusic.GetPositionSeconds();
	fPositionSeconds += 0.08f;	// HACK:  The assist ticks are playing too late, so make them play a tiny bit earlier
	m_pSong->GetBeatAndBPSFromElapsedTime( fPositionSeconds, fSongBeat, fBPS );
	RageLog( "fPositionSeconds = %f, fSongBeat = %f, fBPS = %f", fPositionSeconds, fSongBeat, fBPS );


	m_Player.Update( fDeltaTime, fSongBeat, 0.35f );

	

	int iIndexNow = BeatToStepIndexNotRounded( fSongBeat );	
	static iIndexLastCrossed = 0;
	if( iIndexNow != iIndexLastCrossed )	// we crossed an index boundary
	{
		for( int i=iIndexLastCrossed+1; i<=iIndexNow; i++ )
		{
			if( (i%ELEMENTS_PER_BEAT) == 0 )		// we crossed a beat
			{
				//m_soundAssistTick.PlayRandom();
				break;
			}
		}
	}

	iIndexLastCrossed = iIndexNow;
	RageLog( "iIndexLastCrossed = %d", iIndexLastCrossed );


	m_textDebug.SetText( 
		ssprintf(
			"Offset is %f\n"
			"Elapsed seconds is %f\n"
			"Song beat is %f\n"
			"BPM is %f\n"
			"Playback rate is %f",
			m_pSong->GetBeatOffsetInSeconds(),
			fPositionSeconds, 
			fSongBeat, 
			fBPS*60,
			m_soundMusic.GetPlaybackRate()
		) 
	);
	
	//RageLog( "m_fOffsetInBeats = %f, m_fBeatsPerSecond = %f, m_Music.GetPositionSeconds = %f", m_fOffsetInBeats, m_fBeatsPerSecond, m_Music.GetPositionSeconds() );


	// Check for end of song
	if( m_soundMusic.GetPositionSeconds() > m_soundMusic.GetLengthSeconds()-2 )
	{
		MUSIC->Stop();
		this->SendWindowMessage( SM_MusicEnded, 2 );
	}
}


void WindowSynchronize::RenderPrimitives()
{
	Window::RenderPrimitives();

	
	float fSongBeat, fBPS;
	float fPositionSeconds = m_soundMusic.GetPositionSeconds();
	
	m_pSong->GetBeatAndBPSFromElapsedTime( fPositionSeconds, fSongBeat, fBPS );


}


void WindowSynchronize::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	//RageLog( "WindowSynchronize::Input()" );


	float fSongBeat, fBPS;
	m_pSong->GetBeatAndBPSFromElapsedTime( m_soundMusic.GetPositionSeconds(), fSongBeat, fBPS );


	///////////////////////////
	// handle pad inputs
	///////////////////////////

	switch( PadI.button )
	{
	case BUTTON_UP:
		m_Player.HandlePlayerStep( fSongBeat, STEP_PAD1_UP, 0.35f );
		return;
	case BUTTON_DOWN:
		m_Player.HandlePlayerStep( fSongBeat, STEP_PAD1_DOWN, 0.35f );
		return;
	case BUTTON_LEFT:
		m_Player.HandlePlayerStep( fSongBeat, STEP_PAD1_LEFT, 0.35f );
		return;
	case BUTTON_RIGHT:
		m_Player.HandlePlayerStep( fSongBeat, STEP_PAD1_RIGHT, 0.35f );
		return;
	case BUTTON_BACK:
		m_Fade.CloseWipingLeft( SM_GoToPrevState );
		return;
	case BUTTON_NEXT:
		m_Fade.CloseWipingLeft( SM_GoToNextState );
		return;

	}

	if( DeviceI.device == DEVICE_KEYBOARD )
	{
		switch( DeviceI.button )
		{
		case DIK_INSERT:
			m_soundMusic.SetPlaybackRate( m_soundMusic.GetPlaybackRate() - 0.01f );
			m_pSong->SetChangedSinceLastSave();
			break;
		case DIK_DELETE:
			m_soundMusic.SetPlaybackRate( m_soundMusic.GetPlaybackRate() + 0.01f );
			m_pSong->SetChangedSinceLastSave();
			break;
		case DIK_HOME:
			m_soundMusic.SetPlaybackRate( m_soundMusic.GetPlaybackRate() - 0.1f );
			m_pSong->SetChangedSinceLastSave();
			break;
		case DIK_END:
			m_soundMusic.SetPlaybackRate( m_soundMusic.GetPlaybackRate() + 0.1f );
			m_pSong->SetChangedSinceLastSave();
			break;
		case DIK_PRIOR:
			m_pSong->SetBeatOffsetInSeconds( m_pSong->GetBeatOffsetInSeconds()-(1/fBPS) );
			m_pSong->SetChangedSinceLastSave();
			break;
		case DIK_NEXT:
			m_pSong->SetBeatOffsetInSeconds( m_pSong->GetBeatOffsetInSeconds()+(1/fBPS) );
			m_pSong->SetChangedSinceLastSave();
			break;
		}
	}
}

void WindowSynchronize::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_StartMusic:
		m_soundMusic.Play();
		m_soundMusic.SetPlaybackRate( 1.0f );
		break;
	case SM_MusicEnded:
		m_Fade.CloseWipingRight( SM_GoToNextState );
		break;

	case SM_GoToPrevState:
		WM->SetNewWindow( new WindowSynchronizeMenu );
		break;
	case SM_GoToNextState:
		WM->SetNewWindow( new WindowSynchronizeMenu );
		break;
	}


}
