#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowSynchronizeMenu.h

 Desc: The main title screen and menu.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowSynchronizeMenu.h"
#include "SongManager.h"
#include "WindowManager.h"
#include "WindowTitleMenu.h"
#include "WindowSynchronize.h"
#include "ScreenDimensions.h"
#include "RageUtil.h"
#include "ThemeManager.h"
#include "WindowPrompt.h"


//
// Defines specific to WindowSynchronizeMenu
//

const float BANNER_X	=	CENTER_X;
const float BANNER_Y	=	CENTER_Y-100;

const float TEXT_BANNER_X	=	BANNER_X;
const float TEXT_BANNER_Y	=	BANNER_Y+80;

const float ARROWS_X[2]	=	{BANNER_X-200, BANNER_X+200};
const float ARROWS_Y[2]	=	{BANNER_Y, BANNER_Y};

const float EXPLANATION_X	=	CENTER_X;
const float EXPLANATION_Y	=	CENTER_Y+100;


const WindowMessage SM_GoToPrevState		=	WindowMessage(SM_User+1);
const WindowMessage SM_GoToNextState		=	WindowMessage(SM_User+2);
const WindowMessage SM_ConfirmChange		=	WindowMessage(SM_User+3);


WindowSynchronizeMenu::WindowSynchronizeMenu() : 
	WindowMenu( THEME->GetPathTo(GRAPHIC_SYNCHRONIZE_BACKGROUND), 
				THEME->GetPathTo(GRAPHIC_SYNCHRONIZE_TOP_EDGE),
				ssprintf("%s %s change music    NEXT to continue", CString(char(1)), CString(char(2)) )
				)
{
	RageLog( "WindowSynchronizeMenu::WindowSynchronizeMenu()" );


	// init songs array
	m_pSongs.Copy( SONGS->m_pSongs );
	SortSongPointerArrayByTitle( m_pSongs );
	m_iSelection = 0;


	// select the current song
	for( int i=0; i<m_pSongs.GetSize(); i++ )
	{
		if( m_pSongs[i] == SONGS->m_pCurSong )
		{
			m_iSelection = i;
			break;
		}
	}

	ReloadBanners();



	m_Banner.SetXY( BANNER_X, BANNER_Y );
	this->AddActor( &m_Banner );

	m_TextBanner.SetXY( TEXT_BANNER_X, TEXT_BANNER_Y );
	this->AddActor( &m_TextBanner );

	m_sprArrowLeft.Load( THEME->GetPathTo(GRAPHIC_ARROWS_LEFT) );
	m_sprArrowLeft.SetXY( ARROWS_X[0], ARROWS_Y[0] );
	this->AddActor( &m_sprArrowLeft );

	m_sprArrowRight.Load( THEME->GetPathTo(GRAPHIC_ARROWS_RIGHT) );
	m_sprArrowRight.SetXY( ARROWS_X[1], ARROWS_Y[1] );
	this->AddActor( &m_sprArrowRight );


	m_textExplanation.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textExplanation.SetXY( EXPLANATION_X, EXPLANATION_Y );
	m_textExplanation.SetText( ssprintf("This mode will allow you to match\nthe music with the steps.") );
	m_textExplanation.SetZoom( 0.7f );
	this->AddActor( &m_textExplanation );

	m_Fade.SetOpened();
	this->AddActor( &m_Fade);


	m_soundChangeMusic.Load( THEME->GetPathTo(SOUND_SWITCH_MUSIC) );
	m_soundSelect.Load( THEME->GetPathTo(SOUND_SELECT) );

	MUSIC->Load( THEME->GetPathTo(SOUND_TRAINING_MUSIC) );
	MUSIC->Play( true );


	if( SONGS->m_pCurSong != NULL  &&  SONGS->m_pCurSong->HasChangedSinceLastSave() )
	{
		this->SendWindowMessage( SM_ConfirmChange, 0.1f );
	}
}


WindowSynchronizeMenu::~WindowSynchronizeMenu()
{
	RageLog( "WindowSynchronizeMenu::~WindowSynchronizeMenu()" );
}


void WindowSynchronizeMenu::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	RageLog( "WindowSynchronizeMenu::Input()" );

	
	Window::Input( DeviceI, PadI, PlayerI );
}

void WindowSynchronizeMenu::HandleWindowMessage( WindowMessage SM )
{
	static bool bAnswer = false;

	switch( SM )
	{
	case SM_ConfirmChange:
		bAnswer = true;
		WM->AddWindowToTop( 
			new WindowPrompt(
				ssprintf( 
					"You have changed the offset of\n\n"
					"'%s'.\n\n"
					"Would you like to save changes\n"
					"to song file(s)?", 
					SONGS->m_pCurSong->GetTitle() 
				),
				PROMPT_YES_NO,
				&bAnswer
			)
		);
		break;
	case SM_RegainingInputFocus:
		if( bAnswer == true )
		{
			SONGS->m_pCurSong->SaveOffsetChangeToDisk();
		}
		break;

	
	
	case SM_GoToPrevState:
		WM->SetNewWindow( new WindowTitleMenu );
		break;
	case SM_GoToNextState:
		WM->SetNewWindow( new WindowSynchronize );
		break;
	}
}

void WindowSynchronizeMenu::ReloadBanners()
{
	m_Banner.LoadFromSong( m_pSongs[m_iSelection] );
	m_TextBanner.LoadFromSong( m_pSongs[m_iSelection] );
}

void WindowSynchronizeMenu::Up()
{
}

void WindowSynchronizeMenu::Down()
{
}

void WindowSynchronizeMenu::Left()
{
	m_iSelection--;
	if( m_iSelection < 0 )
		m_iSelection = m_pSongs.GetSize()-1;

	m_soundChangeMusic.PlayRandom();

	ReloadBanners();
}

void WindowSynchronizeMenu::Right()
{
	m_iSelection++;
	if( m_iSelection > m_pSongs.GetSize()-1 )
		m_iSelection = 0;

	m_soundChangeMusic.PlayRandom();

	ReloadBanners();
}

void WindowSynchronizeMenu::Next()
{
	WindowMenu::Next();

	MUSIC->Stop();

	SONGS->m_pCurSong = m_pSongs[m_iSelection];

	m_soundSelect.PlayRandom();
	WindowMenu::TweenOffScreen();
	m_Fade.CloseWipingRight( SM_GoToNextState );
}

void WindowSynchronizeMenu::Back()
{	
	WindowMenu::Back();


	MUSIC->Stop();

	WindowMenu::TweenOffScreen();
	m_Fade.CloseWipingLeft( SM_GoToPrevState );
}
