#include "stdafx.h"
/*
-----------------------------------------------------------------------------
 File: WindowTitleMenu.h

 Desc: The main title screen and menu.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "WindowTitleMenu.h"
#include "WindowManager.h"
#include "WindowCaution.h"
#include "WindowConfigurePads.h"
#include "WindowGameOptions.h"
#include "WindowIntroCovers.h"
#include "WindowSynchronizeMenu.h"
#include "WindowEdit.h"
#include "ScreenDimensions.h"
#include "RageUtil.h"
#include "StepMania.h"
#include "ThemeManager.h"
#include "WindowEditMenu.h"
#include "WindowMenuSelectStyle.h"


//
// Defines specific to WindowTitleMenu
//

const CString CHOICE_TEXT[WindowTitleMenu::NUM_TITLE_MENU_CHOICES] = {
	"GAME MODE",
	"CONFIGURE PADS",
	"GAME OPTIONS",
	"SYNCHRONIZE",
	"TRAINING",
	"EDIT/RECORD",
	"HELP",
	"CHECK FOR UPDATE",
	"EXIT",
};

const float CHOICES_START_Y		= 70;
const float CHOICES_GAP_Y		= 34;

const float HELP_X				= CENTER_X;
const float HELP_Y				= SCREEN_HEIGHT-55;


const WindowMessage SM_TimeToFadeOut		=	WindowMessage(SM_User+1);
const WindowMessage SM_GoToCaution			=	WindowMessage(SM_User+2);
const WindowMessage SM_GoToSelectStyle		=	WindowMessage(SM_User+3);
const WindowMessage SM_GoToIntroCovers		=	WindowMessage(SM_User+4);
const WindowMessage SM_GoToConfigurePads	=	WindowMessage(SM_User+5);
const WindowMessage SM_GoToGameOptions		=	WindowMessage(SM_User+6);
const WindowMessage SM_GoToSynchronize		=	WindowMessage(SM_User+7);
const WindowMessage SM_GoToEdit				=	WindowMessage(SM_User+8);
const WindowMessage SM_DoneOpening			=	WindowMessage(SM_User+9);

WindowTitleMenu::WindowTitleMenu()
{
	RageLog( "WindowTitleMenu::WindowTitleMenu()" );

	m_sprBG.Load( THEME->GetPathTo(GRAPHIC_TITLE_MENU_BACKGROUND) );
	m_sprBG.StretchTo( CRect(0,0,SCREEN_WIDTH,SCREEN_HEIGHT) );
	m_sprBG.SetDiffuseColor( D3DXCOLOR(0.6f,0.6f,0.6f,1) );
	this->AddActor( &m_sprBG );

	
	m_textVersion.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textVersion.SetText( "v1.64" );
	m_textVersion.SetDiffuseColorTopEdge( D3DXCOLOR(0.6f,0.6f,0.6f,1) );	// light gray
	m_textVersion.SetDiffuseColorBottomEdge( D3DXCOLOR(0.4f,0.4f,0.4f,1) );	// medium gray
	//m_textVersion.SetEffectGlowing( 1.0f, D3DXCOLOR(1,1,1,0), D3DXCOLOR(1,1,1,0.7f) );
	m_textVersion.SetXY( SCREEN_WIDTH-60, SCREEN_HEIGHT-30 );
//	m_textVersion.SetRotation();
	m_textVersion.SetZoom( 0.5f );
	this->AddActor( &m_textVersion );


	m_rectHelpFrame.StretchTo( CRect(0+100, (int)HELP_Y-32, (int)SCREEN_WIDTH-100, (int)HELP_Y+32) );
	m_rectHelpFrame.SetDiffuseColor( D3DXCOLOR(0,0,0,0.3f) );
	this->AddActor( &m_rectHelpFrame );


	m_textHelp.Load( THEME->GetPathTo(FONT_NORMAL) );
	m_textHelp.SetXY( HELP_X, HELP_Y );
	m_textHelp.SetText( ssprintf("Use %s %s to select, then press NEXT", CString(char(3)), CString(char(4)) ) );
	m_textHelp.SetZoom( 0.7f );
	//m_textHelp.SetDiffuseColor( D3DXCOLOR(1,1,1,1.5f) );
	m_textHelp.SetEffectBlinking();
	this->AddActor( &m_textHelp );


	for( int i=0; i< NUM_TITLE_MENU_CHOICES; i++ )
	{
		m_textChoice[i].Load( THEME->GetPathTo(FONT_OUTLINE) );
		m_textChoice[i].SetText( CHOICE_TEXT[i] );
		m_textChoice[i].SetXY( CENTER_X, CHOICES_START_Y + i*CHOICES_GAP_Y );
		this->AddActor( &m_textChoice[i] );
	}	
	
	m_Fade.SetClosed();
	m_Fade.OpenWipingRight( SM_DoneOpening );
	this->AddActor( &m_Fade );


	
	m_soundTitle.Load( THEME->GetPathTo(ANNOUNCER_TITLE) );
	m_soundChange.Load( THEME->GetPathTo(SOUND_TITLE_CHANGE) );	
	m_soundSelect.Load( THEME->GetPathTo(SOUND_SELECT) );
	m_soundInvalid.Load( THEME->GetPathTo(SOUND_INVALID) );


	m_TitleMenuChoice = CHOICE_GAME_MODE;
	GainFocus( m_TitleMenuChoice );

/*
	MUSIC->Stop();
	
	// find a random song and play it
	if( GAMEINFO->m_pSongs.GetSize() > 0 )
	{
		for( i=0; i<50; i++ )	// try 50 times to find a song with music
		{
			int iRandomSongIndex = rand() % GAMEINFO->m_pSongs.GetSize();
			Song* pSong = GAMEINFO->m_pSongs[iRandomSongIndex];
			if( pSong->HasMusic() )
			{
				MUSIC->Load( pSong->GetMusicPath() );
				MUSIC->Play();
				break;
			}
		}
	}
*/

	//this->SendWindowMessage( SM_TimeToFadeOut, 30.0 );
}


WindowTitleMenu::~WindowTitleMenu()
{
	RageLog( "WindowTitleMenu::~WindowTitleMenu()" );
}


void WindowTitleMenu::Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI )
{
	RageLog( "WindowTitleMenu::Input()" );

	if( m_Fade.IsClosing() )
		return;
	
	Window::Input( DeviceI, PadI, PlayerI );
}

void WindowTitleMenu::HandleWindowMessage( WindowMessage SM )
{
	switch( SM )
	{
	case SM_DoneOpening:
		if( PREFS->m_GameOptions.m_bAnnouncer )
			m_soundTitle.PlayRandom();
		break;
	case SM_TimeToFadeOut:
		m_Fade.CloseWipingRight( SM_GoToIntroCovers );
		break;
	case SM_GoToCaution:
		WM->SetNewWindow( new WindowCaution );
		break;
	case SM_GoToSelectStyle:
		WM->SetNewWindow( new WindowMenuSelectStyle );
		break;
	case SM_GoToIntroCovers:
		WM->SetNewWindow( new WindowIntroCovers );
		break;
	case SM_GoToConfigurePads:
		WM->SetNewWindow( new WindowConfigurePads );
		break;
	case SM_GoToGameOptions:
		WM->SetNewWindow( new WindowGameOptions );
		break;
	case SM_GoToSynchronize:
		WM->SetNewWindow( new WindowSynchronizeMenu );
		break;
	case SM_GoToEdit:
		WM->SetNewWindow( new WindowEditMenu );
		break;
	}
}


void WindowTitleMenu::LoseFocus( int iChoiceIndex )
{
	m_soundChange.PlayRandom();

	m_textChoice[iChoiceIndex].SetEffectNone();
	m_textChoice[iChoiceIndex].SetAddColor( D3DXCOLOR(0,0,0,0) );
	m_textChoice[iChoiceIndex].SetDiffuseColor( D3DXCOLOR(1,1,1,1) );
	m_textChoice[iChoiceIndex].BeginTweening( 0.3f );
	m_textChoice[iChoiceIndex].SetTweenZoom( 1 );

}

void WindowTitleMenu::GainFocus( int iChoiceIndex )
{
	m_textChoice[iChoiceIndex].SetDiffuseColor( D3DXCOLOR(0.5f,1,0.5f,1) );
	m_textChoice[iChoiceIndex].BeginTweening( 0.3f );
	m_textChoice[iChoiceIndex].SetTweenZoom( 1.5f );
	m_textChoice[iChoiceIndex].SetEffectCamelion( 2.5f, D3DXCOLOR(0.5f,1,0.5f,1), D3DXCOLOR(0.3f,0.6f,0.3f,1) );
}

void WindowTitleMenu::Up()
{
	LoseFocus( m_TitleMenuChoice );

	if( m_TitleMenuChoice == 0 ) // wrap around
		m_TitleMenuChoice = (WindowTitleMenu::TitleMenuChoice)((int)NUM_TITLE_MENU_CHOICES); 
	
	m_TitleMenuChoice = TitleMenuChoice( m_TitleMenuChoice-1 );

	GainFocus( m_TitleMenuChoice );
}


void WindowTitleMenu::Down()
{
	LoseFocus( m_TitleMenuChoice );

	if( m_TitleMenuChoice == (int)WindowTitleMenu::NUM_TITLE_MENU_CHOICES-1 ) 
		m_TitleMenuChoice = (TitleMenuChoice)-1; // wrap around

	m_TitleMenuChoice = TitleMenuChoice( m_TitleMenuChoice+1 );

	GainFocus( m_TitleMenuChoice );
}


void WindowTitleMenu::Next()
{	

	switch( m_TitleMenuChoice )
	{
	case CHOICE_GAME_MODE:
		m_soundSelect.PlayRandom();
		if( PREFS->m_GameOptions.m_bSkipCaution )
			m_Fade.CloseWipingRight( SM_GoToSelectStyle );
		else
			m_Fade.CloseWipingRight( SM_GoToCaution );
		return;
	case CHOICE_CONFIGURE_PADS:
		m_soundSelect.PlayRandom();
		m_Fade.CloseWipingRight( SM_GoToConfigurePads );
		return;
	case CHOICE_GAME_OPTIONS:
		m_soundSelect.PlayRandom();
		m_Fade.CloseWipingRight( SM_GoToGameOptions );
		return;
	case CHOICE_SYNCHRONIZE:
		m_soundSelect.PlayRandom();
		m_Fade.CloseWipingRight( SM_GoToSynchronize );
		return;
	case CHOICE_TRAINING:
		m_soundInvalid.PlayRandom();
		return;
	case CHOICE_EDIT:
#ifdef DEBUG
		m_soundSelect.PlayRandom();
		m_Fade.CloseWipingRight( SM_GoToEdit );
#else
		m_soundInvalid.PlayRandom();
#endif
		return;
	case CHOICE_HELP:
		m_soundSelect.PlayRandom();
		PREFS->m_GameOptions.m_bWindowed = false;
		SwitchDisplayMode();// true, SCREEN->GetWidth(), SCREEN->GetHeight(), SCREEN->GetBPP() );
		GotoURL( "Docs/index.htm" );
		return;
	case CHOICE_CHECK_FOR_UPDATE:
		m_soundSelect.PlayRandom();
		PREFS->m_GameOptions.m_bWindowed = false;
		SwitchDisplayMode();// true, SCREEN->GetWidth(), SCREEN->GetHeight(), SCREEN->GetBPP() );
		GotoURL( "http://www.stepmania.com" );
		return;
	case CHOICE_EXIT:
		m_soundSelect.PlayRandom();
		PostQuitMessage(0);
		return;
	}
}

void WindowTitleMenu::Back()
{	
	//m_Fade.CloseWipingLeft( SM_GoToIntroCovers );
}

