/*
-----------------------------------------------------------------------------
 File: WindowTitleMenu.h

 Desc: The main title screen and menu.

 Copyright (c) 2001 Chris Danford.  All rights reserved.
-----------------------------------------------------------------------------
*/

#include "Window.h"
#include "Sprite.h"
#include "ColorArrow.h"
#include "BitmapText.h"
#include "TransitionFadeWipeWithLogo.h"
#include "TransitionFade.h"
#include "RandomSample.h"
#include "RandomStream.h"



class WindowTitleMenu : public Window
{
public:
	WindowTitleMenu();
	virtual ~WindowTitleMenu();

	virtual void Input( DeviceInput& DeviceI, PadInput &PadI, PlayerInput &PlayerI );
	virtual void HandleWindowMessage( WindowMessage SM );

	enum TitleMenuChoice {
		CHOICE_GAME_MODE = 0,
		CHOICE_CONFIGURE_PADS,
		CHOICE_GAME_OPTIONS,
		CHOICE_SYNCHRONIZE,
		CHOICE_TRAINING,
		CHOICE_EDIT,
		CHOICE_HELP,
		CHOICE_CHECK_FOR_UPDATE,
		CHOICE_EXIT,
		NUM_TITLE_MENU_CHOICES	// leave this at the end!
	};

private:
	void GainFocus( int iChoiceIndex );
	void LoseFocus( int iChoiceIndex );
	void Up();
	void Down();
	void Back();
	void Next();

	TitleMenuChoice m_TitleMenuChoice;

	Sprite			m_sprBG;
	BitmapText		m_textVersion;
	RectangleActor	m_rectHelpFrame;
	BitmapText		m_textHelp;
	BitmapText		m_textChoice[NUM_TITLE_MENU_CHOICES];

	TransitionFade	m_Fade;

	RandomStream		m_soundTitle;
	RandomSample		m_soundChange;
	RandomSample		m_soundSelect;
	RandomSample		m_soundInvalid;
};



